@@ -1,5 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2006-2013,2018 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,12 +15,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import logging
+
 from ..core import exceptions
 from ..target import TARGET
 from ..target.pack import pack_target
 from ..utility.graph import GraphNode
-import logging
-import six
 
 LOG = logging.getLogger(__name__)
 
@@ -30,6 +31,10 @@ class Board(GraphNode):
     def __init__(self, session, target=None):
         super(Board, self).__init__()
         
+        # Use the session option if no target type was given to us.
+        if target is None:
+            target = session.options.get('target_override')
+        
         # As a last resort, default the target to 'cortex_m'.
         if target is None:
             target = 'cortex_m'
@@ -42,8 +47,17 @@ class Board(GraphNode):
                             "'target_override' option. Use 'pyocd list --targets' to see available "
                             "targets types.")
 
+        assert target is not None
+        
+        # Convert dashes to underscores in the target type, and convert to lower case.
+        target = target.replace('-', '_').lower()
+        
+        # Write the effective target type back to options if it's different.
+        if target != session.options.get('target_override'):
+            session.options['target_override'] = target
+
         self._session = session
-        self._target_type = target.lower()
+        self._target_type = target
         self._test_binary = session.options.get('test_binary')
         self._delegate = None
         self._inited = False
@@ -60,11 +74,11 @@ class Board(GraphNode):
         try:
             self.target = TARGET[self._target_type](session)
         except KeyError as exc:
-            six.raise_from(exceptions.TargetSupportError(
-                "Target type '%s' not recognized. Use 'pyocd list --targets' to see currently "
+            raise exceptions.TargetSupportError(
+                f"Target type {self._target_type} not recognized. Use 'pyocd list --targets' to see currently "
                 "available target types. "
-                "See <https://github.com/mbedmicro/pyOCD/blob/master/docs/target_support.md> "
-                "for how to install additional target support." % self._target_type), exc)
+                "See <https://github.com/pyocd/pyOCD/blob/master/docs/target_support.md> "
+                "for how to install additional target support.") from exc
         
         # Tell the user what target type is selected.
         LOG.info("Target type is %s", self._target_type)
@@ -97,8 +111,8 @@ class Board(GraphNode):
                 resume = self.session.options.get('resume_on_disconnect')
                 self.target.disconnect(resume)
                 self._inited = False
-            except:
-                LOG.error("link exception during target disconnect:", exc_info=self._session.log_tracebacks)
+            except exceptions.Error as err:
+                LOG.error("link exception during target disconnect: %s", err, exc_info=self._session.log_tracebacks)
 
     @property
     def session(self):
