@@ -1,5 +1,6 @@
 # pyOCD debugger
-# Copyright (c) 2017 Arm Limited
+# Copyright (c) 2017-2020 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -61,6 +62,7 @@ BOARD_ID_TO_INFO = {
     "0238": BoardInfo(  "MIMXRT1024-EVK",       "mimxrt1024",       "evkmimxrt1024.bin",    ),
     "0239": BoardInfo(  "FRDM-K32L2B3",         "k32l2b3",          "l1_frdm_k32l2b3.bin",         ),
     "0240": BoardInfo(  "FRDM-K64F",            "k64f",             "l1_k64f.bin",          ),
+    "0244": BoardInfo(  "MIMXRT1170-EVK",       "mimxrt1170_cm7",   "l1_rt1170.bin",          ),
     "0245": BoardInfo(  "IBMEthernetKit",       "k64f",             "l1_k64f.bin"           ),
     "0250": BoardInfo(  "FRDM-KW24D512",        "kw24d5",           "l1_kw24d5.bin"         ),
     "0251": BoardInfo(  "FRDM-KW36",            "kw36z4",           "l1_kw36z.bin",         ),
@@ -84,7 +86,7 @@ BOARD_ID_TO_INFO = {
     "0330": BoardInfo(  "TWR-KV58F220M",        "mkv58f512vll24",   None,                   ),
     "0340": BoardInfo(  "TWR-K80F150M",         "mk80fn256vll15",   None,                   ),
     "0341": BoardInfo(  "FRDM-KV31F",           "mkv31f512vll12",   None,                   ),
-    "0350": BoardInfo(  "XDOT_L151CC",          "stm32l151cc",      None                    ),
+    "0350": BoardInfo(  "XDOT_L151CC",          "stm32l151cctx",    None                    ),
     "0400": BoardInfo(  "MAXWSNENV",            "max32600",         "l1_maxwsnenv.bin",     ),
     "0405": BoardInfo(  "MAX32600MBED",         "max32600",         "l1_max32600mbed.bin",  ),
     "0406": BoardInfo(  "MAX32620MBED",         "max32620",         None                    ),
@@ -101,12 +103,14 @@ BOARD_ID_TO_INFO = {
     "0462": BoardInfo(  "MTB USI WM-BN-BM-22",  "stm32f412xg",      "mtb_usi_wm-bn-bm-22.bin",),
     "0602": BoardInfo(  "EV_COG_AD3029LZ",      "aducm3029",        None                    ),
     "0603": BoardInfo(  "EV_COG_AD4050LZ",      "aducm4050",        None                    ),
+    "0604": BoardInfo(  "SDK-K1",               "stm32f469nihx",    None,                   ),
     "0700": BoardInfo(  "NUCLEO-F103RB",        "stm32f103rb",      "ST-Nucleo-F103RB.bin", ),
     "0705": BoardInfo(  "NUCLEO-F302R8",        "stm32f302r8tx",    None,                   ),
     "0710": BoardInfo(  "NUCLEO-L152RE",        "stm32l152re",      "NUCLEO_L152RE.bin",    ),
     "0715": BoardInfo(  "NUCLEO-L053R8",        "stm32l053r8tx",    "NUCLEO_L053R8.bin",    ),
     "0720": BoardInfo(  "NUCLEO-F401RE",        "stm32f401retx",    None,                   ),
     "0725": BoardInfo(  "NUCLEO-F030R8",        "stm32f030r8tx",    None,                   ),
+    "0729": BoardInfo(  "NUCLEO-G071RB",        "stm32g071rbtx",    None,                   ),
     "0730": BoardInfo(  "NUCLEO-F072RB",        "stm32f072rbtx",    "NUCLEO_F072RB.bin",    ),
     "0735": BoardInfo(  "NUCLEO-F334R8",        "stm32f334r8tx",    "NUCLEO_F334R8.bin",    ),
     "0740": BoardInfo(  "NUCLEO-F411RE",        "stm32f411retx",    "NUCLEO_F411RE.bin",    ),
@@ -121,7 +125,7 @@ BOARD_ID_TO_INFO = {
     "0760": BoardInfo(  "NUCLEO-L073RZ",        "stm32l073rztx",    None,                   ),
     "0764": BoardInfo(  "DISCO-L475VG-IOT01A",  "stm32l475xg",      "stm32l475vg_iot01a.bin",),
     "0765": BoardInfo(  "NUCLEO-L476RG",        "stm32l476rgtx",    "NUCLEO_L476RG.bin",    ),
-    "0770": BoardInfo(  "NUCLEO-L432KC",        "stm32l432kcux",    None,                   ),
+    "0770": BoardInfo(  "NUCLEO-L432KC",        "stm32l432kcux",    "NUCLEO_L432KC.bin",    ),
     "0774": BoardInfo(  "DISCO-L4R9I",          "stm32l4r9aiix",    None,                   ),
     "0775": BoardInfo(  "NUCLEO-F303K8",        "stm32f303k8tx",    None,                   ),
     "0776": BoardInfo(  "NUCLEO-L4R5ZI",        "stm32l4r5zitx",    None,                   ),
@@ -130,14 +134,14 @@ BOARD_ID_TO_INFO = {
     "0779": BoardInfo(  "NUCLEO-L433RC-P",      "stm32l433rctx",    None,                   ),
     "0780": BoardInfo(  "NUCLEO-L011K4",        "stm32l011k4tx",    None,                   ),
     "0781": BoardInfo(  "NUCLEO-L4R5ZI-P",      "stm32l4r5zitx",    None,                   ),
+    "0783": BoardInfo(  "NUCLEO-L010RB",        "stm32l010rbtx",    None,                   ),
     "0785": BoardInfo(  "NUCLEO-F042K6",        "stm32f042k6tx",    None,                   ),
     "0788": BoardInfo(  "DISCO-F469NI",         "stm32f469nihx",    None,                   ),
     "0790": BoardInfo(  "NUCLEO-L031K6",        "stm32l031x6",      None,                   ),
     "0791": BoardInfo(  "NUCLEO-F031K6",        "stm32f031k6tx",    None,                   ),
-    "0795": BoardInfo(  "DISCO-F429ZI",         "stm32f429xi",      None,                   ),
+    "0795": BoardInfo(  "DISCO-F429ZI",         "stm32f429zitx",    None,                   ),
     "0796": BoardInfo(  "NUCLEO-F429ZI",        "stm32f429xi",      "nucleo_f429zi.bin",    ),
-    "0797": BoardInfo(  "NUCLEO-F439ZI",        "stm32f439xi",      None,                   ),
-    "0799": BoardInfo(  "NUCLEO-L073RZ",        "stm32l073rztx",    None,                   ),
+    "0797": BoardInfo(  "NUCLEO-F439ZI",        "stm32f439zitx",    None,                   ),
     "0805": BoardInfo(  "DISCO-L053C8",         "stm32l053c8tx",    None,                   ),
     "0810": BoardInfo(  "DISCO-F334C8",         "stm32f334c8tx",    None,                   ),
     "0812": BoardInfo(  "NUCLEO-F722ZE",        "stm32f722zetx",    None,                   ),
@@ -152,6 +156,7 @@ BOARD_ID_TO_INFO = {
     "0822": BoardInfo(  "DISCO-L496AG",         "stm32l496agix",    None,                   ),
     "0823": BoardInfo(  "NUCLEO-L496ZG",        "stm32l496zgtx",    None,                   ),
     "0824": BoardInfo(  "LPCXpresso824-MAX",    "lpc824",           "l1_lpc824.bin",        ),
+    "0825": BoardInfo(  "DISCO-F412ZG",         "stm32f412xg",      "nucleo_f412zg.bin",    ),
     "0826": BoardInfo(  "NUCLEO-F412ZG",        "stm32f412xg",      "nucleo_f412zg.bin",    ),
     "0827": BoardInfo(  "NUCLEO-L486RG",        "stm32l486rgtx",    None,                   ),
     "0828": BoardInfo(  "NUCLEO-L496ZG-P",      "stm32l496zgtx",    None,                   ),
@@ -162,9 +167,26 @@ BOARD_ID_TO_INFO = {
     "0836": BoardInfo(  "NUCLEO-H743ZI2",       "stm32h743zitx",    None,                   ),
     "0839": BoardInfo(  "NUCLEO-WB55RG",        "stm32wb55rgvx",    None,                   ),
     "0840": BoardInfo(  "B96B-F446VE",          "stm32f446vetx",    None,                   ),
+    "0841": BoardInfo(  "NUCLEO-G474RE",        "stm32g474retx",    None,                   ),
+    "0842": BoardInfo(  "NUCLEO-H753ZI",        "stm32h753zitx",    None,                   ),
+    "0843": BoardInfo(  "NUCLEO-H745ZI-Q",      "stm32h745zitx",    None,                   ),
+    "0847": BoardInfo(  "DISCO-H745I",          "stm32h745zihx",    None,                   ),
+    "0849": BoardInfo(  "NUCLEO-G070RB",        "stm32g070rbtx",    None,                   ),
+    "0850": BoardInfo(  "NUCLEO-G431RB",        "stm32g431rbtx",    None,                   ),
+    "0851": BoardInfo(  "NUCLEO-G431KB",        "stm32g431kbtx",    None,                   ),
+    "0852": BoardInfo(  "NUCLEO-G031K8",        "stm32g031K8tx",    None,                   ),
+    "0853": BoardInfo(  "NUCLEO-F301K8",        "stm32f301k8tx",    None,                   ),
     "0854": BoardInfo(  "NUCLEO-L552ZE-Q",      "stm32l552zetxq",   None,                   ),
     "0855": BoardInfo(  "DISCO-L562QE",         "stm32l562qeixq",   None,                   ),
+    "0860": BoardInfo(  "NUCLEO-H7A3ZI-Q",      "stm32h7a3zitxq",   None,                   ),
+    "0866": BoardInfo(  "NUCLEO-WL55JC",        "stm32wl55jcix",    None,                   ),
     "0879": BoardInfo(  "NUCLEO-F756ZG",        "stm32f756zgtx",    None,                   ),
+    "0882": BoardInfo(  "NUCLEO-G491RE",        "stm32g491retx",    None,                   ),
+    "0883": BoardInfo(  "NUCLEO-WB15CC",        "stm32wb15ccux",    None,                   ),
+    "0884": BoardInfo(  "DISCO-WB5MMG",         "stm32wb5mmghx",    None,                   ),
+    "0885": BoardInfo(  "B-L4S5I-IOT01A",       "stm32l4s5vitx",    None,                   ),
+    "0886": BoardInfo(  "NUCLEO-U575ZI-Q",      "stm32u575zitx",    None,                   ),
+    "0887": BoardInfo(  "B-U585I-IOT02A",       "stm32u585aiix",    None,                   ),
     "1010": BoardInfo(  "mbed NXP LPC1768",     "lpc1768",          "l1_lpc1768.bin",       ),
     "1017": BoardInfo(  "mbed HRM1017",         "nrf51",            "l1_nrf51.bin",         ),
     "1018": BoardInfo(  "Switch-Science-mbed-LPC824", "lpc824",     "l1_lpc824.bin",        ),
@@ -194,6 +216,7 @@ BOARD_ID_TO_INFO = {
     "1304": BoardInfo(  "NuMaker-PFM-M487",     "m487jidae",        None,                   ),
     "1309": BoardInfo(  "NuMaker-M252KG",       "m252kg6ae",        None,                   ),
     "1310": BoardInfo(  "NuMaker-IoT-M263A",    "m263kiaae",        None,                   ),
+    "1312": BoardInfo(  "NuMaker-M2354",        "m2354kjfae",       None,                   ),
     "1549": BoardInfo(  "LPC1549",              "lpc1549jbd100",    None,                   ),
     "1600": BoardInfo(  "Bambino 210",          "lpc4330",          "l1_lpc4330.bin",       ),
     "1605": BoardInfo(  "Bambino 210E",         "lpc4330",          "l1_lpc4330.bin",       ),
@@ -216,20 +239,22 @@ BOARD_ID_TO_INFO = {
     "1910": BoardInfo(  "CY8CKIT-064B0S2-4343W","cy8c64xa_cm4",     "l1_cy8c6xxa.bin",      ),
     "1911": BoardInfo(  "CY8CKIT-064S0S2-4343W","cy8c64xa_cm4",     "l1_cy8c6xxa.bin",      ),
     "1912": BoardInfo(  "CYSBSYSKIT-01",        "cy8c6xxa",         "l1_cy8c6xxa.bin",      ),
-    "2201": BoardInfo(  "WIZwik_W7500",         "w7500",            "l1_w7500mbed.bin",     ),
+    "2201": BoardInfo(  "WIZwiki_W7500",        "w7500",            "l1_w7500mbed.bin",     ),
+    "2203": BoardInfo(  "WIZwiki_W7500P",       "w7500",            "l1_w7500mbed.bin",     ),
     "2600": BoardInfo(  "ep_agora",             "nrf52840",         None,                   ),
     "3300": BoardInfo(  "CC3220SF_LaunchXL",    "cc3220sf",         "l1_cc3220sf.bin",      ),
     "3701": BoardInfo(  "Samsung_S5JS100",      "s5js100",          "s5js100.bin",          ),
+    "4100": BoardInfo(  "NAMote72",             "stm32l152rctx",    None,                   ),
     "4337": BoardInfo(  "LPC4337",              "lpc4337",          None,                   ),
     "4600": BoardInfo(  "Realtek RTL8195AM",    "rtl8195am",        "l1_rtl8195am.bin",     ),
     "5002": BoardInfo(  "Arm V2M-MPS3",         "cortex_m",         None,                   ),
     "5005": BoardInfo(  "Arm V2M-MPS3",         "cortex_m",         None,                   ),
     "5006": BoardInfo(  "Arm Musca-A1",         "musca_a1",         "l1_musca_a1.bin",      ),
     "5007": BoardInfo(  "Arm Musca-B1",         "musca_b1",         "l1_musca_b1.bin",      ),
-    "5009": BoardInfo(  "Arm Musca-S1",         "musca-s1",         None,                   ),
+    "5009": BoardInfo(  "Arm Musca-S1",         "musca_s1",         None,                   ),
     "7402": BoardInfo(  "mbed 6LoWPAN Border Router HAT", "k64f",   "l1_k64f.bin",          ),
     "7778": BoardInfo(  "Teensy 3.1",           "mk20dx256vlh7",    None,                   ),
-    "8080": BoardInfo(  "L-Tek FF1705",         "stm32l151cc",      None,                   ),
+    "8080": BoardInfo(  "L-Tek FF1705",         "stm32l151cctx",    None,                   ),
     "8081": BoardInfo(  "L-Tek FF-LPC546XX",    "lpc54606",         None,                   ),
     "9004": BoardInfo(  "Arch Pro",             "lpc1768",          "l1_lpc1768.bin",       ),
     "9009": BoardInfo(  "Arch BLE",             "nrf51",            "l1_nrf51.bin",         ),
@@ -237,6 +262,8 @@ BOARD_ID_TO_INFO = {
     "9014": BoardInfo(  "Seeed 96Boards Nitrogen", "nrf52",         "l1_nrf52-dk.bin",      ),
     "9900": BoardInfo(  "micro:bit",            "nrf51",            "l1_microbit.bin",      ),
     "9901": BoardInfo(  "micro:bit",            "nrf51",            "l1_microbit.bin",      ),
+    "9903": BoardInfo(  "micro:bit v2",         "nrf52833",         "microbitv2.bin",       ),
+    "9904": BoardInfo(  "micro:bit v2",         "nrf52833",         "microbitv2.bin",       ),
     "C004": BoardInfo(  "tinyK20",              "k20d50m",          "l1_k20d50m.bin",       ),
     "C006": BoardInfo(  "VBLUno51",             "nrf51",            "l1_nrf51.bin",         ),
 }
