@@ -1,5 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2015-2020 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,10 +19,12 @@ import logging
 import os
 from natsort import natsort
 import textwrap
+from time import sleep
+from shutil import get_terminal_size
 
 from .. import coresight
 from ..core.helpers import ConnectHelper
-from ..core import (exceptions, session)
+from ..core import exceptions
 from ..probe.tcp_probe_server import DebugProbeServer
 from ..core.target import Target
 from ..flash.loader import FlashLoader
@@ -29,19 +32,18 @@ from ..flash.eraser import FlashEraser
 from ..flash.file_programmer import FileProgrammer
 from ..gdbserver.gdbserver import GDBServer
 from ..utility import conversion
-from ..utility.cmdline import UniquePrefixMatcher
+from ..utility.strings import UniquePrefixMatcher
+from ..utility.cmdline import convert_reset_type
 from ..utility.hex import (
     format_hex_width,
     dump_hex_data_to_str,
     )
 from ..utility.progress import print_progress
-from ..utility.compatibility import get_terminal_size
 from ..utility.columns import ColumnFormatter
 from ..utility.mask import (
     msb,
     bfx,
     bfi,
-    round_up_div,
     )
 from .base import CommandBase
 
@@ -114,9 +116,9 @@ class StatusCommand(CommandBase):
 
 class RegisterCommandBase(CommandBase):
     def dump_register_group(self, group_name):
-        regs = natsort(self.context.target.core_registers.iter_matching(
+        regs = natsort(self.context.selected_core.core_registers.iter_matching(
                 lambda r: r.group == group_name), key=lambda r: r.name)
-        reg_values = self.context.target.read_core_registers_raw(r.name for r in regs)
+        reg_values = self.context.selected_core.read_core_registers_raw(r.name for r in regs)
         
         col_printer = ColumnFormatter()
         for info, value in zip(regs, reg_values):
@@ -126,11 +128,11 @@ class RegisterCommandBase(CommandBase):
         col_printer.write()
 
     def dump_registers(self, show_all=False, show_group=None):
-        if not self.context.target.is_halted():
+        if not self.context.selected_core.is_halted():
             self.context.write("Core is not halted; cannot read core registers")
             return
 
-        all_groups = sorted(self.context.target.core_registers.groups)
+        all_groups = sorted(self.context.selected_core.core_registers.groups)
         if show_all:
             groups_to_show = all_groups
         elif show_group:
@@ -147,7 +149,7 @@ class RegisterCommandBase(CommandBase):
     def _dump_peripheral_register(self, periph, reg, show_fields):
         size = reg.size or 32
         addr = periph.base_address + reg.address_offset
-        value = self.context.target.read_memory(addr, size)
+        value = self.context.selected_ap.read_memory(addr, size)
         value_str = format_hex_width(value, size)
         self.context.writei("%s.%s @ %08x = %s", periph.name, reg.name, addr, value_str)
 
@@ -217,19 +219,19 @@ class RegCommand(RegisterCommandBase):
             return
         
         # Check register names first.
-        if self.reg in self.context.target.core_registers.by_name:
-            if not self.context.target.is_halted():
+        if self.reg in self.context.selected_core.core_registers.by_name:
+            if not self.context.selected_core.is_halted():
                 self.context.write("Core is not halted; cannot read core registers")
                 return
 
-            info = self.context.target.core_registers.by_name[self.reg]
-            value = self.context.target.read_core_register(self.reg)
+            info = self.context.selected_core.core_registers.by_name[self.reg]
+            value = self.context.selected_core.read_core_register(self.reg)
             value_str = self._format_core_register(info, value)
             self.context.writei("%s = %s", self.reg, value_str)
             return
         
         # Now look for matching group name.
-        matcher = UniquePrefixMatcher(self.context.target.core_registers.groups)
+        matcher = UniquePrefixMatcher(self.context.selected_core.core_registers.groups)
         group_matches = matcher.find_all(self.reg)
         if len(group_matches) == 1:
             self.dump_registers(show_group=group_matches[0])
@@ -277,8 +279,8 @@ class WriteRegCommand(RegisterCommandBase):
         self.value = args[idx + 1]
 
     def execute(self):
-        if self.reg in self.context.target.core_registers.by_name:
-            if not self.context.target.is_halted():
+        if self.reg in self.context.selected_core.core_registers.by_name:
+            if not self.context.selected_core.is_halted():
                 self.context.write("Core is not halted; cannot write core registers")
                 return
 
@@ -286,7 +288,7 @@ class WriteRegCommand(RegisterCommandBase):
                 value = float(self.value)
             else:
                 value = self._convert_value(self.value)
-            self.context.target.write_core_register(self.reg, value)
+            self.context.selected_core.write_core_register(self.reg, value)
             self.context.target.flush()
         else:
             value = self._convert_value(self.value)
@@ -301,22 +303,22 @@ class WriteRegCommand(RegisterCommandBase):
                     addr = p.base_address + r.address_offset
                     if len(subargs) == 2:
                         self.context.writei("writing 0x%x to 0x%x:%d (%s)", value, addr, r.size, r.name)
-                        self.context.target.write_memory(addr, value, r.size)
+                        self.context.selected_ap.write_memory(addr, value, r.size)
                     elif len(subargs) == 3:
                         f = [x for x in r.fields if x.name.lower() == subargs[2]]
                         if len(f):
                             f = f[0]
                             msb = f.bit_offset + f.bit_width - 1
                             lsb = f.bit_offset
-                            originalValue = self.context.target.read_memory(addr, r.size)
+                            originalValue = self.context.selected_ap.read_memory(addr, r.size)
                             value = bfi(originalValue, msb, lsb, value)
                             self.context.writei("writing 0x%x to 0x%x[%d:%d]:%d (%s.%s)",
                                     value, addr, msb, lsb, r.size, r.name, f.name)
-                            self.context.target.write_memory(addr, value, r.size)
+                            self.context.selected_ap.write_memory(addr, value, r.size)
                     else:
                         raise exceptions.CommandError("too many dots")
                     self.context.target.flush()
-                    if do_readback:
+                    if self.do_readback:
                         self._dump_peripheral_register(p, r, True)
                 else:
                     raise exceptions.CommandError("invalid register '%s' for %s" % (subargs[1], p.name))
@@ -328,29 +330,38 @@ class ResetCommand(CommandBase):
             'names': ['reset'],
             'group': 'standard',
             'category': 'device',
-            'nargs': [0, 1],
-            'usage': "[halt|-halt|-h]",
-            'help': "Reset the target.",
+            'nargs': [0, 1, 2],
+            'usage': "[halt|-halt|-h] [TYPE]",
+            'help': "Reset the target, optionally specifying the reset type.",
+            'extra_help': "The reset type must be one of 'default', 'hw', 'sw', 'hardware', 'software', "
+                          "'sw_sysresetreq', 'sw_vectreset', 'sw_emulated', 'sysresetreq', 'vectreset', "
+                          "or 'emulated'.",
+
             }
     
     def parse(self, args):
         self.do_halt = False
-        if len(args) == 1:
+        self.reset_type = None
+        if len(args) >= 1:
             self.do_halt = (args[0] in ('-h', '--halt', 'halt'))
+            if self.do_halt:
+                args.pop(0)
+        if len(args) == 1:
+            self.reset_type = convert_reset_type(args[0])
 
     def execute(self):
         if self.do_halt:
             self.context.write("Resetting target with halt")
-            self.context.target.reset_and_halt()
+            self.context.selected_core.reset_and_halt(self.reset_type)
 
-            status = self.context.target.get_state()
+            status = self.context.selected_core.get_state()
             if status != Target.State.HALTED:
                 self.context.writei("Failed to halt device on reset (state is %s)", status.name)
             else:
                 self.context.write("Successfully halted device on reset")
         else:
             self.context.write("Resetting target")
-            self.context.target.reset()
+            self.context.selected_core.reset(self.reset_type)
 
 class DisassembleCommand(CommandBase):
     INFO = {
@@ -382,7 +393,7 @@ class DisassembleCommand(CommandBase):
         self.addr &= ~1
 
         # Print disasm of data.
-        data = self.context.target.read_memory_block8(self.addr, self.count)
+        data = self.context.selected_ap.read_memory_block8(self.addr, self.count)
         print_disasm(self.context, bytes(bytearray(data)), self.addr)
 
 class ReadCommandBase(CommandBase):
@@ -400,13 +411,9 @@ class ReadCommandBase(CommandBase):
 
         if self.width == 8:
             data = self.context.selected_ap.read_memory_block8(self.addr, self.count)
-            byteData = data
-        elif self.width == 16:
-            byteData = self.context.selected_ap.read_memory_block8(self.addr, self.count)
-            data = conversion.byte_list_to_u16le_list(byteData)
-        elif self.width == 32:
-            byteData = self.context.selected_ap.read_memory_block8(self.addr, self.count)
-            data = conversion.byte_list_to_u32le_list(byteData)
+        else:
+            byte_data = self.context.selected_ap.read_memory_block8(self.addr, self.count)
+            data = conversion.byte_list_to_nbit_le_list(byte_data, self.width)
 
         # Print hex dump of output.
         self.context.write(dump_hex_data_to_str(data, start_address=self.addr, width=self.width), end='')
@@ -433,9 +440,9 @@ class Read16Command(ReadCommandBase):
             'usage': "ADDR [LEN]",
             'width': 16,
             'help': "Read 16-bit halfwords.",
-            'extra_help': "Optional length parameter is the number of bytes to read. It must be "
-                           "divisible by 2. If the length is not provided, one halfword is read. "
-                           "The address may be unaligned.",
+            'extra_help': "Optional length parameter is the number of bytes (not half-words) to read. It "
+                           "must be divisible by 2. If the length is not provided, one halfword is read. "
+                           "The address may be unaligned."
             }
 
 class Read32Command(ReadCommandBase):
@@ -447,23 +454,32 @@ class Read32Command(ReadCommandBase):
             'usage': "ADDR [LEN]",
             'width': 32,
             'help': "Read 32-bit words.",
-            'extra_help': "Optional length parameter is the number of bytes to read. It must be "
+            'extra_help': "Optional length parameter is the number of bytes (not words) to read. It must be "
                            "divisible by 4. If the length is not provided, one word is read. "
                            "The address may be unaligned.",
             }
 
+class Read64Command(ReadCommandBase):
+    INFO = {
+            'names': ['read64', 'rd'],
+            'group': 'standard',
+            'category': 'memory',
+            'nargs': [1, 2],
+            'usage': "ADDR [LEN]",
+            'width': 64,
+            'help': "Read 64-bit words.",
+            'extra_help': "Optional length parameter is the number of bytes (not double-words!) to read. "
+                           "It must be divisible by 8. If the length is not provided, one word is read. "
+                           "The address may be unaligned."
+            }
+
 def is_flash_write(context, addr, width, data):
     mem_map = context.target.get_memory_map()
     region = mem_map.get_region_for_address(addr)
     if (region is None) or (not region.is_flash):
         return False
 
-    if width == 8:
-        l = len(data)
-    elif width == 16:
-        l = len(data) * 2
-    elif width == 32:
-        l = len(data) * 4
+    l = len(data) * (width // 8)
 
     return region.contains_range(addr, length=l)
 
@@ -478,12 +494,8 @@ class WriteCommandBase(CommandBase):
         self.data = [self._convert_value(d) for d in args[1:]]
 
     def execute(self):
-        if self.width == 8:
-            pass
-        elif self.width == 16:
-            self.data = conversion.u16le_list_to_byte_list(self.data)
-        elif self.width == 32:
-            self.data = conversion.u32le_list_to_byte_list(self.data)
+        if self.width != 8:
+            self.data = conversion.nbit_le_list_to_byte_list(self.data, self.width)
 
         if is_flash_write(self.context, self.addr, self.width, self.data):
             # Look up flash region.
@@ -546,6 +558,21 @@ class Write32Command(WriteCommandBase):
                           "page must have been previously erased.",
             }
 
+class Write64Command(WriteCommandBase):
+    INFO = {
+            'names': ['write64', 'wd'],
+            'group': 'standard',
+            'category': 'memory',
+            'nargs': '*',
+            'usage': "ADDR DATA...",
+            'width': 64,
+            'help': "Write 64-bit double-words to memory.",
+            'extra_help': "The data arguments are 64-bit words in big-endian format and are written as "
+                          "little-endian. The address may be unaligned. Can write to both RAM and flash. "
+                          "Flash writes are subject to minimum write size and alignment, and the flash "
+                          "page must have been previously erased."
+            }
+
 class SavememCommand(CommandBase):
     INFO = {
             'names': ['savemem'],
@@ -899,8 +926,8 @@ class ContinueCommand(CommandBase):
             }
     
     def execute(self):
-        self.context.target.resume()
-        status = self.context.target.get_state()
+        self.context.selected_core.resume()
+        status = self.context.selected_core.get_state()
         if status == Target.State.RUNNING:
             self.context.write("Successfully resumed device")
         elif status == Target.State.SLEEPING:
@@ -931,16 +958,16 @@ class StepCommand(CommandBase):
             self.count = 1
     
     def execute(self):
-        if not self.context.target.is_halted():
+        if not self.context.selected_core.is_halted():
             self.context.write("Core is not halted; cannot step")
             return
 
         for i in range(self.count):
-            self.context.target.step(disable_interrupts=not self.context.session.options['step_into_interrupt'])
-            addr = self.context.target.read_core_register('pc')
+            self.context.selected_core.step(disable_interrupts=not self.context.session.options['step_into_interrupt'])
+            addr = self.context.selected_core.read_core_register('pc')
             if IS_CAPSTONE_AVAILABLE:
                 addr &= ~1
-                data = self.context.target.read_memory_block8(addr, 4)
+                data = self.context.selected_ap.read_memory_block8(addr, 4)
                 print_disasm(self.context, bytes(bytearray(data)), addr, max_instructions=1)
             else:
                 self.context.writei("PC = 0x%08x", addr)
@@ -956,9 +983,9 @@ class HaltCommand(CommandBase):
             }
     
     def execute(self):
-        self.context.target.halt()
+        self.context.selected_core.halt()
 
-        status = self.context.target.get_state()
+        status = self.context.selected_core.get_state()
         if status != Target.State.HALTED:
             self.context.writei("Failed to halt device; target state is %s", status.name.capitalize())
             return 1
@@ -979,8 +1006,8 @@ class BreakpointCommand(CommandBase):
         self.addr = self._convert_value(args[0])
 
     def execute(self):
-        if self.context.target.set_breakpoint(self.addr):
-            self.context.target.selected_core.bp_manager.flush()
+        if self.context.selected_core.set_breakpoint(self.addr):
+            self.context.selected_core.bp_manager.flush()
             self.context.writei("Set breakpoint at 0x%08x", self.addr)
         else:
             self.context.writei("Failed to set breakpoint at 0x%08x", self.addr)
@@ -1000,9 +1027,8 @@ class RemoveBreakpointCommand(CommandBase):
 
     def execute(self):
         try:
-            type = self.context.target.get_breakpoint_type(self.addr)
-            self.context.target.remove_breakpoint(self.addr)
-            self.context.target.selected_core.bp_manager.flush()
+            self.context.selected_core.remove_breakpoint(self.addr)
+            self.context.selected_core.bp_manager.flush()
             self.context.writei("Removed breakpoint at 0x%08x", self.addr)
         except Exception:
             self.context.writei("Failed to remove breakpoint at 0x%08x", self.addr)
@@ -1018,9 +1044,9 @@ class ListBreakpointsCommand(CommandBase):
             }
     
     def execute(self):
-        availableBpCount = self.context.target.selected_core.available_breakpoint_count
+        availableBpCount = self.context.selected_core.available_breakpoint_count
         self.context.writei("%d hardware breakpoints available", availableBpCount)
-        bps = self.context.target.selected_core.bp_manager.get_breakpoints()
+        bps = self.context.selected_core.bp_manager.get_breakpoints()
         if not len(bps):
             self.context.write("No breakpoints installed")
         else:
@@ -1049,14 +1075,14 @@ class WatchpointCommand(CommandBase):
         if len(args) > 2:
             self.sz = self._convert_value(args[2])
             if self.sz not in (1, 2, 4):
-                raise exceptions.CommandError("unsupported watchpoint size (%d)" % sz)
+                raise exceptions.CommandError("unsupported watchpoint size (%d)" % self.sz)
         else:
             self.sz = 4
 
     def execute(self):
-        if self.context.target.selected_core.dwt is None:
+        if self.context.selected_core.dwt is None:
             raise exceptions.CommandError("DWT not present")
-        if self.context.target.set_watchpoint(self.addr, self.sz, self.wptype):
+        if self.context.selected_core.set_watchpoint(self.addr, self.sz, self.wptype):
             self.context.writei("Set watchpoint at 0x%08x", self.addr)
         else:
             self.context.writei("Failed to set watchpoint at 0x%08x", self.addr)
@@ -1075,10 +1101,10 @@ class RemoveWatchpointCommand(CommandBase):
         self.addr = self._convert_value(args[0])
 
     def execute(self):
-        if self.context.target.selected_core.dwt is None:
+        if self.context.selected_core.dwt is None:
             raise exceptions.CommandError("DWT not present")
         try:
-            self.context.target.remove_watchpoint(self.addr)
+            self.context.selected_core.remove_watchpoint(self.addr)
             self.context.writei("Removed watchpoint at 0x%08x", self.addr)
         except Exception:
             self.context.writei("Failed to remove watchpoint at 0x%08x", self.addr)
@@ -1094,11 +1120,11 @@ class ListWatchpointsCommand(CommandBase):
             }
     
     def execute(self):
-        if self.context.target.selected_core.dwt is None:
+        if self.context.selected_core.dwt is None:
             raise exceptions.CommandError("DWT not present")
-        availableWpCount = self.context.target.selected_core.dwt.watchpoint_count
+        availableWpCount = self.context.selected_core.dwt.watchpoint_count
         self.context.writei("%d hardware watchpoints available", availableWpCount)
-        wps = self.context.target.selected_core.dwt.get_watchpoints()
+        wps = self.context.selected_core.dwt.get_watchpoints()
         if not len(wps):
             self.context.write("No watchpoints installed")
         else:
@@ -1106,7 +1132,7 @@ class ListWatchpointsCommand(CommandBase):
                 # TODO fix requirement to access WATCH_TYPE_TO_FUNCT
                 self.context.writei("%d: 0x%08x, %d bytes, %s",
                     i, wp.addr, wp.size, 
-                    WATCHPOINT_FUNCTION_NAME_MAP[self.context.target.selected_core.dwt.WATCH_TYPE_TO_FUNCT[wp.func]])
+                    WATCHPOINT_FUNCTION_NAME_MAP[self.context.selected_core.dwt.WATCH_TYPE_TO_FUNCT[wp.func]])
 
 class SelectCoreCommand(CommandBase):
     INFO = {
@@ -1121,20 +1147,19 @@ class SelectCoreCommand(CommandBase):
     def parse(self, args):
         if len(args) == 0:
             self.show_core = True
-            self.core = None
+            self.core_num = None
         else:
             self.show_core = False
-            self.core = int(args[0], base=0)
+            self.core_num = int(args[0], base=0)
 
     def execute(self):
         if self.show_core:
-            self.context.writei("Core %d is selected", self.context.target.selected_core.core_number)
+            self.context.writei("Core %d is selected", self.context.selected_core.core_number)
             return
-        original_core_ap = core_ap = self.context.target.selected_core.ap
-        self.context.target.selected_core = self.core
-        core_ap = self.context.target.selected_core.ap
+        self.context.selected_core = self.context.session.target.cores[self.core_num]
+        core_ap = self.context.selected_core.ap
         self.context.selected_ap_address = core_ap.address
-        self.context.writef("Selected core {} ({})", self.core, core_ap.short_description)
+        self.context.writef("Selected core {} ({})", self.core_num, core_ap.short_description)
 
 class ReadDpCommand(CommandBase):
     INFO = {
@@ -1225,8 +1250,7 @@ class InitDpCommand(CommandBase):
             }
 
     def execute(self):
-        self.context.target.dp.init()
-        self.context.target.dp.power_up_debug()
+        self.context.target.dp.connect()
 
 class MakeApCommand(CommandBase):
     INFO = {
@@ -1241,7 +1265,7 @@ class MakeApCommand(CommandBase):
     # TODO support ADIv6 AP addresses
     def parse(self, args):
         self.apsel = self._convert_value(args[0])
-        self.ap_addr = coresight.ap.APv1Address(apsel)
+        self.ap_addr = coresight.ap.APv1Address(self.apsel)
 
     def execute(self):
         if self.ap_addr in self.context.target.aps:
@@ -1283,7 +1307,7 @@ class WhereCommand(CommandBase):
         if len(args) >= 1:
             self.addr = self._convert_value(args[0])
         else:
-            self.addr = self.context.target.read_core_register('pc')
+            self.addr = self.context.selected_core.read_core_register('pc')
         self.addr &= ~0x1 # remove thumb bit
 
     def execute(self):
@@ -1356,7 +1380,7 @@ class GdbserverCommand(CommandBase):
             raise exceptions.CommandError("invalid action")
 
     def execute(self):
-        core_number = self.context.target.selected_core.core_number
+        core_number = self.context.selected_core.core_number
         if self.action == 'start':
             if core_number not in self.context.session.gdbservers:
                 # Persist the gdbserver
@@ -1436,15 +1460,20 @@ class ShowCommand(CommandBase):
         self.args = args[1:]
 
     def execute(self):
+        # Look up value class by name.
         try:
             value_class = self.context.command_set.values[self.name]
-            if 'r' not in value_class.INFO['access']:
-                raise exceptions.CommandError("value '%s' is not readable" % self.name)
-            value_object = value_class(self.context)
-            value_object.display(self.args)
         except KeyError:
             raise exceptions.CommandError("unknown value name '%s'" % self.name)
 
+        # Check readability.
+        if 'r' not in value_class.INFO['access']:
+            raise exceptions.CommandError("value '%s' is not readable" % self.name)
+        
+        # Execute show operation.
+        value_object = value_class(self.context)
+        value_object.display(self.args)
+
     @classmethod
     def format_help(cls, context, max_width=72):
         text = "Usage: {cmd} {usage}\n".format(cmd=cls.INFO['names'][0], usage=cls.INFO['usage'])
@@ -1477,15 +1506,20 @@ class SetCommand(CommandBase):
         self.args = args[1:]
 
     def execute(self):
+        # Look up value class by name.
         try:
             value_class = self.context.command_set.values[self.name]
-            if 'w' not in value_class.INFO['access']:
-                raise exceptions.CommandError("value '%s' is not modifiable" % self.name)
-            value_object = value_class(self.context)
-            value_object.modify(self.args)
         except KeyError:
             raise exceptions.CommandError("unknown value name '%s'" % self.name)
 
+        # Check writability.
+        if 'w' not in value_class.INFO['access']:
+            raise exceptions.CommandError("value '%s' is not modifiable" % self.name)
+        
+        # Execute set operation.
+        value_object = value_class(self.context)
+        value_object.modify(self.args)
+
     @classmethod
     def format_help(cls, context, max_width=72):
         text = "Usage: {cmd} {usage}\n".format(cmd=cls.INFO['names'][0], usage=cls.INFO['usage'])
@@ -1536,7 +1570,7 @@ Prefix line with ! to execute a shell command."""
                         ", ".join("'%s'" % c for c in matched_commands))
                 return
             elif len(matched_commands) == 0:
-                raise exceptions.CommandError("Error: unrecognized command '%s'", cmd_name)
+                raise exceptions.CommandError("Error: unrecognized command '%s'" % cmd_name)
             cmd_name = matched_commands[0]
             cmd_class = self.context.command_set.commands[cmd_name]
 
@@ -1550,7 +1584,7 @@ Prefix line with ! to execute a shell command."""
                                 ", ".join("'%s'" % c for c in matched_values))
                         return
                     elif len(matched_values) == 0:
-                        raise exceptions.CommandError("Error: unrecognized value '%s'", value_name)
+                        raise exceptions.CommandError("Error: unrecognized value '%s'" % value_name)
                     value_name = matched_values[0]
                     cmd_class = self.context.command_set.values[value_name]
                     self.context.write(cmd_class.format_help(self.context, self.term_width))
@@ -1588,7 +1622,6 @@ def print_disasm(context, code, start_addr, max_instructions=None):
         pc = -1
     md = capstone.Cs(capstone.CS_ARCH_ARM, capstone.CS_MODE_THUMB)
 
-    addrLine = 0
     text = ''
     n = 0
     for i in md.disasm(code, start_addr):
