@@ -1,5 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2015-2020 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,10 +17,9 @@
 
 import logging
 import prettytable
-import textwrap
 
 from .. import coresight
-from ..core import (exceptions, session)
+from ..core import exceptions
 from ..probe.debug_probe import DebugProbe
 from ..coresight.ap import MEM_AP
 from ..core.target import Target
@@ -96,7 +96,26 @@ class CoresValue(ValueBase):
             for i, core in self.context.target.cores.items():
                 self.context.writei("Core %d type:  %s%s", i,
                         coresight.core_ids.CORE_TYPE_NAME[core.core_type],
-                        " (selected)" if (self.context.target.selected_core.core_number == i) else "")
+                        " (selected)" if ((self.context.selected_core is not None) \
+                                            and (self.context.selected_core.core_number == i)) else "")
+
+class APsValue(ValueBase):
+    INFO = {
+            'names': ['aps'],
+            'group': 'standard',
+            'category': 'target',
+            'access': 'r',
+            'help': "List discovered Access Ports.",
+            }
+
+    def display(self, args):
+        if self.context.target.is_locked():
+            self.context.write("Target is locked")
+        else:
+            self.context.writei("%d APs:", len(self.context.target.aps))
+            for addr, ap in sorted(self.context.target.aps.items(), key=lambda x: x[0]):
+                self.context.writei("%s: %s%s", addr, ap.type_name,
+                        " (selected)" if (self.context.selected_ap_address == addr) else "")
 
 class MemoryMapValue(ValueBase):
     INFO = {
@@ -108,10 +127,14 @@ class MemoryMapValue(ValueBase):
             }
 
     def display(self, args):
+        if self.context.selected_core is None:
+            self.context.write("No core is selected")
+            return
+
         pt = prettytable.PrettyTable(["Region", "Type", "Start", "End", "Size", "Access", "Sector", "Page"])
         pt.align = 'l'
         pt.border = False
-        for region in self.context.target.get_memory_map():
+        for region in self.context.selected_core.get_memory_map():
             pt.add_row([
                 region.name,
                 region.type.name.capitalize(),
@@ -156,7 +179,7 @@ class FaultValue(ValueBase):
         DFSR = 0xe000ed30
         MMFAR = 0xe000ed34
         BFAR = 0xe000ed38
-        AFSR = 0xe000ed3c
+#         AFSR = 0xe000ed3c
         
         MMFSR_fields = [
                 ('IACCVIOL', 0),
@@ -205,14 +228,18 @@ class FaultValue(ValueBase):
                 if showAll or bit != 0:
                     self.context.writei("    %s = 0x%x", name, bit)
         
-        cfsr = self.context.target.read32(CFSR)
+        if self.context.selected_core is None:
+            self.context.write("No core is selected")
+            return
+
+        cfsr = self.context.selected_core.read32(CFSR)
         mmfsr = cfsr & 0xff
         bfsr = (cfsr >> 8) & 0xff
         ufsr = (cfsr >> 16) & 0xffff
-        hfsr = self.context.target.read32(HFSR)
-        dfsr = self.context.target.read32(DFSR)
-        mmfar = self.context.target.read32(MMFAR)
-        bfar = self.context.target.read32(BFAR)
+        hfsr = self.context.selected_core.read32(HFSR)
+        dfsr = self.context.selected_core.read32(DFSR)
+        mmfar = self.context.selected_core.read32(MMFAR)
+        bfar = self.context.selected_core.read32(BFAR)
         
         print_fields('MMFSR', mmfsr, MMFSR_fields, showAll)
         if showAll or mmfsr & (1 << 7): # MMFARVALID
@@ -243,7 +270,13 @@ class NresetValue(ValueBase):
             raise exceptions.CommandError("missing reset state")
         state = int(args[0], base=0)
         self.context.writef("nRESET = {}", state)
-        self.context.probe.assert_reset((state == 0))
+        
+        # Use the probe to assert reset if the DP doesn't exist for some reason, otherwise
+        # use the DP so reset notifications are sent.
+        if self.context.target.dp is None:
+            self.context.probe.assert_reset((state == 0))
+        else:
+            self.context.target.dp.assert_reset((state == 0))
 
 class SessionOptionValue(ValueBase):
     INFO = {
@@ -286,6 +319,9 @@ class MemApValue(ValueBase):
             }
 
     def display(self, args):
+        if self.context.selected_ap is None:
+            self.context.write("No MEM-AP is selected")
+            return
         self.context.writef("{} is selected", self.context.selected_ap.short_description)
 
     def modify(self, args):
@@ -316,6 +352,9 @@ class HnonsecValue(ValueBase):
             }
 
     def display(self, args):
+        if self.context.selected_ap is None:
+            self.context.write("No MEM-AP is selected")
+            return
         self.context.writef("{} HNONSEC = {} ({})",
             self.context.selected_ap.short_description,
             self.context.selected_ap.hnonsec,
@@ -324,6 +363,9 @@ class HnonsecValue(ValueBase):
     def modify(self, args):
         if len(args) == 0:
             raise exceptions.CommandError("missing argument")
+        if self.context.selected_ap is None:
+            self.context.write("No MEM-AP is selected")
+            return
         value = int(args[0], base=0)
         self.context.selected_ap.hnonsec = value
 
@@ -337,6 +379,9 @@ class HprotValue(ValueBase):
             }
 
     def display(self, args):
+        if self.context.selected_ap is None:
+            self.context.write("No MEM-AP is selected")
+            return
         hprot = self.context.selected_ap.hprot
         self.context.writef("{} HPROT = {:#x}",
             self.context.selected_ap.short_description,
@@ -353,6 +398,9 @@ class HprotValue(ValueBase):
     def modify(self, args):
         if len(args) == 0:
             raise exceptions.CommandError("missing argument")
+        if self.context.selected_ap is None:
+            self.context.write("No MEM-AP is selected")
+            return
         value = int(args[0], base=0)
         self.context.selected_ap.hprot = value
 
@@ -393,7 +441,10 @@ class RegisterGroupsValue(ValueBase):
             }
 
     def display(self, args):
-        for g in sorted(self.context.target.core_registers.groups):
+        if self.context.selected_core is None:
+            self.context.write("No core is selected")
+            return
+        for g in sorted(self.context.selected_core.core_registers.groups):
             self.context.write(g)
 
 class VectorCatchValue(ValueBase):
@@ -409,7 +460,11 @@ class VectorCatchValue(ValueBase):
             }
 
     def display(self, args):
-        catch = self.context.target.get_vector_catch()
+        if self.context.selected_core is None:
+            self.context.write("No core is selected")
+            return
+
+        catch = self.context.selected_core.get_vector_catch()
 
         self.context.write("Vector catch:")
         for mask in sorted(VC_NAMES_MAP.keys()):
@@ -420,9 +475,13 @@ class VectorCatchValue(ValueBase):
     def modify(self, args):
         if len(args) == 0:
             raise exceptions.CommandError("missing vector catch setting")
+
+        if self.context.selected_core is None:
+            self.context.write("No core is selected")
+            return
     
         try:
-            self.context.target.set_vector_catch(convert_vector_catch(args[0]))
+            self.context.selected_core.set_vector_catch(convert_vector_catch(args[0]))
         except ValueError as e:
             self.context.write(e)
 
@@ -472,11 +531,9 @@ class LogLevelValue(ValueBase):
     def modify(self, args):
         if len(args) < 1:
             raise exceptions.CommandError("no log level provided")
-            return 1
         level_name = args[0].lower()
         if level_name not in self.LEVELS:
             raise exceptions.CommandError("log level must be one of {%s}" % ','.join(self.LEVELS.keys()))
-            return 1
         level = self.LEVELS[level_name]
         if len(args) == 1:
             logging.getLogger().setLevel(level)
@@ -512,12 +569,10 @@ class ClockFrequencyValue(ValueBase):
     def modify(self, args):
         if len(args) < 1:
             raise exceptions.CommandError("no clock frequency provided")
-            return 1
         try:
             freq_Hz = convert_frequency(args[0])
         except:
             raise exceptions.CommandError("invalid frequency")
-            return 1
         self.context.probe.set_clock(freq_Hz)
         if self.context.probe.wire_protocol == DebugProbe.Protocol.SWD:
             swd_jtag = 'SWD'
