@@ -1,5 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2018-2019 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,14 +15,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import print_function
-from .session import Session
-from ..probe.aggregator import DebugProbeAggregator
 from time import sleep
 import colorama
-import six
 import prettytable
 
+from .session import Session
+from ..probe.aggregator import DebugProbeAggregator
+
 # Init colorama here since this is currently the only module that uses it.
 colorama.init()
 
@@ -168,7 +168,7 @@ class ConnectHelper(object):
             while True:
                 print(colorama.Style.RESET_ALL)
                 print("Enter the number of the debug probe or 'q' to quit", end='')
-                line = six.moves.input("> ")
+                line = input("> ")
                 valid = False
                 if line.strip().lower() == 'q':
                     return None
