@@ -1,5 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2018-2020 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,16 +17,10 @@
 
 import logging
 import logging.config
-import six
 import yaml
 import os
 import weakref
-
-# inspect.getargspec is deprecated in Python 3.
-try:
-    from inspect import getfullargspec as getargspec
-except ImportError:
-    from inspect import getargspec
+from inspect import getfullargspec
 
 from . import exceptions
 from .options_manager import OptionsManager
@@ -153,7 +148,7 @@ class Session(Notifier):
         
         # Init project directory.
         if self.options.get('project_dir') is None:
-            self._project_dir = os.getcwd()
+            self._project_dir = os.environ.get('PYOCD_PROJECT_DIR') or os.getcwd()
         else:
             self._project_dir = os.path.abspath(os.path.expanduser(self.options.get('project_dir')))
         LOG.debug("Project directory: %s", self.project_dir)
@@ -185,8 +180,7 @@ class Session(Notifier):
         self._load_user_script()
         
         # Ask the probe if it has an associated board, and if not then we create a generic one.
-        self._board = probe.create_associated_board() \
-                        or Board(self, self.options.get('target_override'))
+        self._board = probe.create_associated_board() or Board(self)
     
     def _get_config(self):
         # Load config file if one was provided via options, and no_config option was not set.
@@ -198,7 +192,11 @@ class Session(Notifier):
                     with open(configPath, 'r') as configFile:
                         LOG.debug("Loading config from: %s", configPath)
                         config = yaml.safe_load(configFile)
-                        if not isinstance(config, dict):
+                        # Allow an empty config file.
+                        if config is None:
+                            return {}
+                        # But fail if someone tries to put something other than a dict at the top.
+                        elif not isinstance(config, dict):
                             raise exceptions.Error("configuration file %s does not contain a top-level dictionary"
                                     % configPath)
                         return config
@@ -240,7 +238,7 @@ class Session(Notifier):
         config = self.options.get('logging')
         
         # Allow logging setting to refer to another file.
-        if isinstance(config, six.string_types):
+        if isinstance(config, str):
             loggingConfigPath = self.find_user_file(None, [config])
             
             if loggingConfigPath is not None:
@@ -423,7 +421,7 @@ class Session(Notifier):
                 # functions or classes. A single namespace is shared for both globals and
                 # locals so that script-level definitions are available within the
                 # script functions.
-                six.exec_(scriptCode, self._user_script_namespace, self._user_script_namespace)
+                exec(scriptCode, self._user_script_namespace, self._user_script_namespace)
                 
                 # Create the proxy for the user script. It becomes the delegate unless
                 # another delegate was already set.
@@ -475,17 +473,17 @@ class Session(Notifier):
             try:
                 self.board.uninit()
                 self._inited = False
-            except:
+            except exceptions.Error:
                 LOG.error("exception during board uninit:", exc_info=self.log_tracebacks)
         
         if self._probe.is_open:
             try:
                 self._probe.disconnect()
-            except:
+            except exceptions.Error:
                 LOG.error("probe exception during disconnect:", exc_info=self.log_tracebacks)
             try:
                 self._probe.close()
-            except:
+            except exceptions.Error:
                 LOG.error("probe exception during close:", exc_info=self.log_tracebacks)
 
 class UserScriptFunctionProxy(object):
@@ -496,7 +494,7 @@ class UserScriptFunctionProxy(object):
 
     def __init__(self, fn):
         self._fn = fn
-        self._spec = getargspec(fn)
+        self._spec = getfullargspec(fn)
     
     def __call__(self, **kwargs):
         args = {}
