@@ -1,5 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2015-2020 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -20,6 +21,7 @@ from functools import total_ordering
 from enum import Enum
 
 from ..core import (exceptions, memory_interface)
+from ..core.target import Target
 from ..utility.concurrency import locked
 
 LOG = logging.getLogger(__name__)
@@ -103,6 +105,9 @@ CSW_SIZE     =  0x00000007
 CSW_SIZE8    =  0x00000000
 CSW_SIZE16   =  0x00000001
 CSW_SIZE32   =  0x00000002
+CSW_SIZE64   =  0x00000003
+CSW_SIZE128  =  0x00000004
+CSW_SIZE256  =  0x00000005
 CSW_ADDRINC  =  0x00000030
 CSW_NADDRINC =  0x00000000 # No increment
 CSW_SADDRINC =  0x00000010 # Single increment by SIZE field
@@ -121,7 +126,10 @@ DEFAULT_CSW_VALUE = CSW_SADDRINC
 
 TRANSFER_SIZE = {8: CSW_SIZE8,
                  16: CSW_SIZE16,
-                 32: CSW_SIZE32
+                 32: CSW_SIZE32,
+                 64: CSW_SIZE64,
+                 128: CSW_SIZE128,
+                 256: CSW_SIZE256,
                  }
 
 CSW_HPROT_MASK = 0x0f000000 # HPROT[3:0]
@@ -190,7 +198,7 @@ class APAddressBase(object):
     @property
     def ap_version(self):
         """! @brief Version of the AP, as an APVersion enum."""
-        raise NotImplemented()
+        raise NotImplementedError()
     
     @property
     def nominal_address(self):
@@ -205,12 +213,12 @@ class APAddressBase(object):
         
         This value can be passed to the DebugPort's read_ap() or write_ap() methods. Offsets of
         registers can be added to this value to create register addresses."""
-        raise NotImplemented()
+        raise NotImplementedError()
     
     @property
     def idr_address(self):
         """! @brief Address of the IDR register."""
-        raise NotImplemented()
+        raise NotImplementedError()
     
     def __hash__(self):
         return hash(self.nominal_address)
@@ -330,7 +338,7 @@ class AccessPort(object):
             name, klass, flags = AP_TYPE_MAP[key]
         except KeyError:
             # The AP ID doesn't match, but we can recognize unknown MEM-APs.
-            if apClass == AP_CLASS_MEM_AP:
+            if (apClass == AP_CLASS_MEM_AP) and (designer == AP_JEP106_ARM):
                 name = "MEM-AP"
                 klass = MEM_AP
             else:
@@ -380,7 +388,7 @@ class AccessPort(object):
     def init(self):
         # Read IDR if it wasn't given to us in the ctor.
         if self.idr is None:
-            self.idr = self.read_reg(AP_IDR)
+            self.idr = self.read_reg(self.address.idr_address)
         
         self.variant = (self.idr & AP_IDR_VARIANT_MASK) >> AP_IDR_VARIANT_SHIFT
         self.revision = (self.idr & AP_IDR_REVISION_MASK) >> AP_IDR_REVISION_SHIFT
@@ -407,10 +415,6 @@ class AccessPort(object):
     def write_reg(self, addr, data):
         self.dp.write_ap(self.address.address + addr, data)
     
-    def reset_did_occur(self):
-        """! @brief Invoked by the DebugPort to inform APs that a reset was performed."""
-        pass
-    
     def lock(self):
         """! @brief Lock the AP from access by other threads."""
         self.dp.probe.lock()
@@ -499,6 +503,9 @@ class MEM_AP(AccessPort, memory_interface.MemoryInterface):
         ## Mask of addresses. This indicates whether 32-bit or 64-bit addresses are supported.
         self._address_mask = 0xffffffff
         
+        ## Whether the Large Data extension is supported.
+        self._has_large_data = False
+        
         # Ask the probe for an accelerated memory interface for this AP. If it provides one,
         # then bind our memory interface APIs to its methods. Otherwise use our standard
         # memory interface based on AP register accesses.
@@ -514,133 +521,217 @@ class MEM_AP(AccessPort, memory_interface.MemoryInterface):
             self.read_memory = self._read_memory
             self.write_memory_block32 = self._write_memory_block32
             self.read_memory_block32 = self._read_memory_block32
+        
+        # Subscribe to reset events.
+        self.dp.session.subscribe(self._reset_did_occur, (Target.Event.PRE_RESET, Target.Event.POST_RESET))
 
     @property
     def supported_transfer_sizes(self):
         """! @brief Tuple of transfer sizes supported by this AP."""
         return self._transfer_sizes
+    
+    @property
+    def is_enabled(self):
+        """! @brief Whether any memory transfers are allowed by this AP.
+        
+        Memory transfers may be disabled by an input signal to the AP. This is often done when debug security
+        is enabled on the device, to disallow debugger access to internal memory.
+        """
+        return self.is_enabled_for(Target.SecurityState.NONSECURE)
+    
+    def is_enabled_for(self, security_state):
+        """! @brief Checks whether memory transfers are allowed by this AP for the given security state.
+        
+        Memory transfers may be disabled by an input signal to the AP. This is often done when debug security
+        is enabled on the device, to disallow debugger access to internal memory.
+        
+        @param self The AP instance.
+        @param security_state One of the @ref pyocd.core.target.Target.SecurityState "SecurityState" enums.
+        @return Boolean indicating whether memory transfers can be performed in the requested security state. You
+            may change the security state used for transfers with the hnonsec property and hnonsec_lock() method.
+        """
+        assert isinstance(security_state, Target.SecurityState)
+        
+        # Call to superclass to read CSW. We want to bypass our CSW cache since the enable signal can change
+        # asynchronously.
+        csw = AccessPort.read_reg(self, self._reg_offset + MEM_AP_CSW)
+        if security_state is Target.SecurityState.NONSECURE:
+            # Nonsecure transfers are always allowed when security transfers are enabled.
+            return (csw & (CSW_DEVICEEN | CSW_SDEVICEEN)) != 0
+        elif security_state is Target.SecurityState.SECURE:
+            return (csw & CSW_SDEVICEEN) != 0
+        else:
+            assert False, "unsupported security state"
 
     @locked
     def init(self):
+        """! @brief Initialize the MEM-AP.
+        
+        This method interrogates the MEM-AP to determine its capabilities, and performs any initial setup
+        that is required.
+        
+        It performs these checks:
+        - Check for Long Address extension.
+        - Check for Large Data extension.
+        - (v2 only) Get the auto-increment page size.
+        - (v2 only) Determine supported error mode.
+        - (v2 only) Get the size of the DAR register window.
+        - Determine supported transfer sizes.
+        - Determine the implemented HPROT and HNONSEC controls.
+        - Read the ROM table base address.
+
+        These controls are configured.
+        - (v2 only) Configure the error mode.
+        """
         super(MEM_AP, self).init()
         
-        self._init_transfer_sizes()
-        self._init_hprot()
-        self._init_rom_table_base()
-        self._init_cfg()
-
-    def _init_transfer_sizes(self):
-        """! @brief Determine supported transfer sizes.
-        
-        If the AP_ALL_TX_SZ flag is set, then we know a priori that this AP implementation
-        supports all transfer sizes.
-        
-        Otherwise an attempt to modify CSW_SIZE to a non-32-bit transfer size is made. If CSW_SIZE
-        is successfully changed, then the AP supports both 8- and 16-bit transfers in addition to
-        the required 32-bit.
-        """
-        # If AP_ALL_TX_SZ is set, we can skip the test.
-        if (self._flags & AP_ALL_TX_SZ) == 0:
-            self._transfer_sizes = (8, 16, 32)
-            return
-
-        # Read initial CSW.
-        csw = AccessPort.read_reg(self, self._reg_offset + MEM_AP_CSW)
-        original_csw = csw
-        
-        # Write CSW_SIZE to select 16-bit transfer.
-        AccessPort.write_reg(self, self._reg_offset + MEM_AP_CSW, csw & ~CSW_SIZE | CSW_SIZE16)
-        
-        # Read back CSW and see if SIZE changed.
-        csw = AccessPort.read_reg(self, self._reg_offset + MEM_AP_CSW)
-        if (csw & CSW_SIZE) == CSW_SIZE16:
-            self._transfer_sizes = (8, 16, 32)
- 
-        # Restore unmodified value of CSW.
-        AccessPort.write_reg(self, self._reg_offset + MEM_AP_CSW, original_csw)
-
-    def _init_hprot(self):
-        """! @brief Init HPROT HNONSEC.
-        
-        Determines the implemented bits of HPROT and HNONSEC in this MEM-AP. The defaults for these
-        fields of the CSW are based on the implemented bits.
-        """
-        # Read initial values of HPROT and HNONSEC. Superclass register access methods are used to
-        # avoid the CSW cache.
-        csw = AccessPort.read_reg(self, self._reg_offset + MEM_AP_CSW)
-        original_csw = csw
-        
-        default_hprot = (csw & CSW_HPROT_MASK) >> CSW_HPROT_SHIFT
-        default_hnonsec = (csw & CSW_HNONSEC_MASK) >> CSW_HNONSEC_SHIFT
-        LOG.debug("%s default HPROT=%x HNONSEC=%x", self.short_description, default_hprot, default_hnonsec)
-        
-        # Now attempt to see which HPROT and HNONSEC bits are implemented.
-        AccessPort.write_reg(self, self._reg_offset + MEM_AP_CSW, csw | CSW_HNONSEC_MASK | CSW_HPROT_MASK)
-        csw = AccessPort.read_reg(self, self._reg_offset + MEM_AP_CSW)
-        
-        self._impl_hprot = (csw & CSW_HPROT_MASK) >> CSW_HPROT_SHIFT
-        self._impl_hnonsec = (csw & CSW_HNONSEC_MASK) >> CSW_HNONSEC_SHIFT
-        LOG.debug("%s implemented HPROT=%x HNONSEC=%x", self.short_description, self._impl_hprot, self._impl_hnonsec)
-        
-        # Update current HPROT and HNONSEC, and the current base CSW value.
-        self.hprot = self._hprot & self._impl_hprot
-        self.hnonsec = self._hnonsec & self._impl_hnonsec
- 
-        # Restore unmodified value of CSW.
-        AccessPort.write_reg(self, self._reg_offset + MEM_AP_CSW, original_csw)
-
-    def _init_rom_table_base(self):
-        """! @brief Read ROM table base address."""
-        base = self.read_reg(self._reg_offset + MEM_AP_BASE)
-        is_adiv5_base = (base & AP_BASE_FORMAT_MASK) != 0
-        is_base_present = (base & AP_BASE_ENTRY_PRESENT_MASK) != 0
-        is_legacy_base_present = not is_adiv5_base and not is_base_present
-        if is_legacy_base_present:
-            self.has_rom_table = True
-            self.rom_addr = base & AP_BASE_LEGACY_BASEADDR_MASK # clear format and present bits
-        elif (base == AP_BASE_LEGACY_NOTPRESENT) or (not is_base_present):
-            self.has_rom_table = False
-            self.rom_addr = 0
-        elif is_adiv5_base and is_base_present:
-            self.has_rom_table = True
-            self.rom_addr = base & AP_BASE_BASEADDR_MASK # clear format and present bits
-        else:
-            raise exceptions.TargetError("invalid AP BASE value 0x%08x" % base)
+        # Read initial CSW. Superclass register access methods are used to avoid the CSW cache.
+        original_csw = AccessPort.read_reg(self, self._reg_offset + MEM_AP_CSW)
     
-    def _init_cfg(self):
-        """! @brief Read MEM-AP CFG register."""
-        cfg = self.read_reg(self._reg_offset + MEM_AP_CFG)
+        def _init_cfg():
+            """! @brief Read MEM-AP CFG register."""
+            cfg = self.read_reg(self._reg_offset + MEM_AP_CFG)
         
-        # Check for 64-bit address support.
-        if cfg & MEM_AP_CFG_LA_MASK:
-            self._address_mask = 0xffffffffffffffff
+            # Check for 64-bit address support.
+            if cfg & MEM_AP_CFG_LA_MASK:
+                self._address_mask = 0xffffffffffffffff
         
-        # Check v2 MEM-AP CFG fields.
-        if self.ap_version == APVersion.APv2:
-            # Set autoinc page size if TARINC is non-zero. Otherwise we've already set the
-            # default of 1 kB in the ctor.
-            tarinc = (cfg & MEM_AP_CFG_TARINC_MASK) >> MEM_AP_CFG_TARINC_SHIFT
-            if tarinc != 0:
-                self.auto_increment_page_size = 1 << (9 + tarinc)
+            # Check for Large Data extension.
+            if cfg & MEM_AP_CFG_LD_MASK:
+                self._has_large_data = True
         
-            # Determine supported err mode.
-            err = (cfg & MEM_AP_CFG_ERR_MASK) >> MEM_AP_CFG_ERR_SHIFT
-            if err == MEM_AP_CFG_ERR_V1:
-                # Configure the error mode such that errors are passed upstream, but they don't
-                # prevent future transactions.
-                self._csw &= ~(CSW_ERRSTOP | CSW_ERRNPASS)
+            # Check v2 MEM-AP CFG fields.
+            if self.ap_version == APVersion.APv2:
+                # Set autoinc page size if TARINC is non-zero. Otherwise we've already set the
+                # default of 1 kB in the ctor.
+                tarinc = (cfg & MEM_AP_CFG_TARINC_MASK) >> MEM_AP_CFG_TARINC_SHIFT
+                if tarinc != 0:
+                    self.auto_increment_page_size = 1 << (9 + tarinc)
+        
+                # Determine supported err mode.
+                err = (cfg & MEM_AP_CFG_ERR_MASK) >> MEM_AP_CFG_ERR_SHIFT
+                if err == MEM_AP_CFG_ERR_V1:
+                    # Configure the error mode such that errors are passed upstream, but they don't
+                    # prevent future transactions.
+                    self._csw &= ~(CSW_ERRSTOP | CSW_ERRNPASS)
             
-                # Clear TRR in case we attach to a device with a sticky error already set.
-                self.write_reg(self._reg_offset + MEM_AP_TRR, MEM_AP_TRR_ERR_MASK)
+                    # Clear TRR in case we attach to a device with a sticky error already set.
+                    self.write_reg(self._reg_offset + MEM_AP_TRR, MEM_AP_TRR_ERR_MASK)
         
-            # Init size of DAR register window.
-            darsize = (cfg & MEM_AP_CFG_DARSIZE_MASK) >> MEM_AP_CFG_DARSIZE_SHIFT
-            self._dar_count = (1 << darsize) // 4
+                # Init size of DAR register window.
+                darsize = (cfg & MEM_AP_CFG_DARSIZE_MASK) >> MEM_AP_CFG_DARSIZE_SHIFT
+                self._dar_count = (1 << darsize) // 4
+
+        def _init_transfer_sizes():
+            """! @brief Determine supported transfer sizes.
+        
+            If the #AP_ALL_TX_SZ flag is set, then we know a priori that this AP implementation
+            supports 8-, 16- and 32- transfer sizes. If the Large Data extension is implemented, then this
+            flag is ignored.
+        
+            Note in ADIv6: "If a MEM-AP implementation does not support the Large Data Extension, but does
+            support various access sizes, it must support word, halfword, and byte accesses."
+
+            So, if the Large Data extension is present, then we have to individually test each
+            transfer size (aside from the required 32-bit).
+            
+            If Large Data is not present, then only one non-32-bit transfer size needs to be tested to
+            determine if the AP supports both 8- and 16-bit transfers in addition to the required 32-bit.
+            """
+            # If AP_ALL_TX_SZ is set, we can skip the test. Double check this by ensuring that LD is not
+            # enabled.
+            if (self._flags & AP_ALL_TX_SZ) and not self._has_large_data:
+                self._transfer_sizes = (8, 16, 32)
+                return
+        
+            def _test_transfer_size(sz):
+                """! @brief Utility to verify whether the MEM-AP supports a given transfer size.
+                
+                From ADIv6:
+                If the CSW.Size field is written with a value corresponding to a size that is not supported,
+                or with a reserved value: A read of the field returns a value corresponding to a supported
+                size.
+                """
+                # Write CSW_SIZE to select requested transfer size.
+                AccessPort.write_reg(self, self._reg_offset + MEM_AP_CSW, original_csw & ~CSW_SIZE | sz)
+        
+                # Read back CSW and see if SIZE matches what we wrote.
+                csw_cb = AccessPort.read_reg(self, self._reg_offset + MEM_AP_CSW, now=False)
+                
+                return lambda: (csw_cb() & CSW_SIZE) == sz
+            
+            # Thus if LD ext is not present, we only need to test one size.
+
+            if self._has_large_data:
+                # Need to scan all sizes except 32-bit, which is required.
+                SIZES_TO_TEST = (CSW_SIZE8, CSW_SIZE16, CSW_SIZE64, CSW_SIZE128, CSW_SIZE256)
+                
+                sz_result_cbs = ((sz, _test_transfer_size(sz)) for sz in SIZES_TO_TEST)
+                self._transfer_sizes = ([32] + [(8 * (1 << sz)) for sz, cb in sz_result_cbs if cb()])
+                self._transfer_sizes.sort()
+                
+            elif _test_transfer_size(CSW_SIZE16)():
+                self._transfer_sizes = (8, 16, 32)
+
+        def _init_hprot():
+            """! @brief Init HPROT HNONSEC.
+        
+            Determines the implemented bits of HPROT and HNONSEC in this MEM-AP. The defaults for these
+            fields of the CSW are based on the implemented bits.
+            """
+            default_hprot = (original_csw & CSW_HPROT_MASK) >> CSW_HPROT_SHIFT
+            default_hnonsec = (original_csw & CSW_HNONSEC_MASK) >> CSW_HNONSEC_SHIFT
+            LOG.debug("%s default HPROT=%x HNONSEC=%x", self.short_description, default_hprot, default_hnonsec)
+        
+            # Now attempt to see which HPROT and HNONSEC bits are implemented.
+            AccessPort.write_reg(self, self._reg_offset + MEM_AP_CSW,
+                    original_csw | CSW_HNONSEC_MASK | CSW_HPROT_MASK)
+            csw = AccessPort.read_reg(self, self._reg_offset + MEM_AP_CSW)
+        
+            self._impl_hprot = (csw & CSW_HPROT_MASK) >> CSW_HPROT_SHIFT
+            self._impl_hnonsec = (csw & CSW_HNONSEC_MASK) >> CSW_HNONSEC_SHIFT
+            LOG.debug("%s implemented HPROT=%x HNONSEC=%x", self.short_description, self._impl_hprot,
+                    self._impl_hnonsec)
+        
+            # Update current HPROT and HNONSEC, and the current base CSW value.
+            self.hprot = self._hprot & self._impl_hprot
+            self.hnonsec = self._hnonsec & self._impl_hnonsec
+
+        def _init_rom_table_base():
+            """! @brief Read ROM table base address."""
+            base = self.read_reg(self._reg_offset + MEM_AP_BASE)
+            is_adiv5_base = (base & AP_BASE_FORMAT_MASK) != 0
+            is_base_present = (base & AP_BASE_ENTRY_PRESENT_MASK) != 0
+            is_legacy_base_present = not is_adiv5_base and not is_base_present
+            if is_legacy_base_present:
+                self.has_rom_table = True
+                self.rom_addr = base & AP_BASE_LEGACY_BASEADDR_MASK # clear format and present bits
+            elif (base == AP_BASE_LEGACY_NOTPRESENT) or (not is_base_present):
+                self.has_rom_table = False
+                self.rom_addr = 0
+            elif is_adiv5_base and is_base_present:
+                self.has_rom_table = True
+                self.rom_addr = base & AP_BASE_BASEADDR_MASK # clear format and present bits
+            else:
+                raise exceptions.TargetError("invalid AP BASE value 0x%08x" % base)
+        
+        # Run the init tests.
+        _init_cfg()
+        _init_transfer_sizes()
+        _init_hprot()
+        _init_rom_table_base()
  
+        # Restore unmodified value of CSW.
+        AccessPort.write_reg(self, self._reg_offset + MEM_AP_CSW, original_csw)
+
     @locked
     def find_components(self):
         try:
             if self.has_rom_table:
+                if not self.is_enabled:
+                    LOG.warning("Skipping CoreSight discovery for %s because it is disabled", self.short_description)
+                    return
+                
                 # Import locally to work around circular import.
                 from .rom_table import (CoreSightComponentID, ROMTable)
                 
@@ -654,7 +745,7 @@ class MEM_AP(AccessPort, memory_interface.MemoryInterface):
                     self.rom_table.init()
         except exceptions.TransferError as error:
             LOG.error("Transfer error while reading %s ROM table: %s", self.short_description, error,
-                exc_info=self.dp.target.session.log_tracebacks)
+                exc_info=self.dp.session.log_tracebacks)
 
     @property
     def implemented_hprot_mask(self):
@@ -784,13 +875,17 @@ class MEM_AP(AccessPort, memory_interface.MemoryInterface):
         except exceptions.ProbeError:
             # Invalidate cached CSW on exception.
             if ap_regaddr == self._reg_offset + MEM_AP_CSW:
-                self._cached_csw = -1
+                self._invalidate_cache()
             raise
     
-    def reset_did_occur(self):
-        """! @copydoc AccessPort.reset_did_occur()"""
-        # TODO use notifications to invalidate CSW cache.
+    def _invalidate_cache(self):
+        """! @brief Invalidate cached registers associated with this AP."""
         self._cached_csw = -1
+    
+    def _reset_did_occur(self, notification):
+        """! @brief Handles reset notifications to invalidate CSW cache."""
+        # We clear the cache on all resets just to be safe.
+        self._invalidate_cache()
 
     @locked
     def _write_memory(self, addr, data, transfer_size=32):
@@ -813,10 +908,18 @@ class MEM_AP(AccessPort, memory_interface.MemoryInterface):
             data = data << ((addr & 0x03) << 3)
         elif transfer_size == 16:
             data = data << ((addr & 0x02) << 3)
+        elif transfer_size > 32:
+            # Split the value into a tuple of 32-bit words, least-significant first.
+            data = (((data >> (32 * i)) & 0xffffffff) for i in range(transfer_size // 32))
 
         try:
             self.write_reg(self._reg_offset + MEM_AP_TAR, addr)
-            self.write_reg(self._reg_offset + MEM_AP_DRW, data)
+            
+            if transfer_size <= 32:
+                self.write_reg(self._reg_offset + MEM_AP_DRW, data)
+            else:
+                # Multi-word transfer.
+                self.dp.write_ap_multiple(self.address.address + self._reg_offset + MEM_AP_DRW, data)
         except exceptions.TransferFaultError as error:
             # Annotate error with target address.
             self._handle_error(error, num)
@@ -844,11 +947,16 @@ class MEM_AP(AccessPort, memory_interface.MemoryInterface):
         num = self.dp.next_access_number
         TRACE.debug("read_mem:%06d (ap=0x%x; addr=0x%08x, size=%d) {",
             num, self.address.nominal_address, addr, transfer_size)
-        res = None
         try:
             self.write_reg(self._reg_offset + MEM_AP_CSW, self._csw | TRANSFER_SIZE[transfer_size])
             self.write_reg(self._reg_offset + MEM_AP_TAR, addr)
-            result_cb = self.read_reg(self._reg_offset + MEM_AP_DRW, now=False)
+            
+            if transfer_size <= 32:
+                result_cb = self.read_reg(self._reg_offset + MEM_AP_DRW, now=False)
+            else:
+                # Multi-word transfer.
+                result_cb = self.dp.read_ap_multiple(self.address.address + self._reg_offset + MEM_AP_DRW,
+                        transfer_size // 32, now=False)
         except exceptions.TransferFaultError as error:
             # Annotate error with target address.
             self._handle_error(error, num)
@@ -860,12 +968,15 @@ class MEM_AP(AccessPort, memory_interface.MemoryInterface):
             raise
 
         def read_mem_cb():
+            res = None
             try:
                 res = result_cb()
                 if transfer_size == 8:
                     res = (res >> ((addr & 0x03) << 3) & 0xff)
                 elif transfer_size == 16:
                     res = (res >> ((addr & 0x02) << 3) & 0xffff)
+                elif transfer_size > 32:
+                    res = sum((w << (32 * i)) for i, w in enumerate(res))
                 TRACE.debug("read_mem:%06d %s(ap=0x%x; addr=0x%08x, size=%d) -> 0x%08x }",
                     num, "" if now else "...", self.address.nominal_address, addr, transfer_size, res)
             except exceptions.TransferFaultError as error:
@@ -976,7 +1087,7 @@ class MEM_AP(AccessPort, memory_interface.MemoryInterface):
 
     def _handle_error(self, error, num):
         self.dp._handle_error(error, num)
-        self._cached_csw = -1
+        self._invalidate_cache()
 
 class AHB_AP(MEM_AP):
     """! @brief AHB-AP access port subclass.
@@ -1042,7 +1153,7 @@ AP_TYPE_AHB5_HPROT = 0x8
 
 # AP flags.
 AP_4K_WRAP = 0x1 # The AP has a 4 kB auto-increment modulus.
-AP_ALL_TX_SZ = 0x2 # The AP is known to support 8-, 16-, and 32-bit transfers.
+AP_ALL_TX_SZ = 0x2 # The AP is known to support 8-, 16-, and 32-bit transfers, *unless* Large Data is implemented.
 AP_MSTRTYPE = 0x4 # The AP is known to support the MSTRTYPE field.
 
 ## Map from AP IDR fields to AccessPort subclass.
