@@ -1,5 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2015-2020 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,14 +16,14 @@
 # limitations under the License.
 
 import logging
-import six
-from collections import namedtuple
 from enum import Enum
+from typing import NamedTuple
 
 from ..core import (exceptions, memory_interface)
+from ..core.target import Target
 from ..probe.debug_probe import DebugProbe
 from ..probe.swj import SWJSequenceSender
-from .ap import (MEM_AP_CSW, APSEL, APBANKSEL, APSEL_APBANKSEL, APREG_MASK, AccessPort)
+from .ap import APSEL_APBANKSEL
 from ..utility.sequencer import CallSequence
 from ..utility.timeout import Timeout
 
@@ -97,14 +98,19 @@ MASKLANE = 0x00000f00
 DP_POWER_REQUEST_TIMEOUT = 5.0
 
 ## @brief Class to hold fields from DP IDR register.
-DPIDR = namedtuple('DPIDR', 'idr partno version revision mindp')
+class DPIDR(NamedTuple):
+    idr: int
+    partno: int
+    version: int
+    revision: int
+    mindp: int
 
 class ADIVersion(Enum):
     """! @brief Supported versions of the Arm Debug Interface."""
     ADIv5 = 5
     ADIv6 = 6
 
-class DPConnector(object):
+class DPConnector:
     """! @brief Establishes a connection to the DP for a given wire protocol.
     
     This class will ask the probe to connect using a given wire protocol. Then it makes multiple
@@ -115,12 +121,25 @@ class DPConnector(object):
         self._probe = probe
         self._session = probe.session
         self._idr = None
+        
+        # Make sure we have a session, since we get the session from the probe and probes have their session set
+        # after creation.
+        assert self._session is not None, "DPConnector requires the probe to have a session"
     
     @property
     def idr(self):
         """! @brief DPIDR instance containing values read from the DP IDR register."""
         return self._idr
     
+    def _get_protocol(self, protocol):
+        # Convert protocol from setting if not passed as parameter.
+        if protocol is None:
+            protocol_name = self._session.options.get('dap_protocol').strip().lower()
+            protocol = DebugProbe.PROTOCOL_NAME_MAP[protocol_name]
+            if protocol not in self._probe.supported_wire_protocols:
+                raise exceptions.DebugError("requested wire protocol %s not supported by the debug probe" % protocol.name)
+        return protocol
+    
     def connect(self, protocol=None):
         """! @brief Establish a connection to the DP.
         
@@ -133,27 +152,52 @@ class DPConnector(object):
         @exception DebugError
         @exception TransferError
         """
-        protocol_name = self._session.options.get('dap_protocol').strip().lower()
-        send_swj = self._session.options.get('dap_swj_enable') \
-                and (DebugProbe.Capability.SWJ_SEQUENCE in self._probe.capabilities)
-        use_dormant = self._session.options.get('dap_swj_use_dormant')
+        try:
+            self._probe.lock()
 
-        # Convert protocol from setting if not passed as parameter.
-        if protocol is None:
-            protocol = DebugProbe.PROTOCOL_NAME_MAP[protocol_name]
-            if protocol not in self._probe.supported_wire_protocols:
-                raise exceptions.DebugError("requested wire protocol %s not supported by the debug probe" % protocol.name)
-        if protocol != DebugProbe.Protocol.DEFAULT:
+            # Determine the requested wire protocol.
+            protocol = self._get_protocol(protocol)
+
+            # If this is not None then the probe is already connected.
+            current_wire_protocol = self._probe.wire_protocol
+            already_connected = current_wire_protocol is not None
+        
+            if already_connected:
+                self._check_protocol(current_wire_protocol, protocol)
+            else:
+                self._connect_probe(protocol)
+
+            protocol = self._probe.wire_protocol
+            self._connect_dp(protocol)
+        finally:
+            self._probe.unlock()
+    
+    def _check_protocol(self, current_wire_protocol, protocol):
+        # Warn about mismatched current and requested wire protocols.
+        if (protocol is not current_wire_protocol) and (protocol is not DebugProbe.Protocol.DEFAULT):
+            LOG.warning("Cannot use %s; already connected with %s", protocol.name, current_wire_protocol.name)
+        else:
+            LOG.debug("Already connected with %s", current_wire_protocol.name)
+
+    def _connect_probe(self, protocol):
+        # Debug log with the selected protocol.
+        if protocol is not DebugProbe.Protocol.DEFAULT:
             LOG.debug("Using %s wire protocol", protocol.name)
-            
+        
         # Connect using the selected protocol.
         self._probe.connect(protocol)
 
         # Log the actual protocol if selected was default.
-        if protocol == DebugProbe.Protocol.DEFAULT:
+        if protocol is DebugProbe.Protocol.DEFAULT:
             protocol = self._probe.wire_protocol
             LOG.debug("Default wire protocol selected; using %s", protocol.name)
         
+    def _connect_dp(self, protocol):
+        # Get SWJ settings.
+        use_dormant = self._session.options.get('dap_swj_use_dormant')
+        send_swj = self._session.options.get('dap_swj_enable') \
+                and (DebugProbe.Capability.SWJ_SEQUENCE in self._probe.capabilities)
+
         # Create object to send SWJ sequences.
         swj = SWJSequenceSender(self._probe, use_dormant)
         
@@ -198,12 +242,26 @@ class DPConnector(object):
         is_mindp = (dpidr & DPIDR_MIN_MASK) != 0
         return DPIDR(dpidr, dp_partno, dp_version, dp_revision, is_mindp)
 
-class DebugPort(object):
+class DebugPort:
     """! @brief Represents the Arm Debug Interface (ADI) Debug Port (DP)."""
+
+    ## Sleep for 50 ms between connection tests and reconnect attempts after a reset.
+    _RESET_RECOVERY_SLEEP_INTERVAL = 0.05
+
+    ## Number of times to try to read DP registers after hw reset before attempting reconnect.
+    _RESET_RECOVERY_ATTEMPTS_BEFORE_RECONNECT = 1
     
     def __init__(self, probe, target):
+        """! @brief Constructor.
+        @param self The DebugPort object.
+        @param probe The @ref pyocd.probe.debug_probe.DebugProbe "DebugProbe" object. The probe is assumed to not
+            have been opened yet.
+        @param target An instance of @ref pyocd.core.soc_target.SoCTarget "SoCTarget". Assumed to not have been
+            fully initialized.
+        """
         self._probe = probe
         self.target = target
+        self._session = target.session
         self.valid_aps = None
         self.dpidr = None
         self.aps = {}
@@ -213,6 +271,10 @@ class DebugPort(object):
         self._probe_managed_ap_select = False
         self._probe_managed_dpbanksel = False
         self._probe_supports_dpbanksel = False
+        self._probe_supports_apv2_addresses = False
+        self._have_probe_capabilities = False
+        self._did_check_version = False
+        self._log_dp_info = True
         
         # DPv3 attributes
         self._is_dpv3 = False
@@ -221,11 +283,18 @@ class DebugPort(object):
         self._errmode = None
         self._base_addr = None
         self._apacc_mem_interface = None
+        
+        # Subscribe to reset events.
+        self._session.subscribe(self._reset_did_occur, (Target.Event.PRE_RESET, Target.Event.POST_RESET))
 
     @property
     def probe(self):
         return self._probe
     
+    @property
+    def session(self):
+        return self._session
+    
     @property
     def adi_version(self):
         return ADIVersion.ADIv6 if self._is_dpv3 else ADIVersion.ADIv5
@@ -255,24 +324,31 @@ class DebugPort(object):
         """! @brief Unlock the DP."""
         self.probe.unlock()
 
-    def init(self, protocol=None):
+    def connect(self, protocol=None):
         """! @brief Connect to the target.
         
         This method causes the debug probe to connect using the selected wire protocol. The probe
         must have already been opened prior to this call.
         
-        Unlike init_sequence(), this method is intended to be used when manually constructing a
-        DebugPort instance. It simply calls init_sequence() and invokes the returned call sequence.
+        Unlike create_connect_sequence(), this method is intended to be used when manually constructing a
+        DebugPort instance. It simply calls create_connect_sequence() and invokes the returned call sequence.
         
         @param self
         @param protocol One of the @ref pyocd.probe.debug_probe.DebugProbe.Protocol
             "DebugProbe.Protocol" enums. If not provided, will default to the `protocol` setting.
         """
         self._protocol = protocol
-        self.init_sequence().invoke()
+        self.create_connect_sequence().invoke()
+    
+    def disconnect(self):
+        """! @brief Disconnect from target.
+        
+        DP debug is powered down. See power_down_debug().
+        """
+        self.power_down_debug()
 
-    def init_sequence(self):
-        """! @brief Init task to connect to the target.
+    def create_connect_sequence(self):
+        """! @brief Returns call sequence to connect to the target.
         
         Returns a @ref pyocd.utility.sequence.CallSequence CallSequence that will connect to the
         DP, power up debug and the system, check the DP version to identify whether the target uses
@@ -281,14 +357,28 @@ class DebugPort(object):
         The probe must have already been opened prior to this method being called.
         
         @param self
+        @return @ref pyocd.utility.sequence.CallSequence CallSequence
         """
-        return CallSequence(
-            ('get_probe_capabilities', self._get_probe_capabilities),
+        seq = [
+            ('lock_probe',          self.probe.lock),
+            ]
+        if not self._have_probe_capabilities:
+            seq += [
+                ('get_probe_capabilities', self._get_probe_capabilities),
+                ]
+        seq += [
             ('connect',             self._connect),
             ('clear_sticky_err',    self.clear_sticky_err),
             ('power_up_debug',      self.power_up_debug),
-            ('check_version',       self._check_version),
-            )
+            ]
+        if not self._did_check_version:
+            seq += [
+                ('check_version',       self._check_version),
+                ]
+        seq += [
+            ('unlock_probe',        self.probe.unlock),
+            ]
+        return CallSequence(*seq)
 
     def _get_probe_capabilities(self):
         """! @brief Examine the probe's capabilities."""
@@ -296,6 +386,8 @@ class DebugPort(object):
         self._probe_managed_ap_select = (DebugProbe.Capability.MANAGED_AP_SELECTION in caps)
         self._probe_managed_dpbanksel = (DebugProbe.Capability.MANAGED_DPBANKSEL in caps)
         self._probe_supports_dpbanksel = (DebugProbe.Capability.BANKED_DP_REGISTERS in caps)
+        self._probe_supports_apv2_addresses = (DebugProbe.Capability.APv2_ADDRESSES in caps)
+        self._have_probe_capabilities = True
 
     def _connect(self):
         # Attempt to connect.
@@ -304,12 +396,17 @@ class DebugPort(object):
 
         # Report on DP version.
         self.dpidr = connector.idr
-        LOG.info("DP IDR = 0x%08x (v%d%s rev%d)", self.dpidr.idr, self.dpidr.version,
+        LOG.log(logging.INFO if self._log_dp_info else logging.DEBUG,
+            "DP IDR = 0x%08x (v%d%s rev%d)", self.dpidr.idr, self.dpidr.version,
             " MINDP" if self.dpidr.mindp else "", self.dpidr.revision)
         
     def _check_version(self):
         self._is_dpv3 = (self.dpidr.version == 3)
         if self._is_dpv3:
+            # Check that the probe will be able to access ADIv6 APs.
+            if self._probe_managed_ap_select and not self._probe_supports_apv2_addresses:
+                raise exceptions.ProbeError("connected to ADIv6 target with probe that does not support APv2 addresses")
+            
             idr1 = self.read_reg(DP_IDR1)
             
             self._addr_size = idr1 & DPIDR1_ASIZE_MASK
@@ -334,6 +431,8 @@ class DebugPort(object):
             else:
                 LOG.warning("DPv3 has no valid base address")
 
+        self._did_check_version = True
+
     def flush(self):
         try:
             self.probe.flush()
@@ -365,8 +464,6 @@ class DebugPort(object):
                     break
             else:
                 return False
-
-        self.write_reg(DP_CTRL_STAT, CSYSPWRUPREQ | CDBGPWRUPREQ | MASKLANE | TRNNORMAL)
         
         return True
 
@@ -404,23 +501,114 @@ class DebugPort(object):
         
         return True
 
-    def reset(self):
+    def _invalidate_cache(self):
+        """! @brief Invalidate cached DP registers."""
         self._cached_dp_select = None
-        for ap in self.aps.values():
-            ap.reset_did_occur()
+    
+    def _reset_did_occur(self, notification):
+        """! @brief Handles reset notifications to invalidate register cache.
+        
+        The cache is cleared on all resets just to be safe. On most devices, warm resets do not reset
+        debug logic, but it does happen on some devices.
+        """
+        self._invalidate_cache()
+    
+    def post_reset_recovery(self):
+        """! @brief Wait for the target to recover from reset, with auto-reconnect if needed."""
+        # Check if we can access DP registers. If this times out, then reconnect the DP and retry.
+        with Timeout(self.session.options.get('reset.dap_recover.timeout'),
+                self._RESET_RECOVERY_SLEEP_INTERVAL) as time_out:
+            attempt = 0
+            while time_out.check():
+                try:
+                    # Try to read CTRL/STAT. If the power-up bits request are reset, then the DP
+                    # connection was not lost and we can just return.
+                    value = self.read_reg(DP_CTRL_STAT)
+                    if (value & (CSYSPWRUPREQ | CDBGPWRUPREQ)) == (CSYSPWRUPREQ | CDBGPWRUPREQ):
+                        return
+                except exceptions.TransferError:
+                    # Ignore errors caused by flushing.
+                    try:
+                        self.flush()
+                    except exceptions.TransferError:
+                        pass
+
+                if attempt == self._RESET_RECOVERY_ATTEMPTS_BEFORE_RECONNECT:
+                    LOG.info("DAP is not accessible after reset; attempting reconnect")
+                elif attempt > self._RESET_RECOVERY_ATTEMPTS_BEFORE_RECONNECT:
+                    # Try reconnect.
+                    try:
+                        self._log_dp_info = False
+                        self.connect()
+                    finally:
+                        self._log_dp_info = True
+
+                attempt += 1
+            else:
+                LOG.error("DAP is not accessible after reset followed by attempted reconnect")
+
+    def reset(self, *, send_notifications=True):
+        """! @brief Hardware reset.
+        
+        Pre- and post-reset notifications are sent.
+        
+        This method can be called before the DebugPort is connected.
+
+        @param self This object.
+        @param send_notifications Optional keyword-only parameter used by higher-level reset methods so they can
+            manage the sending of reset notifications themselves, in order to provide more context in the notification.
+        
+        @todo Should automatic recovery from a disconnected DAP be provided for these low-level hardware resets
+            like is done for CortexM.reset()?
+        """
+        if send_notifications:
+            self.session.notify(Target.Event.PRE_RESET, self)
+
         self.probe.reset()
+        self.post_reset_recovery()
+
+        if send_notifications:
+            self.session.notify(Target.Event.POST_RESET, self)
+
+    def assert_reset(self, asserted, *, send_notifications=True):
+        """! @brief Assert or deassert the hardware reset signal.
+        
+        A pre-reset notification is sent before asserting reset, whereas a post-reset notification is sent
+        after deasserting reset.
+        
+        This method can be called before the DebugPort is connected.
+        
+        @param self This object.
+        @param asserted True if nRESET is to be driven low; False will drive nRESET high.
+        @param send_notifications Optional keyword-only parameter used by higher-level reset methods so they can
+            manage the sending of reset notifications themselves, in order to provide more context in the notification.
+        """
+        is_asserted = False
+        if send_notifications:
+            is_asserted = self.is_reset_asserted()
+            if asserted and not is_asserted:
+                self.session.notify(Target.Event.PRE_RESET, self)
 
-    def assert_reset(self, asserted):
-        self._cached_dp_select = None
-        if asserted:
-            for ap in self.aps.values():
-                ap.reset_did_occur()
         self.probe.assert_reset(asserted)
 
+        if send_notifications and not asserted and is_asserted:
+            self.session.notify(Target.Event.POST_RESET, self)
+
     def is_reset_asserted(self):
+        """! @brief Returns the current state of the nRESET signal.
+        
+        This method can be called before the DebugPort is initalized.
+
+        @retval True Reset is asserted; nRESET is low.
+        @retval False Reset is not asserted; nRESET is high.
+        """
         return self.probe.is_reset_asserted()
 
     def set_clock(self, frequency):
+        """! @brief Change the wire protocol's clock frequency.
+        @param self This object.
+        @param frequency New wire protocol frequency in Hertz.
+        """
         self.probe.set_clock(frequency)
 
     def _write_dp_select(self, mask, value):
@@ -484,6 +672,8 @@ class DebugPort(object):
             return False
 
     def read_dp(self, addr, now=True):
+        if (addr & DPADDR_MASK) % 4 != 0:
+            raise ValueError("DP address must be word aligned")
         num = self.next_access_number
         
         # Update DPBANKSEL if required.
@@ -522,6 +712,8 @@ class DebugPort(object):
             return read_dp_cb
 
     def write_dp(self, addr, data):
+        if (addr & DPADDR_MASK) % 4 != 0:
+            raise ValueError("DP address must be word aligned")
         num = self.next_access_number
         
         # Update DPBANKSEL if required.
@@ -564,8 +756,9 @@ class DebugPort(object):
         return True
 
     def write_ap(self, addr, data):
-        assert type(addr) in (six.integer_types)
+        assert isinstance(addr, int)
         num = self.next_access_number
+        did_lock = False
 
         try:
             did_lock = self._select_ap(addr)
@@ -581,8 +774,9 @@ class DebugPort(object):
         return True
 
     def read_ap(self, addr, now=True):
-        assert type(addr) in (six.integer_types)
+        assert isinstance(addr, int)
         num = self.next_access_number
+        did_lock = False
 
         try:
             did_lock = self._select_ap(addr)
@@ -618,9 +812,10 @@ class DebugPort(object):
             return read_ap_cb
 
     def write_ap_multiple(self, addr, values):
-        assert type(addr) in (six.integer_types)
+        assert isinstance(addr, int)
         num = self.next_access_number
-        
+        did_lock = False
+
         try:
             did_lock = self._select_ap(addr)
             TRACE.debug("write_ap_multiple:%06d (addr=0x%08x) = (%i values)", num, addr, len(values))
@@ -633,8 +828,9 @@ class DebugPort(object):
                 self.unlock()
 
     def read_ap_multiple(self, addr, count=1, now=True):
-        assert type(addr) in (six.integer_types)
+        assert isinstance(addr, int)
         num = self.next_access_number
+        did_lock = False
         
         try:
             did_lock = self._select_ap(addr)
@@ -677,7 +873,7 @@ class DebugPort(object):
             self.write_reg(DP_ABORT, ABORT_DAPABORT)
 
     def clear_sticky_err(self):
-        self._cached_dp_select = None
+        self._invalidate_cache()
         mode = self.probe.wire_protocol
         if mode == DebugProbe.Protocol.SWD:
             self.write_reg(DP_ABORT, ABORT_ORUNERRCLR | ABORT_WDERRCLR | ABORT_STKERRCLR | ABORT_STKCMPCLR)
