@@ -1,5 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2015-2020 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,6 +16,7 @@
 # limitations under the License.
 
 import logging
+from time import sleep
 
 from ..core import exceptions
 from .component import CoreSightComponent
@@ -102,6 +104,7 @@ class CoreSightComponentID(object):
         self.devtype = 0
         self.devid = [0, 0, 0]
         self.name = ''
+        self.product_name = None
         self.factory = None
         self.valid = False
 
@@ -150,8 +153,13 @@ class CoreSightComponentID(object):
         else:
             key = (self.designer, self.component_class, self.part, self.devtype, self.archid)
             info = COMPONENT_MAP.get(key, None)
+            if info is None:
+                # Try just the archid with no partno or devtype as backup.
+                key = (self.designer, self.component_class, None, None, self.archid)
+                info = COMPONENT_MAP.get(key, None)
             if info is not None:
                 self.name = info.name
+                self.product_name = info.product
                 self.factory = info.factory
             else:
                 self.name = '???'
@@ -166,19 +174,22 @@ class CoreSightComponentID(object):
         return result
 
     def __repr__(self):
+        name = self.name
+        if self.product_name:
+            name += " " + self.product_name
         if not self.valid:
-            return "<%08x:%s cidr=%x, pidr=%x, component invalid>" % (self.address, self.name, self.cidr, self.pidr)
+            return "<%08x:%s cidr=%x, pidr=%x, component invalid>" % (self.address, name, self.cidr, self.pidr)
         if self.power_id is not None:
             pwrid = " pwrid=%d" % self.power_id
         else:
             pwrid = ""
         if self.component_class == self.CORESIGHT_CLASS:
             return "<%08x:%s class=%d designer=%03x part=%03x devtype=%02x archid=%04x devid=%x:%x:%x%s>" % (
-                self.address, self.name, self.component_class, self.designer, self.part,
+                self.address, name, self.component_class, self.designer, self.part,
                 self.devtype, self.archid, self.devid[0], self.devid[1], self.devid[2], pwrid)
         else:
             return "<%08x:%s class=%d designer=%03x part=%03x%s>" % (
-                self.address, self.name,self.component_class, self.designer, self.part, pwrid)
+                self.address, name,self.component_class, self.designer, self.part, pwrid)
 
 
 class ROMTable(CoreSightComponent):
@@ -350,7 +361,7 @@ class Class1ROMTable(ROMTable):
                 except exceptions.TransferError as err:
                     LOG.error("Error attempting to probe CoreSight component referenced by "
                             "ROM table entry #%d: %s", entryNumber, err,
-                            exc_info=self.session.get_current().log_tracebacks)
+                            exc_info=self.ap.dp.session.get_current().log_tracebacks)
 
                 entryAddress += 4
                 entryNumber += 1
@@ -383,7 +394,10 @@ class Class1ROMTable(ROMTable):
         if (entry & self.ROM_TABLE_ADDR_OFFSET_NEG_MASK) != 0:
             offset = ~bit_invert(offset)
         address = self.address + offset
-        
+        # Handle address going negative, since python doesn't have unsigned ints.
+        if address < 0:
+            address = 0x100000000 + address
+
         # Check power ID.
         if (entry & self.ROM_TABLE_POWERIDVALID_MASK) != 0:
             powerid = (entry & self.ROM_TABLE_POWERID_MASK) >> self.ROM_TABLE_POWERID_SHIFT
@@ -539,7 +553,7 @@ class Class9ROMTable(ROMTable):
                     except exceptions.TransferError as err:
                         LOG.error("Error attempting to probe CoreSight component referenced by "
                                 "ROM table entry #%d: %s", entryNumber, err,
-                                exc_info=self.session.get_current().log_tracebacks)
+                                exc_info=self.ap.dp.session.get_current().log_tracebacks)
 
                 entryAddress += 4 * entrySizeMultiplier
                 entryNumber += 1
@@ -571,7 +585,10 @@ class Class9ROMTable(ROMTable):
         if (entry & self.ROM_TABLE_ADDR_OFFSET_NEG_MASK[self._width]) != 0:
             offset = ~bit_invert(offset, width=self._width)
         address = self.address + offset
-        
+        # Handle address going negative, since python doesn't have unsigned ints.
+        if address < 0:
+            address = (1 << self._width) + address
+
         # Check power ID.
         if (entry & self.ROM_TABLE_ENTRY_POWERIDVALID_MASK) != 0:
             powerid = (entry & self.ROM_TABLE_ENTRY_POWERID_MASK) >> self.ROM_TABLE_ENTRY_POWERID_SHIFT
