@@ -1,5 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2020 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,13 +16,11 @@
 # limitations under the License.
 
 import logging
-from time import sleep
 from enum import Enum
 
 from .component import CoreSightComponent
 from ..core import exceptions
 from ..utility.timeout import Timeout
-from ..utility.hex import dump_hex_data
 
 LOG = logging.getLogger(__name__)
 
@@ -58,7 +57,7 @@ class SDC600(CoreSightComponent):
         """! @brief COM Port link phases."""
         ## Hardware-defined link phase.
         PHASE1 = 1
-        ## Software-defiend link phase.
+        ## Software-defined link phase.
         PHASE2 = 2
     
     class Register:
@@ -257,7 +256,7 @@ class SDC600(CoreSightComponent):
             raise LinkError()
         # Catch reserved flags.
         elif (0xA2 <= value <= 0xA5) or (0xB0 <= value <= 0xBF):
-            raise UnexpectedFlagError("received reserved flag value ({:#04x})".format(flag))
+            raise UnexpectedFlagError("received reserved flag value ({:#04x})".format(value))
 
     def _expect_flag(self, flag, to_):
         """! @brief Read a byte and compare to expected value.
@@ -402,7 +401,7 @@ class SDC600(CoreSightComponent):
         """
         with Timeout(timeout) as to_:
             if phase == self.LinkPhase.PHASE1:
-                assert self._current_link_phase == None
+                assert self._current_link_phase is None
 
                 # Close link phase 1 first, to put it in a known state.
                 self.close_link(self.LinkPhase.PHASE1)
