@@ -1,5 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2006-2020 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,19 +18,17 @@
 import logging
 import threading
 from struct import unpack
-from time import (sleep, time)
+from time import sleep
 import sys
 import six
-from xml.etree.ElementTree import (Element, SubElement, tostring)
 import io
+from xml.etree.ElementTree import (Element, SubElement, tostring)
 
 from ..core import exceptions
 from ..core.target import Target
 from ..flash.loader import FlashLoader
-from ..flash.eraser import FlashEraser
 from ..utility.cmdline import convert_vector_catch
 from ..utility.conversion import (hex_to_byte_list, hex_encode, hex_decode, hex8_to_u32le)
-from ..utility.progress import print_progress
 from ..utility.compatibility import (iter_single_bytes, to_bytes_safe, to_str_safe)
 from ..utility.server import StreamServer
 from ..trace.swv import SWVReader
@@ -40,15 +39,14 @@ from .context_facade import GDBDebugContextFacade
 from .symbols import GDBSymbolProvider
 from ..rtos import RTOS
 from . import signals
+from . import gdbserver_commands # lgtm[py/unused-import]
 from .packet_io import (
-    CTRL_C,
     checksum,
     ConnectionClosedException,
     GDBServerPacketIOThread,
     )
 from ..commands.execution_context import CommandExecutionContext
 from ..commands.commander import ToolExitException
-from ..commands.base import CommandBase
 
 LOG = logging.getLogger(__name__)
 
@@ -101,7 +99,14 @@ class GDBServer(threading.Thread):
     This class start a GDB server listening a gdb connection on a specific port.
     It implements the RSP (Remote Serial Protocol).
     """
-    def __init__(self, session, core=None, server_listening_callback=None):
+
+    ## Notification event for the gdbserver beginnning to listen on its RSP port.
+    GDBSERVER_START_LISTENING_EVENT = 'gdbserver-start-listening'
+    
+    ## Timer delay for sending the notification that the server is listening.
+    START_LISTENING_NOTIFY_DELAY = 0.03 # 30 ms
+    
+    def __init__(self, session, core=None):
         super(GDBServer, self).__init__()
         self.session = session
         self.board = session.board
@@ -139,7 +144,6 @@ class GDBServer(threading.Thread):
                 'report_core_number',
                 ])
 
-        self.server_listening_callback = server_listening_callback
         self.packet_size = 2048
         self.packet_io = None
         self.gdb_features = []
@@ -160,8 +164,9 @@ class GDBServer(threading.Thread):
         self.first_run_after_reset_or_flash = True
 
         self.abstract_socket = ListenerSocket(self.port, self.packet_size)
-        if self.serve_local_only:
-            self.abstract_socket.host = 'localhost'
+        if not self.serve_local_only:
+            # We really should be binding to explicit interfaces, not all available.
+            self.abstract_socket.host = ''
         self.abstract_socket.init()
         # Read back bound port in case auto-assigned (port 0)
         self.port = self.abstract_socket.port
@@ -260,16 +265,15 @@ class GDBServer(threading.Thread):
         """! @brief Initialize the remote command processor infrastructure."""
         # Create command execution context. The output stream will default to stdout
         # but we'll change it to a fresh StringIO prior to running each command.
+        #
+        # Note we also modify the selected_core property so it is initially set to the gdbserver's core.
         self._command_context = CommandExecutionContext()
+        self._command_context.selected_core = self.target
         self._command_context.attach_session(self.session)
         
         # Add the gdbserver command group.
         self._command_context.command_set.add_command_group('gdbserver')
 
-    def restart(self):
-        if self.is_alive():
-            self.detach_event.set()
-
     def stop(self):
         if self.is_alive():
             self.shutdown_event.set()
@@ -301,17 +305,18 @@ class GDBServer(threading.Thread):
 
     def run(self):
         LOG.info('GDB server started on port %d (core %d)', self.port, self.core)
-        
-        # Make sure the target is halted. Otherwise gdb gets easily confused.
-        self.target.halt()
 
         while True:
             try:
                 self.detach_event.clear()
 
-                # Inform callback that the server is running.
-                if self.server_listening_callback:
-                    self.server_listening_callback(self)
+                # Notify listeners that the server is running after a short delay.
+                #
+                # This timer prevents a race condition where the notification is sent before the server is
+                # actually listening. It's not a 100% guarantee, though.
+                notify_timer = threading.Timer(self.START_LISTENING_NOTIFY_DELAY, self.session.notify,
+                        args=(self.GDBSERVER_START_LISTENING_EVENT, self))
+                notify_timer.start()
 
                 while not self.shutdown_event.isSet() and not self.detach_event.isSet():
                     connected = self.abstract_socket.connect()
@@ -325,10 +330,13 @@ class GDBServer(threading.Thread):
 
                 if self.detach_event.isSet():
                     continue
+        
+                # Make sure the target is halted. Otherwise gdb gets easily confused.
+                self.target.halt()
 
-                LOG.info("One client connected!")
+                LOG.info("Client connected to port %d!", self.port)
                 self._run_connection()
-                LOG.info("Client disconnected!")
+                LOG.info("Client disconnected from port %d!", self.port)
                 self._cleanup_for_next_connection()
 
             except Exception as e:
@@ -567,7 +575,7 @@ class GDBServer(threading.Thread):
         return addr
 
     def resume(self, data):
-        addr = self._get_resume_step_addr(data)
+#         addr = self._get_resume_step_addr(data)
         self.target.resume()
         LOG.debug("target resumed")
 
@@ -613,7 +621,7 @@ class GDBServer(threading.Thread):
             except exceptions.Error as e:
                 try:
                     self.target.halt()
-                except:
+                except exceptions.Error:
                     pass
                 LOG.warning('Exception while target was running: %s', e, exc_info=self.session.log_tracebacks)
                 val = ('S%02x' % self.target_facade.get_signal_value()).encode()
@@ -622,7 +630,7 @@ class GDBServer(threading.Thread):
         return self.create_rsp_packet(val)
 
     def step(self, data, start=0, end=0):
-        addr = self._get_resume_step_addr(data)
+        #addr = self._get_resume_step_addr(data)
         LOG.debug("GDB step: %s (start=0x%x, end=0x%x)", data, start, end)
         
         # Use the step hook to check for an interrupt event.
@@ -999,7 +1007,7 @@ class GDBServer(threading.Thread):
         LOG.debug('Remote command: %s', cmd)
 
         # Create a new stream to collect the command output.
-        stream = six.StringIO()
+        stream = io.StringIO()
         self._command_context.output_stream = stream
         
         # TODO run this in a separate thread so we can cancel the command with ^C from gdb?
@@ -1204,65 +1212,3 @@ class GDBServer(threading.Thread):
         elif notification.event == 'report_core_number':
             self.report_core = notification.data.new_value
 
-class ThreadsCommand(CommandBase):
-    INFO = {
-            'names': ['threads'],
-            'group': 'gdbserver',
-            'category': 'threads',
-            'nargs': 1,
-            'usage': "{flush,enable,disable,status}",
-            'help': "Control thread awareness.",
-            }
-    
-    def parse(self, args):
-        self.action = args[0]
-        if self.action not in ('flush', 'enable', 'disable', 'status'):
-            raise exceptions.CommandError("invalid action")
-    
-    def execute(self):
-        # Get the gdbserver for the selected core.
-        core_number = self.context.target.selected_core.core_number
-        try:
-            gdbserver = self.context.session.gdbservers[core_number]
-        except KeyError:
-            raise exceptions.CommandError("no gdbserver for core #%i" % core_number)
-
-        if gdbserver.thread_provider is None:
-            self.context.write("Threads are unavailable")
-            return
-
-        if self.action == 'flush':
-            gdbserver.thread_provider.invalidate()
-            self.context.write("Threads flushed")
-        elif self.action == 'enable':
-            gdbserver.thread_provider.read_from_target = True
-            self.context.write("Threads enabled")
-        elif self.action == 'disable':
-            gdbserver.thread_provider.read_from_target = False
-            self.context.write("Threads disabled")
-        elif self.action == 'status':
-            self.context.write("Threads are " +
-                    ("enabled" if gdbserver.thread_provider.read_from_target else "disabled"))
-
-class ArmSemihostingCommand(CommandBase):
-    INFO = {
-            'names': ['arm'],
-            'group': 'gdbserver',
-            'category': 'semihosting',
-            'nargs': 2,
-            'usage': "semihosting {enable,disable}",
-            'help': "Enable or disable semihosting.",
-            'extra_help': "Provided for compatibility with OpenOCD. The same functionality can be achieved "
-                            "by setting the 'enable_semihosting' session option.",
-            }
-    
-    def parse(self, args):
-        if args[0] != 'semihosting':
-            raise exceptions.CommandError("invalid action")
-        if args[1] not in ('enable', 'disable'):
-            raise exceptions.CommandError("invalid action")
-        self.action = args[1]
-    
-    def execute(self):
-        enable = (self.action == 'enable')
-        self.context.session.options['enable_semihosting'] = enable
