@@ -1,5 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2018-2020 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,8 +15,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import pkg_resources
-
 from ..core import exceptions
 from ..core.plugin import load_plugin_classes_of_type
 from .debug_probe import DebugProbe
@@ -51,6 +50,15 @@ class DebugProbeAggregator(object):
         klasses, unique_id, is_explicit = DebugProbeAggregator._get_probe_classes(unique_id)
         
         probes = []
+        
+        # First look for a match against the full ID, as this can be more efficient for certain probes.
+        if unique_id is not None:
+            for cls in klasses:
+                probe = cls.get_probe_with_id(unique_id, is_explicit)
+                if probe is not None:
+                    return [probe]
+        
+        # No full match, so ask probe classes for probes.
         for cls in klasses:
             probes += cls.get_all_connected_probes(unique_id, is_explicit)
         
@@ -69,8 +77,7 @@ class DebugProbeAggregator(object):
             probe = cls.get_probe_with_id(unique_id, is_explicit)
             if probe is not None:
                 return probe
-        else:
-            return None
+        return None
 
 # Load plugins when this module is loaded.
 load_plugin_classes_of_type('pyocd.probe', PROBE_CLASSES, DebugProbe)
