@@ -1,5 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2018-2020 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,8 +15,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import six
 from time import sleep
+import logging
+from typing import (Dict, Tuple)
 
 from .debug_probe import DebugProbe
 from ..core import exceptions
@@ -24,6 +26,10 @@ from .pydapaccess import DAPAccess
 from ..board.mbed_board import MbedBoard
 from ..board.board_ids import BOARD_ID_TO_INFO
 
+LOG = logging.getLogger(__name__)
+TRACE = LOG.getChild("trace")
+TRACE.setLevel(logging.CRITICAL)
+
 class CMSISDAPProbe(DebugProbe):
     """! @brief Wraps a pydapaccess link as a DebugProbe.
     
@@ -35,8 +41,11 @@ class CMSISDAPProbe(DebugProbe):
     JTAG_CAPABILITY_MASK = 2
 
     # Map from DebugProbe protocol types to/from DAPAccess port types.
+    #
+    # Note that Protocol.DEFAULT gets mapped to PORT.SWD. We need a concrete port type because some
+    # non-reference CMSIS-DAP implementations do not accept the default port type.
     PORT_MAP = {
-        DebugProbe.Protocol.DEFAULT: DAPAccess.PORT.DEFAULT,
+        DebugProbe.Protocol.DEFAULT: DAPAccess.PORT.SWD,
         DebugProbe.Protocol.SWD: DAPAccess.PORT.SWD,
         DebugProbe.Protocol.JTAG: DAPAccess.PORT.JTAG,
         DAPAccess.PORT.DEFAULT: DebugProbe.Protocol.DEFAULT,
@@ -52,7 +61,7 @@ class CMSISDAPProbe(DebugProbe):
     A32 = 0x0000000c
     
     # Map from AP/DP and 2-bit register address to the enums used by pydapaccess.
-    REG_ADDR_TO_ID_MAP = {
+    REG_ADDR_TO_ID_MAP: Dict[Tuple[int, int], DAPAccess.REG] = {
         # APnDP A32
         ( 0,    0x0 ) : DAPAccess.REG.DP_0x0,
         ( 0,    0x4 ) : DAPAccess.REG.DP_0x4,
@@ -72,7 +81,7 @@ class CMSISDAPProbe(DebugProbe):
         try:
             return [cls(dev) for dev in DAPAccess.get_connected_devices()]
         except DAPAccess.Error as exc:
-            six.raise_from(cls._convert_exception(exc), exc)
+            raise cls._convert_exception(exc) from exc
     
     @classmethod
     def get_probe_with_id(cls, unique_id, is_explicit=False):
@@ -83,7 +92,7 @@ class CMSISDAPProbe(DebugProbe):
             else:
                 return None
         except DAPAccess.Error as exc:
-            six.raise_from(cls._convert_exception(exc), exc)
+            raise cls._convert_exception(exc) from exc
 
     def __init__(self, device):
         super(CMSISDAPProbe, self).__init__()
@@ -93,11 +102,26 @@ class CMSISDAPProbe(DebugProbe):
         self._is_open = False
         self._caps = set()
     
+    @property
+    def board_id(self):
+        """! @brief Unique identifier for the board.
+        
+        Only board IDs for DAPLink firmware are supported. We can't assume other
+        CMSIS-DAP firmware is using the same serial number format, so we cannot reliably
+        extract the board ID.
+        
+        @return Either a 4-character board ID string, or None if the probe doesn't have a board ID.
+        """
+        if self._link.vidpid == self.DAPLINK_VIDPID:
+            return self.unique_id[0:4]
+        else:
+            return None
+
     @property
     def description(self):
         try:
-            board_id = self.unique_id[0:4]
-            board_info = BOARD_ID_TO_INFO[board_id]
+            # self.board_id may be None.
+            board_info = BOARD_ID_TO_INFO[self.board_id]
         except KeyError:
             return self.vendor_name + " " + self.product_name
         else:
@@ -138,13 +162,15 @@ class CMSISDAPProbe(DebugProbe):
         # Only support associated Mbed boards for DAPLink firmware. We can't assume other
         # CMSIS-DAP firmware is using the same serial number format, so we cannot reliably
         # extract the board ID.
-        if self._link.vidpid == self.DAPLINK_VIDPID:
-            return MbedBoard(self.session, board_id=self.unique_id[0:4])
+        if self.board_id is not None:
+            return MbedBoard(self.session, board_id=self.board_id)
         else:
             return None
     
     def open(self):
         try:
+            TRACE.debug("trace: open")
+            
             self._link.open()
             self._is_open = True
             self._link.set_deferred_transfer(self.session.options.get('cmsis_dap.deferred_transfers'))
@@ -161,83 +187,123 @@ class CMSISDAPProbe(DebugProbe):
                 self.Capability.SWJ_SEQUENCE,
                 self.Capability.BANKED_DP_REGISTERS,
                 self.Capability.APv2_ADDRESSES,
+                self.Capability.JTAG_SEQUENCE,
                 }
+            if self._link.has_swd_sequence:
+                self._caps.add(self.Capability.SWD_SEQUENCE)
             if self._link.has_swo():
                 self._caps.add(self.Capability.SWO)
         except DAPAccess.Error as exc:
-            six.raise_from(self._convert_exception(exc), exc)
+            raise self._convert_exception(exc) from exc
     
     def close(self):
         try:
+            TRACE.debug("trace: close")
+            
             self._link.close()
             self._is_open = False
         except DAPAccess.Error as exc:
-            six.raise_from(self._convert_exception(exc), exc)
+            raise self._convert_exception(exc) from exc
 
     # ------------------------------------------- #
     #          Target control functions
     # ------------------------------------------- #
     def connect(self, protocol=None):
+        TRACE.debug("trace: connect(%s)", protocol.name if (protocol is not None) else "None")
+        
         # Convert protocol to port enum.
-        if protocol is not None:
-            port = self.PORT_MAP[protocol]
-        else:
-            port = DAPAccess.PORT.DEFAULT
+        # 
+        # We must get a non-default port, since some CMSIS-DAP implementations do not accept the default
+        # port. Note that the conversion of the default port type is contained in the PORT_MAP dict so it 
+        # is one location.
+        port = self.PORT_MAP.get(protocol, self.PORT_MAP[DebugProbe.Protocol.DEFAULT])
+        assert port is not DAPAccess.PORT.DEFAULT
         
         try:
             self._link.connect(port)
         except DAPAccess.Error as exc:
-            six.raise_from(self._convert_exception(exc), exc)
+            raise self._convert_exception(exc) from exc
         
         # Read the current mode and save it.
         actualMode = self._link.get_swj_mode()
         self._protocol = self.PORT_MAP[actualMode]
 
     def swj_sequence(self, length, bits):
+        TRACE.debug("trace: swj_sequence(length=%i, bits=%x)", length, bits)
+        
         try:
             self._link.swj_sequence(length, bits)
         except DAPAccess.Error as exc:
-            six.raise_from(self._convert_exception(exc), exc)
+            raise self._convert_exception(exc) from exc
+
+    def swd_sequence(self, sequences):
+        TRACE.debug("trace: swd_sequence(sequences=%r)", sequences)
+        
+        try:
+            self._link.swd_sequence(sequences)
+        except DAPAccess.Error as exc:
+            raise self._convert_exception(exc) from exc
+
+    def jtag_sequence(self, cycles, tms, read_tdo, tdi):
+        TRACE.debug("trace: jtag_sequence(cycles=%i, tms=%x, read_tdo=%s, tdi=%x)", cycles, tms, read_tdo, tdi)
+        
+        try:
+            self._link.jtag_sequence(cycles, tms, read_tdo, tdi)
+        except DAPAccess.Error as exc:
+            raise self._convert_exception(exc) from exc
 
     def disconnect(self):
+        TRACE.debug("trace: disconnect")
+        
         try:
             self._link.disconnect()
             self._protocol = None
         except DAPAccess.Error as exc:
-            six.raise_from(self._convert_exception(exc), exc)
+            raise self._convert_exception(exc) from exc
 
     def set_clock(self, frequency):
+        TRACE.debug("trace: set_clock(freq=%i)", frequency)
+        
         try:
             self._link.set_clock(frequency)
         except DAPAccess.Error as exc:
-            six.raise_from(self._convert_exception(exc), exc)
+            raise self._convert_exception(exc) from exc
 
     def reset(self):
+        TRACE.debug("trace: reset")
+        
         try:
             self._link.assert_reset(True)
             sleep(self.session.options.get('reset.hold_time'))
             self._link.assert_reset(False)
             sleep(self.session.options.get('reset.post_delay'))
         except DAPAccess.Error as exc:
-            six.raise_from(self._convert_exception(exc), exc)
+            raise self._convert_exception(exc) from exc
 
     def assert_reset(self, asserted):
+        TRACE.debug("trace: assert_reset(%s)", asserted)
+        
         try:
             self._link.assert_reset(asserted)
         except DAPAccess.Error as exc:
-            six.raise_from(self._convert_exception(exc), exc)
+            raise self._convert_exception(exc) from exc
     
     def is_reset_asserted(self):
         try:
-            return self._link.is_reset_asserted()
+            result = self._link.is_reset_asserted()
+            TRACE.debug("trace: is_reset_asserted -> %s", result)
+            return result
         except DAPAccess.Error as exc:
-            six.raise_from(self._convert_exception(exc), exc)
+            raise self._convert_exception(exc) from exc
 
     def flush(self):
+        TRACE.debug("trace: flush")
+        
         try:
             self._link.flush()
         except DAPAccess.Error as exc:
-            six.raise_from(self._convert_exception(exc), exc)
+            TRACE.debug("trace: error from flush: %r", exc)
+            raise self._convert_exception(exc) from exc
 
     # ------------------------------------------- #
     #          DAP Access functions
@@ -247,18 +313,28 @@ class CMSISDAPProbe(DebugProbe):
         reg_id = self.REG_ADDR_TO_ID_MAP[self.DP, addr]
         
         try:
+            if not now:
+                TRACE.debug("trace: read_dp(addr=%#010x) -> ...", addr)
             result = self._link.read_reg(reg_id, now=now)
         except DAPAccess.Error as error:
-            six.raise_from(self._convert_exception(error), error)
+            TRACE.debug("trace: read_dp(addr=%#010x) -> error(%s)", addr, error)
+            raise self._convert_exception(error) from error
 
         # Read callback returned for async reads.
         def read_dp_result_callback():
             try:
-                return result()
+                value = result()
+                TRACE.debug("trace: ... read_dp(addr=%#010x) -> %#010x", addr, value)
+                return value
             except DAPAccess.Error as error:
-                six.raise_from(self._convert_exception(error), error)
+                TRACE.debug("trace: ... read_dp(addr=%#010x) -> error(%s)", addr, error)
+                raise self._convert_exception(error) from error
 
-        return result if now else read_dp_result_callback
+        if now:
+            TRACE.debug("trace: read_dp(addr=%#010x) -> %#010x", addr, result)
+            return result
+        else:
+            return read_dp_result_callback
 
     def write_dp(self, addr, data):
         reg_id = self.REG_ADDR_TO_ID_MAP[self.DP, addr]
@@ -266,90 +342,125 @@ class CMSISDAPProbe(DebugProbe):
         # Write the DP register.
         try:
             self._link.write_reg(reg_id, data)
+            TRACE.debug("trace: write_dp(addr=%#010x, data=%#010x)", addr, data)
         except DAPAccess.Error as error:
-            six.raise_from(self._convert_exception(error), error)
+            TRACE.debug("trace: write_dp(addr=%#010x, data=%#010x) -> error(%s)", addr, data, error)
+            raise self._convert_exception(error) from error
 
         return True
 
     def read_ap(self, addr, now=True):
-        assert type(addr) in (six.integer_types)
+        assert isinstance(addr, int)
         ap_reg = self.REG_ADDR_TO_ID_MAP[self.AP, (addr & self.A32)]
 
         try:
+            if not now:
+                TRACE.debug("trace: read_ap(addr=%#010x) -> ...", addr)
             result = self._link.read_reg(ap_reg, now=now)
         except DAPAccess.Error as error:
-            six.raise_from(self._convert_exception(error), error)
+            raise self._convert_exception(error) from error
 
         # Read callback returned for async reads.
         def read_ap_result_callback():
             try:
-                return result()
+                value = result()
+                TRACE.debug("trace: ... read_ap(addr=%#010x) -> %#010x", addr, value)
+                return value
             except DAPAccess.Error as error:
-                six.raise_from(self._convert_exception(error), error)
+                TRACE.debug("trace: ... read_ap(addr=%#010x) -> error(%s)", addr, error)
+                raise self._convert_exception(error) from error
 
-        return result if now else read_ap_result_callback
+        if now:
+            TRACE.debug("trace: read_ap(addr=%#010x) -> %#010x", addr, result)
+            return result
+        else:
+            return read_ap_result_callback
 
     def write_ap(self, addr, data):
-        assert type(addr) in (six.integer_types)
+        assert isinstance(addr, int)
         ap_reg = self.REG_ADDR_TO_ID_MAP[self.AP, (addr & self.A32)]
 
         try:
             # Perform the AP register write.
             self._link.write_reg(ap_reg, data)
+            TRACE.debug("trace: write_ap(addr=%#010x, data=%#010x)", addr, data)
         except DAPAccess.Error as error:
-            six.raise_from(self._convert_exception(error), error)
+            TRACE.debug("trace: write_ap(addr=%#010x, data=%#010x) -> error(%s)", addr, data, error)
+            raise self._convert_exception(error) from error
 
         return True
 
     def read_ap_multiple(self, addr, count=1, now=True):
-        assert type(addr) in (six.integer_types)
+        assert isinstance(addr, int)
         ap_reg = self.REG_ADDR_TO_ID_MAP[self.AP, (addr & self.A32)]
         
         try:
+            if not now:
+                TRACE.debug("trace: read_ap_multi(addr=%#010x, count=%i) -> ...", addr, count)
             result = self._link.reg_read_repeat(count, ap_reg, dap_index=0, now=now)
         except DAPAccess.Error as exc:
-            six.raise_from(self._convert_exception(exc), exc)
+            raise self._convert_exception(exc) from exc
 
         # Need to wrap the deferred callback to convert exceptions.
         def read_ap_repeat_callback():
             try:
-                return result()
+                values = result()
+                TRACE.debug("trace: ... read_ap_multi(addr=%#010x, count=%i) -> [%s]", addr, count,
+                        ", ".join(["%#010x" % v for v in values]))
+                return values
             except DAPAccess.Error as exc:
-                six.raise_from(self._convert_exception(exc), exc)
+                TRACE.debug("trace: ... read_ap_multi(addr=%#010x, count=%i) -> error(%s)",
+                    addr, count, exc)
+                raise self._convert_exception(exc) from exc
 
-        return result if now else read_ap_repeat_callback
+        if now:
+            TRACE.debug("trace: read_ap_multi(addr=%#010x, count=%i) -> [%s]", addr, count,
+                    ", ".join(["%#010x" % v for v in result]))
+            return result
+        else:
+            return read_ap_repeat_callback
 
     def write_ap_multiple(self, addr, values):
-        assert type(addr) in (six.integer_types)
+        assert isinstance(addr, int)
         ap_reg = self.REG_ADDR_TO_ID_MAP[self.AP, (addr & self.A32)]
         
         try:
-            return self._link.reg_write_repeat(len(values), ap_reg, values, dap_index=0)
+            self._link.reg_write_repeat(len(values), ap_reg, values, dap_index=0)
+            TRACE.debug("trace: write_ap_multi(addr=%#010x, (%i)[%s])", addr, len(values),
+                    ", ".join(["%#010x" % v for v in values]))
         except DAPAccess.Error as exc:
-            six.raise_from(self._convert_exception(exc), exc)
+            TRACE.debug("trace: write_ap_multi(addr=%#010x, (%i)[%s]) -> error(%s)", addr, len(values),
+                    ", ".join(["%#010x" % v for v in values]), exc)
+            raise self._convert_exception(exc) from exc
     
     # ------------------------------------------- #
     #          SWO functions
     # ------------------------------------------- #
 
     def swo_start(self, baudrate):
+        TRACE.debug("trace: swo_start(baud=%i)", baudrate)
+        
         try:
             self._link.swo_configure(True, baudrate)
             self._link.swo_control(True)
         except DAPAccess.Error as exc:
-            six.raise_from(self._convert_exception(exc), exc)
+            raise self._convert_exception(exc) from exc
 
     def swo_stop(self):
+        TRACE.debug("trace: swo_stop")
+        
         try:
             self._link.swo_configure(False, 0)
         except DAPAccess.Error as exc:
-            six.raise_from(self._convert_exception(exc), exc)
+            raise self._convert_exception(exc) from exc
 
     def swo_read(self):
         try:
-            return self._link.swo_read()
+            data = self._link.swo_read()
+            TRACE.debug("trace: swo_read -> %i bytes", len(data))
+            return data
         except DAPAccess.Error as exc:
-            six.raise_from(self._convert_exception(exc), exc)
+            raise self._convert_exception(exc) from exc
 
     @staticmethod
     def _convert_exception(exc):
