@@ -1,5 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2018-2020 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,7 +19,40 @@ from enum import Enum
 import threading
 
 class DebugProbe(object):
-    """! @brief Abstract debug probe class."""
+    """! @brief Abstract debug probe class.
+    
+    Subclasses of this abstract class are drivers for different debug probe interfaces, either hardware such as a
+    USB based probe, or software such as connecting with a simulator.
+    
+    The constructor is private. To create an instance, use either of get_all_connected_probes() or get_probe_with_id().
+    Normally, the @ref pyocd.probe.aggregator.DebugProbeAggregator "DebugProbeAggregator" class is used instead of
+    directly calling methods on a specific probe class.
+    
+    Use an instance as follows:
+    
+    1. Call open().
+    2. Optionally inspect the `supported_wire_protocols` property and select a protocol to use.
+    3. Call connect(), passing the chosen wire protocol.
+    4. Use by instance by calling other methods.
+    5. Call disconnect().
+    6. Call close().
+    
+    Most methods are required to be overridden by a subclass, with a few exceptions.
+    
+    These methods are completely optional:
+    
+    - create_associated_board()
+    - flush()
+    - get_memory_interface_for_ap()
+    
+    These methods must be implemented depending on the probe capabilities, as returned from the `capabilities` property.
+    
+    - swj_sequence(): Capability.SWJ_SEQUENCE; if not provided it is assumed the probe automatically enables SWD or JTAG
+        on the target based on the protocol passed into connect().
+    - swd_sequence(): Capability.SWD_SEQUENCE
+    - jtag_sequence(): Capability.JTAG_SEQUENCE
+    - swo_*(): Capability.SWO
+    """
 
     class Protocol(Enum):
         """! @brief Debug wire protocols."""
@@ -45,9 +79,16 @@ class DebugProbe(object):
         SWO = 1
 
         ## @brief Whether the probe can access banked DP registers.
+        #
+        # Currently only used to verify that the probe supports banked DP registers when the #MANAGED_DPBANKSEL
+        # capability is present.
         BANKED_DP_REGISTERS = 2
         
         ## @brief Whether the probe can access APv2 registers.
+        #
+        # This capability is currently only used to verify that a probe with the #MANAGED_AP_SELECTION capability
+        # can support the wider AP addresses used in version 2 APs. For probes without #MANAGED_AP_SELECTION,
+        # DP_SELECT is written directly by the DAP layer when selecting an AP.
         APv2_ADDRESSES = 3
         
         ## @brief Whether the probe automatically handles AP selection in the DP.
@@ -58,15 +99,25 @@ class DebugProbe(object):
         
         ## @brief whether the probe automatically handles access of banked DAP registers.
         MANAGED_DPBANKSEL = 5
+
+        ## @brief Whether the probe supports the swd_sequence() API.
+        SWD_SEQUENCE = 6
+
+        ## @brief Whether the probe supports the jtag_sequence() API.
+        JTAG_SEQUENCE = 7
     
     @classmethod
     def get_all_connected_probes(cls, unique_id=None, is_explicit=False):
         """! @brief Returns a list of DebugProbe instances.
         
+        To filter the list of returned probes, the `unique_id` parameter may be set to a string with a full or
+        partial unique ID (canonically the serial number). Alternatively, the probe class may simply return all
+        available probes and let the caller handle filtering.
+        
         @param cls The class instance.
-        @param unique_id Optional unique ID value on which probes are being filtered. May be
-            used by the probe to optimize retreiving the probe list.
-        @param is_explicit Whether the probe type was explicitly specified in the unique ID. This
+        @param unique_id String. Optional partial unique ID value used to filter available probes. May be used by the
+            probe to optimize retrieving the probe list; there is no requirement to filter the results.
+        @param is_explicit Boolean. Whether the probe type was explicitly specified in the unique ID. This
             can be used, for instance, to specially interpret the unique ID as an IP address or
             domain name when the probe class was specifically requested but not for general lists
             of available probes.
@@ -78,12 +129,11 @@ class DebugProbe(object):
     def get_probe_with_id(cls, unique_id, is_explicit=False):
         """! @brief Returns a DebugProbe instance for a probe with the given unique ID.
         
-        If no probe is connected with a matching unique ID, then None will be returned.
+        If no probe is connected with a fully matching unique ID, then None will be returned.
         
         @param cls The class instance.
-        @param unique_id Optional unique ID value on which probes are being filtered. May be
-            used by the probe to optimize retreiving the probe list.
-        @param is_explicit Whether the probe type was explicitly specified in the unique ID.
+        @param unique_id Unique ID string to match against probes' full unique ID. No partial matches are allowed.
+        @param is_explicit Boolean. Whether the probe type was explicitly specified in the unique ID.
         @return DebugProbe instance, or None
         """
         raise NotImplementedError()
@@ -139,7 +189,8 @@ class DebugProbe(object):
         """! @brief Currently selected wire protocol.
         
         If the probe is not open and connected, i.e., open() and connect() have not been called,
-        then this property will be None.
+        then this property will be None. If a value other than None is returned, then the probe
+        has been connected successfully.
         """
         raise NotImplementedError()
     
@@ -185,6 +236,8 @@ class DebugProbe(object):
         
         This lock is recursive, so locking multiple times from a single thread is acceptable as long
         as the thread unlocks the same number of times.
+        
+        This method does not return until the calling thread has ownership of the lock.
         """
         self._lock.acquire()
     
@@ -216,6 +269,37 @@ class DebugProbe(object):
         """
         pass
 
+    def swd_sequence(self, sequences):
+        """! @brief Send a sequences of bits on the SWDIO signal.
+        
+        Each sequence in the _sequences_ parameter is a tuple with 1 or 2 members in this order:
+        - 0: int: number of TCK cycles from 1-64
+        - 1: int: the SWDIO bit values to transfer. The presence of this tuple member indicates the sequence is
+            an output sequence; the absence means that the specified number of TCK cycles of SWDIO data will be
+            read and returned.
+        
+        @param self
+        @param sequences A sequence of sequence description tuples as described above.
+        
+        @return A 2-tuple of the response status, and a sequence of bytes objects, one for each input
+            sequence. The length of the bytes object is (<TCK-count> + 7) / 8. Bits are in LSB first order.
+        """
+        raise NotImplementedError()
+
+    def jtag_sequence(self, cycles, tms, read_tdo, tdi):
+        """! @brief Send JTAG sequence.
+        
+        @param self
+        @param cycles Number of TCK cycles, from 1-64.
+        @param tms Fixed TMS value. Either 0 or 1.
+        @param read_tdo Boolean indicating whether TDO should be read.
+        @param tdi Integer with the TDI bit values to be transferred each TCK cycle. The LSB is
+            sent first.
+        
+        @return Either an integer with TDI bit values, or None, if _read_tdo_ was false.
+        """
+        raise NotImplementedError()
+
     def set_clock(self, frequency):
         """! @brief Set the frequency for JTAG and SWD in Hz.
 
@@ -231,7 +315,7 @@ class DebugProbe(object):
         """! @brief Assert or de-assert target's nRESET signal.
         
         Because nRESET is negative logic and usually open drain, passing True will drive it low, and
-        pasing False will stop driving so nRESET will be pulled up.
+        passing False will stop driving so nRESET will be pulled up.
         """
         raise NotImplementedError()
     
