@@ -1,5 +1,6 @@
 # pyOCD debugger
-# Copyright (c) 2006-2013,2018-2019 Arm Limited
+# Copyright (c) 2006-2013,2018-2021 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,7 +15,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import array
+from typing import (Set, Tuple)
+
 from .dap_access_api import DAPAccessIntf
 
 class Command:
@@ -68,11 +70,33 @@ class Pin:
 # Info IDs that return integer values.
 INTEGER_INFOS = [
     DAPAccessIntf.ID.CAPABILITIES,
+    DAPAccessIntf.ID.TEST_DOMAIN_TIMER,
     DAPAccessIntf.ID.SWO_BUFFER_SIZE,
     DAPAccessIntf.ID.MAX_PACKET_COUNT,
     DAPAccessIntf.ID.MAX_PACKET_SIZE
     ]
 
+class CMSISDAPVersion:
+    """! @brief Known CMSIS-DAP versions.
+
+    The tuple fields are major, minor, patch. Generally, patch release versions are excluded from this
+    list, unless there is a specific reason to know about a particular patch release.
+    """
+    V1_0_0 = (1, 0, 0)
+    V1_1_0 = (1, 1, 0)
+    V1_2_0 = (1, 2, 0)
+    V1_3_0 = (1, 3, 0)
+    V2_0_0 = (2, 0, 0)
+    V2_1_0 = (2, 1, 0)
+
+    @classmethod
+    def minor_versions(cls) -> Set[Tuple[int, int]]:
+        """@brief Returns a set of minor version tuples."""
+        return {
+            v[:2] for k, v in cls.__dict__.items()
+            if k.startswith('V')
+            }
+
 DAP_DEFAULT_PORT = 0
 DAP_SWD_PORT = 1
 DAP_JTAG_PORT = 2
@@ -80,6 +104,11 @@ DAP_JTAG_PORT = 2
 DAP_LED_CONNECT = 0
 DAP_LED_RUNNING = 1
 
+# Masks for DAP_SWD_SEQUENCE command.
+class DAPSWDSequence:
+    MODE_MASK = 0x80 # bit [7]: 0=output, 1=input
+    CYCLES_MASK = 0x1f # bits [5:0]: number of TCK cycles
+
 # Options for DAP_SWO_TRANSPORT command.
 class DAPSWOTransport:
     NONE = 0
@@ -125,6 +154,17 @@ class CMSISDAPProtocol(object):
         self.interface = interface
 
     def dap_info(self, id_):
+        """! @brief Sends the DAP_Info command to read info from the CMSIS-DAP probe.
+        @param self This object.
+        @param id_ One of the @ref pyocd.probe.pydapaccess.dap_access_api.DAPAcessIntf.ID "DAPAcessIntf.ID" constants.
+        @return The `id_` parameter determines the return value data type. For those IDs defined as integer values
+            in the DAP_Info command documentation, an int is returned. Otherwise either a string is returned, or
+            None if the returned info value length is 0.
+        @exception DAPAccessIntf.DeviceError Raised under these conditions:
+            - Response packet is for a different command.
+            - An int-type info was requested, but the returned value length is not 1, 2, or 4.
+            - A string-type info was requested, but the returned value length is greater than the response packet size minus response header and terminating null byte on the string.
+        """
         assert type(id_) is DAPAccessIntf.ID
             
         cmd = []
@@ -137,20 +177,25 @@ class CMSISDAPProtocol(object):
             # Response is to a different command
             raise DAPAccessIntf.DeviceError("expected DAP_INFO")
 
-        if resp[1] == 0:
-            return
+        resp_len = resp[1]
 
         # Integer values
         if id_ in INTEGER_INFOS:
-            if resp[1] == 1:
+            if resp_len == 1:
                 return resp[2]
-            if resp[1] == 2:
+            elif resp_len == 2:
                 return (resp[3] << 8) | resp[2]
-            if resp[1] == 4:
+            elif resp_len == 4:
                 return (resp[5] << 24) | (resp[4] << 16) | (resp[3] << 8) | resp[2]
+            else:
+                raise DAPAccessIntf.DeviceError("invalid DAP_INFO response length for %s" % id_.name)
 
         # String values. They are sent as C strings with a terminating null char, so we strip it out.
-        return bytearray(resp[2:2 + resp[1] - 1]).decode('ascii')
+        if resp_len == 0:
+            return None
+        if resp_len > (len(resp) - 2):
+            raise DAPAccessIntf.DeviceError("invalid DAP_INFO response length for %s" % id_.name)
+        return bytearray(resp[2:2 + resp_len - 1]).decode('utf-8', 'replace')
 
     def set_led(self, type, enabled):
         cmd = []
@@ -240,7 +285,7 @@ class CMSISDAPProtocol(object):
 
         return resp[1]
 
-    def transfer_configure(self, idle_cycles=0x00, wait_retry=0x0050, match_retry=0x0000):
+    def transfer_configure(self, idle_cycles=0x02, wait_retry=0x0050, match_retry=0x0000):
         cmd = []
         cmd.append(Command.DAP_TRANSFER_CONFIGURE)
         cmd.append(idle_cycles)
@@ -320,6 +365,71 @@ class CMSISDAPProtocol(object):
 
         return resp[1]
 
+    def swd_sequence(self, sequences):
+        """! @brief Send the DAP_SWD_Sequence command.
+        
+        Each sequence in the _sequences_ parameter is a tuple with 1 or 2 members:
+        - 0: int: number of TCK cycles from 1-64
+        - 1: int: the SWDIO bit values to transfer. The presence of this tuple member indicates the sequence is
+            an output sequence; the absence means that the specified number of TCK cycles of SWDIO data will be
+            read and returned.
+        
+        The DAP_SWD_Sequence command expects this data for each sequence:
+        - 0: sequence info byte
+            - bit [7]: mode, 0=output, 1=input
+            - bit [6]: reserved
+            - bits [5:0]: number of TCK cycles from 1-64, with 64 encoded as 0
+        - 1: (only present if element 0 bit 7 == 0, for output mode) bytes of data to send, one bit per TCK cycle,
+            transmitted LSB first.
+        
+        @param self
+        @param sequences A sequence of sequence description tuples as described above.
+        
+        @return A 2-tuple of the response status, and a sequence of bytes objects, one for each input
+            sequence. The length of the bytes object is (<TCK-count> + 7) / 8. Bits are in LSB first order.
+        """
+        cmd = [Command.DAP_SWD_SEQUENCE]
+        cmd.append(len(sequences))
+        for seq in sequences:
+            # Construct the control byte.
+            tck_count = seq[0]
+            assert 1 <= tck_count <= 64, "SWD sequence TCK count is out of range (must be 1-64)"
+            is_output = len(seq) == 2
+            info = (0x00 if is_output else 0x80) | (0 if (tck_count == 64) else tck_count)
+            cmd.append(info)
+            
+            # Append SWDIO output data.
+            if is_output:
+                bits = seq[1]
+                for i in range((tck_count + 7) // 8):
+                    cmd.append(bits & 0xff)
+                    bits >>= 8
+        self.interface.write(cmd)
+
+        resp = self.interface.read()
+        if resp[0] != Command.DAP_SWD_SEQUENCE:
+            # Response is to a different command
+            raise DAPAccessIntf.DeviceError("expected DAP_SWD_SEQUENCE")
+
+        if resp[1] != DAP_OK:
+            # DAP SWD Configure failed
+            raise DAPAccessIntf.CommandError("DAP_SWD_SEQUENCE failed")
+
+        result = []
+        offset = 2
+        for seq in sequences:
+            # Only handle input sequences. Convert the bytes into an int, LSB first.
+            if len(seq) == 1:
+                tck_count = seq[0]
+                byte_count = (tck_count + 7) // 8
+                bits = 0
+                for i in range(byte_count):
+                    bits |= resp[offset + i] << (8 * i)
+                offset += byte_count
+                result.append(bits)
+
+        return resp[1], result
+
     def swj_sequence(self, length, bits):
         assert 0 <= length <= 256
         cmd = []
