@@ -42,10 +42,16 @@ class DAPAccessIntf(object):
         VENDOR = 1
         PRODUCT = 2
         SER_NUM = 3
-        FW_VER = 4
+        CMSIS_DAP_PROTOCOL_VERSION = 4
         DEVICE_VENDOR = 5
         DEVICE_NAME = 6
+        BOARD_VENDOR = 7
+        BOARD_NAME = 8
+        PRODUCT_FW_VERSION = 9
         CAPABILITIES = 0xf0
+        TEST_DOMAIN_TIMER = 0xf1
+        UART_RECEIVE_BUFFER_SIZE = 0xfb
+        UART_TRANSMIT_BUFFER_SIZE = 0xfc
         SWO_BUFFER_SIZE = 0xfd
         MAX_PACKET_COUNT = 0xfe
         MAX_PACKET_SIZE = 0xff
@@ -93,6 +99,17 @@ class DAPAccessIntf(object):
         """! @brief Set arguments to configure behavior"""
         raise NotImplementedError()
 
+    @property
+    def protocol_version(self):
+        """! @brief CMSIS-DAP protocol version.
+        
+        The version is represented as 3-tuple with elements, in order, of major version,
+        minor version, and patch version.
+
+        Not valid (returns None) until the device is opened.
+        """
+        raise NotImplementedError()
+
     @property
     def vendor_name(self):
         raise NotImplementedError()
@@ -106,6 +123,14 @@ class DAPAccessIntf(object):
         """! @brief A tuple of USB VID and PID, in that order."""
         raise NotImplementedError()
 
+    @property
+    def has_swd_sequence(self):
+        """! @brief Boolean indicating whether the DAP_SWD_Sequence command is supported.
+        
+        This property is only valid after the probe is opened. Until then, the value will be None.
+        """
+        raise NotImplementedError()
+
     # ------------------------------------------- #
     #          Host control functions
     # ------------------------------------------- #
@@ -163,6 +188,24 @@ class DAPAccessIntf(object):
         """
         raise NotImplementedError()
 
+    def swd_sequence(self, sequences):
+        """! @brief Send a sequences of bits on the SWDIO signal.
+        
+        This method sends the DAP_SWD_Sequence CMSIS-DAP command.
+        
+        Each sequence in the _sequences_ parameter is a tuple with 1 or 2 members:
+        - 0: int: number of TCK cycles from 1-64
+        - 1: int: the SWDIO bit values to transfer. The presence of this tuple member indicates the sequence is
+            an output sequence; the absence means that the specified number of TCK cycles of SWDIO data will be
+            read and returned.
+        
+        @param self
+        @param sequences A sequence of sequence description tuples as described above.
+        
+        @return A 2-tuple of the response status, and a sequence of bytes objects, one for each input
+            sequence. The length of the bytes object is (<TCK-count> + 7) / 8. Bits are in LSB first order.
+        """
+
     def jtag_sequence(self, cycles, tms, read_tdo, tdi):
         """! @brief Send JTAG sequence.
         
