@@ -1,5 +1,7 @@
 # pyOCD debugger
-# Copyright (c) 2006-2013,2018-2020 Arm Limited
+# Copyright (c) 2006-2013,2018-2021 Arm Limited
+# Copyright (c) 2020 Koji Kitayama
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,13 +16,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-
 import re
 import logging
-import time
 import collections
-import six
 import threading
+from typing import (Optional, Tuple)
+
 from .dap_settings import DAPSettings
 from .dap_access_api import DAPAccessIntf
 from .cmsis_dap_core import CMSISDAPProtocol
@@ -32,12 +33,14 @@ from .cmsis_dap_core import (
     DAPSWOTransport,
     DAPSWOMode,
     DAPSWOControl,
-    DAPSWOStatus,
     DAPTransferResponse,
+    CMSISDAPVersion,
     )
 from ...core import session
 from ...utility.concurrency import locked
 
+VersionTuple = Tuple[int, int, int]
+
 # CMSIS-DAP values
 AP_ACC = 1 << 0
 DP_ACC = 0 << 0
@@ -92,9 +95,9 @@ class _Transfer(object):
                  transfer_request, transfer_data):
         # Writes should not need a transfer object
         # since they don't have any response data
-        assert isinstance(dap_index, six.integer_types)
-        assert isinstance(transfer_count, six.integer_types)
-        assert isinstance(transfer_request, six.integer_types)
+        assert isinstance(dap_index, int)
+        assert isinstance(transfer_count, int)
+        assert isinstance(transfer_request, int)
         assert transfer_request & READ
         self.daplink = daplink
         self.dap_index = dap_index
@@ -312,7 +315,7 @@ class _Command(object):
                     buf[pos] = (write_list[write_pos] >> (8 * 3)) & 0xff
                     pos += 1
                     write_pos += 1
-        return buf
+        return buf[:pos]
 
     def _check_response(self, response):
         """! @brief Check the response status byte from CMSIS-DAP transfer commands.
@@ -402,7 +405,7 @@ class _Command(object):
                     buf[pos] = (write_list[write_pos] >> (8 * 3)) & 0xff
                     pos += 1
                     write_pos += 1
-        return buf
+        return buf[:pos]
 
     def _decode_transfer_block_data(self, data):
         """! @brief Take a byte array and extract the data from it
@@ -453,6 +456,10 @@ class _Command(object):
 
 class DAPAccessCMSISDAP(DAPAccessIntf):
     """! @brief An implementation of the DAPAccessIntf layer for DAPLink boards
+    
+    @internal
+    All methods that use the CMSISDAPProtocol instance must be locked and must flush the command queue
+    prior to using methods of that object. Otherwise the command responses may be processed out of order.
     """
 
     # ------------------------------------------- #
@@ -474,7 +481,7 @@ class DAPAccessCMSISDAP(DAPAccessIntf):
 
     @staticmethod
     def get_device(device_id):
-        assert isinstance(device_id, six.string_types)
+        assert isinstance(device_id, str)
         iface = DAPAccessCMSISDAP._lookup_interface_for_unique_id(device_id)
         if iface is not None:
             return DAPAccessCMSISDAP(device_id, iface)
@@ -521,7 +528,7 @@ class DAPAccessCMSISDAP(DAPAccessIntf):
     #          CMSIS-DAP and Other Functions
     # ------------------------------------------- #
     def __init__(self, unique_id, interface=None):
-        assert isinstance(unique_id, six.string_types) or (unique_id is None and interface is not None)
+        assert isinstance(unique_id, str) or (unique_id is None and interface is not None)
         super(DAPAccessCMSISDAP, self).__init__()
 
         # Search for a matching interface if one wasn't provided.
@@ -555,6 +562,24 @@ class DAPAccessCMSISDAP(DAPAccessIntf):
         self._commands_to_read = None
         self._command_response_buf = None
         self._swo_status = None
+        self._cmsis_dap_version: VersionTuple = CMSISDAPVersion.V1_0_0
+        self._fw_version: Optional[str] = None
+
+    @property
+    def protocol_version(self) -> VersionTuple:
+        """! @brief Tuple of CMSIS-DAP protocol version.
+        @return 3-tuple consisting of (major, minor, micro) version of the CMSIS-DAP protocol implemented
+            by the debug probe.
+        """
+        return self._cmsis_dap_version
+    
+    @property
+    def firmware_version(self) -> Optional[str]:
+        """! @brief A string of the product firmware version, or None.
+        
+        Only probes supporting CMSIS-DAP protocol v2.1 or later can return their firmware version.
+        """
+        return self._fw_version
 
     @property
     def vendor_name(self):
@@ -568,6 +593,10 @@ class DAPAccessCMSISDAP(DAPAccessIntf):
     def vidpid(self):
         """! @brief A tuple of USB VID and PID, in that order."""
         return self._vidpid
+
+    @property
+    def has_swd_sequence(self):
+        return self._cmsis_dap_version >= CMSISDAPVersion.V1_2_0
     
     def lock(self):
         """! @brief Lock the interface."""
@@ -577,6 +606,50 @@ class DAPAccessCMSISDAP(DAPAccessIntf):
         """! @brief Unlock the interface."""
         self._lock.release()
 
+    def _read_protocol_version(self):
+        """! Determine the CMSIS-DAP protocol version."""
+        protocol_version_str = self._protocol.dap_info(self.ID.CMSIS_DAP_PROTOCOL_VERSION)
+        # Just in case we don't get a valid response, default to the lowest version (not including betas).
+        if not protocol_version_str:
+            self._cmsis_dap_version = CMSISDAPVersion.V1_0_0
+            return
+
+        # Convert the version to a 3-tuple for easy comparison.
+        # 1.2.3 will be converted to (1,2,3), 1.10 to (1,1,0), and so on.
+        #
+        # There are two version formats returned from the reference CMSIS-DAP code: 2-field and 3-field.
+        # The older versions return versions like "1.07" and "1.10", while recent versions return "1.2.0"
+        # or "2.0.0".
+        #
+        # Some CMSIS-DAP compatible debug probes from various vendors return the probe's firmware version
+        # rather than protocol version (like DAPLink versions 0254 and 0255 do) due to a misunderstanding
+        # based on unclear documentation. These cases are handled by the additional error checking below.
+        #
+        # Note that the exact version identified here is not that important, as it's not used much in
+        # this code (so far at least). There are also DAP_Info Capability bits for availability of certain
+        # commands that should be used instead of checking the version.
+        try:
+            fw_version = protocol_version_str.split('.')
+            major = int(fw_version[0])
+            # Handle version of the form "1.10" by treating the two digits after the dot as minor and patch.
+            if (len(fw_version) == 2) and len(fw_version[1]) == 2:
+                minor = int(fw_version[1][0])
+                patch = int(fw_version[1][1])
+            # All other forms.
+            else:
+                minor = int(fw_version[1] if len(fw_version) > 1 else 0)
+                patch = int(fw_version[2] if len(fw_version) > 2 else 0)
+            self._cmsis_dap_version = (major, minor, patch)
+        except ValueError:
+            # One of the protocol version fields had a non-numeric character, indicating it is not a valid
+            # CMSIS-DAP version number. Default to the lowest version.
+            self._cmsis_dap_version = CMSISDAPVersion.V1_0_0
+        
+        # Validate the version against known CMSIS-DAP minor versions. This will also catch the beta release
+        # versions of CMSIS-DAP, 0.01 and 0.02, and raise them to 1.0.0.
+        if self._cmsis_dap_version[:2] not in CMSISDAPVersion.minor_versions():
+            self._cmsis_dap_version = CMSISDAPVersion.V1_0_0
+
     @locked
     def open(self):
         if self._interface is None:
@@ -585,16 +658,27 @@ class DAPAccessCMSISDAP(DAPAccessIntf):
         self._interface.open()
         self._protocol = CMSISDAPProtocol(self._interface)
 
-        if DAPSettings.limit_packets:
+        if session.Session.get_current().options['cmsis_dap.limit_packets'] or DAPSettings.limit_packets:
             self._packet_count = 1
             LOG.debug("Limiting packet count to %d", self._packet_count)
         else:
             self._packet_count = self._protocol.dap_info(self.ID.MAX_PACKET_COUNT)
 
+        # Get the protocol version.
+        self._read_protocol_version()
+
+        # Read the firmware version if the protocol supports it.
+        # THe PRODUCT_FW_VERSION ID was added in versions 1.3.0 (HID) and 2.1.0 (bulk).
+        if (self._cmsis_dap_version >= CMSISDAPVersion.V2_1_0) or (self._cmsis_dap_version >= CMSISDAPVersion.V1_3_0
+                and self._cmsis_dap_version < CMSISDAPVersion.V2_0_0):
+            self._fw_version = self._protocol.dap_info(self.ID.PRODUCT_FW_VERSION)
+        
         # Log probe's firmware version.
-        fw_version = self._protocol.dap_info(self.ID.FW_VER)
-        if fw_version:
-            LOG.debug("CMSIS-DAP probe %s firmware version: %s", self._unique_id, fw_version)
+        if self._fw_version:
+            LOG.debug("CMSIS-DAP probe %s: firmware version %s, protocol version %i.%i.%i",
+                    self._unique_id, self._fw_version, *self._cmsis_dap_version)
+        else:
+            LOG.debug("CMSIS-DAP probe %s: protocol version %i.%i.%i", self._unique_id, *self._cmsis_dap_version)
 
         self._interface.set_packet_count(self._packet_count)
         self._packet_size = self._protocol.dap_info(self.ID.MAX_PACKET_SIZE)
@@ -713,10 +797,15 @@ class DAPAccessCMSISDAP(DAPAccessIntf):
         self.flush()
         self._protocol.swj_sequence(length, bits)
 
+    @locked
+    def swd_sequence(self, sequences):
+        self.flush()
+        self._protocol.swd_sequence(sequences)
+
     @locked
     def jtag_sequence(self, cycles, tms, read_tdo, tdi):
         self.flush()
-        return self._protocol.jtag_sequence(cycles, length, read_tdo, tdi)
+        return self._protocol.jtag_sequence(cycles, tms, read_tdo, tdi)
 
     @locked
     def disconnect(self):
@@ -728,6 +817,8 @@ class DAPAccessCMSISDAP(DAPAccessIntf):
     
     @locked
     def swo_configure(self, enabled, rate):
+        self.flush()
+
         # Don't send any commands if the SWO commands aren't supported.
         if not self._has_swo_uart:
             return False
@@ -761,6 +852,7 @@ class DAPAccessCMSISDAP(DAPAccessIntf):
             self._swo_disable()
             return False
     
+    # Doesn't need @locked because it is only called from swo_configure().
     def _swo_disable(self):
         try:
             self._protocol.swo_mode(DAPSWOMode.OFF)
@@ -772,6 +864,8 @@ class DAPAccessCMSISDAP(DAPAccessIntf):
     
     @locked
     def swo_control(self, start):
+        self.flush()
+
         # Don't send any commands if the SWO commands aren't supported.
         if not self._has_swo_uart:
             return False
@@ -792,20 +886,24 @@ class DAPAccessCMSISDAP(DAPAccessIntf):
     def get_swo_status(self):
         return self._protocol.swo_status()
     
-    @locked
     def swo_read(self, count=None):
+        # The separate SWO EP can be read without locking.
         if self._interface.has_swo_ep:
             return self._interface.read_swo()
         else:
             if count is None:
                 count = self._packet_size
-            status, count, data = self._protocol.swo_data(count)
-            return bytearray(data)
+            # Must lock and flush the command queue since we're using the SWO read command that shares
+            # the command EP.
+            with self._lock:
+                self.flush()
+                status, count, data = self._protocol.swo_data(count)
+                return bytearray(data)
 
     def write_reg(self, reg_id, value, dap_index=0):
         assert reg_id in self.REG
-        assert isinstance(value, six.integer_types)
-        assert isinstance(dap_index, six.integer_types)
+        assert isinstance(value, int)
+        assert isinstance(dap_index, int)
 
         request = WRITE
         if reg_id.value < 4:
@@ -817,7 +915,7 @@ class DAPAccessCMSISDAP(DAPAccessIntf):
 
     def read_reg(self, reg_id, dap_index=0, now=True):
         assert reg_id in self.REG
-        assert isinstance(dap_index, six.integer_types)
+        assert isinstance(dap_index, int)
         assert isinstance(now, bool)
 
         request = READ
@@ -841,10 +939,10 @@ class DAPAccessCMSISDAP(DAPAccessIntf):
             return read_reg_cb
 
     def reg_write_repeat(self, num_repeats, reg_id, data_array, dap_index=0):
-        assert isinstance(num_repeats, six.integer_types)
+        assert isinstance(num_repeats, int)
         assert num_repeats == len(data_array)
         assert reg_id in self.REG
-        assert isinstance(dap_index, six.integer_types)
+        assert isinstance(dap_index, int)
 
         request = WRITE
         if reg_id.value < 4:
@@ -856,9 +954,9 @@ class DAPAccessCMSISDAP(DAPAccessIntf):
 
     def reg_read_repeat(self, num_repeats, reg_id, dap_index=0,
                         now=True):
-        assert isinstance(num_repeats, six.integer_types)
+        assert isinstance(num_repeats, int)
         assert reg_id in self.REG
-        assert isinstance(dap_index, six.integer_types)
+        assert isinstance(dap_index, int)
         assert isinstance(now, bool)
 
         request = READ
@@ -976,8 +1074,8 @@ class DAPAccessCMSISDAP(DAPAccessIntf):
         """! @brief Write one or more commands
         """
         assert dap_index == 0  # dap index currently unsupported
-        assert isinstance(transfer_count, six.integer_types)
-        assert isinstance(transfer_request, six.integer_types)
+        assert isinstance(transfer_count, int)
+        assert isinstance(transfer_request, int)
         assert transfer_data is None or len(transfer_data) > 0
 
         # Create transfer and add to transfer list
@@ -989,7 +1087,6 @@ class DAPAccessCMSISDAP(DAPAccessIntf):
 
         # Build physical packet by adding it to command
         cmd = self._crnt_cmd
-        is_read = transfer_request & READ
         size_to_transfer = transfer_count
         trans_data_pos = 0
         while size_to_transfer > 0:
