@@ -1,5 +1,8 @@
 # pyOCD debugger
-# Copyright (c) 2006-2020 Arm Limited
+# Copyright (c) 2006-2021 Arm Limited
+# Copyright (c) 2020 Patrick Huesmann
+# Copyright (c) 2021 mentha
+# Copyright (c) Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,7 +19,6 @@
 
 import logging
 import threading
-import six
 from time import sleep
 import platform
 import errno
@@ -26,19 +28,16 @@ from .common import (
     USB_CLASS_HID,
     filter_device_by_class,
     is_known_cmsis_dap_vid_pid,
-    check_ep,
+    generate_device_unique_id,
     )
 from ..dap_access_api import DAPAccessIntf
-from ... import common
 
 LOG = logging.getLogger(__name__)
 
 try:
     import usb.core
     import usb.util
-except:
-    if platform.system() == "Linux":
-        LOG.error("PyUSB is required for CMSIS-DAP support on Linux")
+except ImportError:
     IS_AVAILABLE = False
 else:
     IS_AVAILABLE = True
@@ -105,13 +104,12 @@ class PyUSB(Interface):
         except (NotImplementedError, usb.core.USBError) as e:
             # Some implementations don't don't have kernel attach/detach
             LOG.warning("USB Kernel Driver Detach Failed ([%s] %s). Attached driver may interfere with pyOCD operations.", e.errno, e.strerror)
-            pass
 
         # Explicitly claim the interface
         try:
             usb.util.claim_interface(dev, interface_number)
         except usb.core.USBError as exc:
-            raise six.raise_from(DAPAccessIntf.DeviceError("Unable to open device"), exc)
+            raise DAPAccessIntf.DeviceError("Unable to open device") from exc
 
         # Update all class variables if we made it here
         self.ep_out = ep_out
@@ -168,9 +166,10 @@ class PyUSB(Interface):
             new_board = PyUSB()
             new_board.vid = board.idVendor
             new_board.pid = board.idProduct
-            new_board.product_name = board.product
-            new_board.vendor_name = board.manufacturer
-            new_board.serial_number = board.serial_number
+            new_board.product_name = board.product or f"{board.idProduct:#06x}"
+            new_board.vendor_name = board.manufacturer or f"{board.idVendor:#06x}"
+            new_board.serial_number = board.serial_number \
+                    or generate_device_unique_id(board.idProduct, board.idVendor, board.bus, board.address)
             boards.append(new_board)
 
         return boards
@@ -271,14 +270,25 @@ class MatchCmsisDapv1Interface(object):
             # Must have either 1 or 2 endpoints.
             if interface.bNumEndpoints not in (1, 2):
                 return False
-        
-            # Endpoint 0 must be interrupt in.
-            if not check_ep(interface, 0, usb.util.ENDPOINT_IN, usb.util.ENDPOINT_TYPE_INTR):
-                return False
-        
-            # Endpoint 1 is optional. If present it must be interrupt out.
-            if (interface.bNumEndpoints == 2) \
-                and not check_ep(interface, 1, usb.util.ENDPOINT_OUT, usb.util.ENDPOINT_TYPE_INTR):
+
+            endpoint_attrs = [
+                (usb.util.endpoint_direction(ep.bEndpointAddress),
+                 usb.util.endpoint_type(ep.bmAttributes))
+                 for ep in interface
+            ]
+
+            # Possible combinations of endpoints
+            ENDPOINT_ATTRS_ALLOWED = [
+                # One interrupt endpoint IN
+                [(usb.util.ENDPOINT_IN, usb.util.ENDPOINT_TYPE_INTR)],
+                # Two interrupt endpoints, first one IN, second one OUT
+                [(usb.util.ENDPOINT_IN, usb.util.ENDPOINT_TYPE_INTR),
+                 (usb.util.ENDPOINT_OUT, usb.util.ENDPOINT_TYPE_INTR)],
+                # Two interrupt endpoints, first one OUT, second one IN
+                [(usb.util.ENDPOINT_OUT, usb.util.ENDPOINT_TYPE_INTR),
+                 (usb.util.ENDPOINT_IN, usb.util.ENDPOINT_TYPE_INTR)],
+            ]
+            if endpoint_attrs not in ENDPOINT_ATTRS_ALLOWED:
                 return False
         
             # All checks passed, this is a CMSIS-DAPv2 interface!
@@ -325,7 +335,7 @@ class FindDap(object):
             if error.errno == errno.EACCES and platform.system() == "Linux":
                 msg = ("%s while trying to interrogate a USB device "
                    "(VID=%04x PID=%04x). This can probably be remedied with a udev rule. "
-                   "See <https://github.com/mbedmicro/pyOCD/tree/master/udev> for help." %
+                   "See <https://github.com/pyocd/pyOCD/tree/master/udev> for help." %
                    (error, dev.idVendor, dev.idProduct))
                 # If we recognize this device as one that should be CMSIS-DAP, we can raise
                 # the level of the log message since it's almost certainly a permissions issue.
@@ -344,6 +354,8 @@ class FindDap(object):
         if cmsis_dap_interface is None:
             return False
         if self._serial is not None:
+            if self._serial == "" and dev.serial_number is None:
+                return True
             if self._serial != dev.serial_number:
                 return False
         return True
