@@ -1,5 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2006-2020 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,13 +16,14 @@
 # limitations under the License.
 
 import logging
-import platform
 import collections
 from time import sleep
-import six
 
 from .interface import Interface
-from .common import filter_device_by_usage_page
+from .common import (
+    filter_device_by_usage_page,
+    generate_device_unique_id,
+    )
 from ..dap_access_api import DAPAccessIntf
 from ....utility.timeout import Timeout
 
@@ -31,9 +33,7 @@ LOG = logging.getLogger(__name__)
 
 try:
     import pywinusb.hid as hid
-except:
-    if platform.system() == "Windows":
-        LOG.error("PyWinUSB is required on a Windows Machine")
+except ImportError:
     IS_AVAILABLE = False
 else:
     IS_AVAILABLE = True
@@ -84,14 +84,13 @@ class PyWinUSB(Interface):
                     # If the device could not be opened in read only mode
                     # Then it either has been disconnected or is in use
                     # by another thread/process
-                    raise six.raise_from(DAPAccessIntf.DeviceError("Unable to open device %s"
-                            % self.serial_number), exc)
+                    raise DAPAccessIntf.DeviceError(f"Unable to open device {self.serial_number}") from exc
 
             else:
                 # If this timeout has elapsed then another process
                 # has locked this device in shared mode. This should
                 # not happen.
-                raise DAPAccessIntf.DeviceError("timed out attempting to open device %s" % self.serial_number)
+                raise DAPAccessIntf.DeviceError(f"Timed out attempting to open device {self.serial_number}")
 
     @staticmethod
     def get_all_connected_interfaces():
@@ -102,7 +101,7 @@ class PyWinUSB(Interface):
         # find devices with good vid/pid
         all_mbed_devices = []
         for d in all_devices:
-            if (d.product_name.find("CMSIS-DAP") >= 0):
+            if ("CMSIS-DAP" in d.product_name):
                 all_mbed_devices.append(d)
 
         boards = []
@@ -122,17 +121,18 @@ class PyWinUSB(Interface):
                 new_board = PyWinUSB()
                 new_board.report = report[0]
                 new_board.packet_size = len(new_board.report.get_raw_data()) - 1
-                new_board.vendor_name = dev.vendor_name
-                new_board.product_name = dev.product_name
-                new_board.serial_number = dev.serial_number
+                new_board.vendor_name = dev.vendor_name or f"{dev.vendor_id:#06x}"
+                new_board.product_name = dev.product_name or f"{dev.product_id:#06x}"
+                new_board.serial_number = dev.serial_number \
+                        or generate_device_unique_id(dev.vendor_id, dev.product_id, dev.device_path)
                 new_board.vid = dev.vendor_id
                 new_board.pid = dev.product_id
                 new_board.device = dev
-                dev.close()
                 boards.append(new_board)
             except Exception as e:
                 if (str(e) != "Failure to get HID pre parsed data"):
                     LOG.error("Receiving Exception: %s", e)
+            finally:
                 dev.close()
 
         return boards
