@@ -17,7 +17,7 @@
 import platform
 
 # Make sure that any global generic setup is run
-from . import base  # noqa: F401
+from . import base  # noqa: F401 # lgtm[py/unused-import]
 
 def create_mbed_detector(**kwargs):
     """! Factory used to create host OS specific mbed-lstools object
