@@ -1,5 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2018-2019 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,27 +15,30 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import absolute_import
-from ...core import exceptions
-from .. import common
 import usb.core
 import usb.util
 import logging
-import six
-import threading
-from collections import namedtuple
+from typing import NamedTuple
 import platform
 import errno
 from binascii import hexlify
 
+from ...core import exceptions
+from .. import common
+
 LOG = logging.getLogger(__name__)
 
 TRACE = LOG.getChild("trace")
 TRACE.setLevel(logging.CRITICAL)
 
-STLinkInfo = namedtuple('STLinkInfo', 'version_name out_ep in_ep swv_ep')
+class STLinkInfo(NamedTuple):
+    """@brief STLink USB interface numbers and version name."""
+    version_name: str
+    out_ep: int
+    in_ep: int
+    swv_ep: int
 
-class STLinkUSBInterface(object):
+class STLinkUSBInterface:
     """!@brief Provides low-level USB enumeration and transfers for STLinkV2/3 devices."""
 
     ## Command packet size.
@@ -44,15 +48,22 @@ class STLinkUSBInterface(object):
     USB_VID = 0x0483
 
     ## Map of USB PID to firmware version name and device endpoints.
+    #
+    # Other PIDs:
+    # - 0x3744: STLink V1
+    # - 0x374d: STLink V3 DFU
     USB_PID_EP_MAP = {
         # PID              Version  OUT     IN      SWV
         0x3748: STLinkInfo('V2',    0x02,   0x81,   0x83),
-        0x374b: STLinkInfo('V2-1',  0x01,   0x81,   0x82),
         0x374a: STLinkInfo('V2-1',  0x01,   0x81,   0x82),  # Audio
-        0x3742: STLinkInfo('V2-1',  0x01,   0x81,   0x82),  # No MSD
+        0x374b: STLinkInfo('V2-1',  0x01,   0x81,   0x82),
         0x374e: STLinkInfo('V3',    0x01,   0x81,   0x82),
         0x374f: STLinkInfo('V3',    0x01,   0x81,   0x82),  # Bridge
-        0x3753: STLinkInfo('V3',    0x01,   0x81,   0x82),  # 2VCP
+        0x3752: STLinkInfo('V2-1',  0x01,   0x81,   0x82),  # No MSD
+        0x3753: STLinkInfo('V3',    0x01,   0x81,   0x82),  # 2VCP, No MSD
+        0x3754: STLinkInfo('V3',    0x01,   0x81,   0x82),  # No MSD
+        0x3755: STLinkInfo('V3',    0x01,   0x81,   0x82),
+        0x3757: STLinkInfo('V3',    0x01,   0x81,   0x82),
         }
     
     ## STLink devices only have one USB interface.
@@ -80,7 +91,7 @@ class STLinkUSBInterface(object):
                 # a permissions issue.
                 LOG.warning("%s while trying to get the STLink USB device configuration "
                    "(VID=%04x PID=%04x). This can probably be remedied with a udev rule. "
-                   "See <https://github.com/mbedmicro/pyOCD/tree/master/udev> for help.",
+                   "See <https://github.com/pyocd/pyOCD/tree/master/udev> for help.",
                    error, dev.idVendor, dev.idProduct)
             return False
         except (IndexError, NotImplementedError, ValueError) as error:
@@ -231,7 +242,7 @@ class STLinkUSBInterface(object):
                 TRACE.debug("  USB IN < %s" % ' '.join(['%02x' % i for i in data]))
                 return data
         except usb.core.USBError as exc:
-            six.raise_from(exceptions.ProbeError("USB Error: %s" % exc), exc)
+            raise exceptions.ProbeError("USB Error: %s" % exc) from exc
         return None
 
     def read_swv(self, size, timeout=1000):
