@@ -1,5 +1,6 @@
 # pyOCD debugger
-# Copyright (c) 2020 Arm Limited
+# Copyright (c) 2020-2021 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,12 +18,10 @@
 import logging
 import threading
 import json
-import base64
 import socket
-from six.moves.socketserver import (ThreadingTCPServer, StreamRequestHandler)
+from socketserver import (ThreadingTCPServer, StreamRequestHandler)
 
 from .shared_probe_proxy import SharedDebugProbeProxy
-from ..core.session import Session
 from ..core import exceptions
 from .debug_probe import DebugProbe
 from ..coresight.ap import (APVersion, APv1Address, APv2Address)
@@ -39,7 +38,7 @@ class DebugProbeServer(threading.Thread):
     can be terminated by calling the stop() method, which will also kill the server thread.
     """
     
-    def __init__(self, session, probe, port=None, serve_local_only=True):
+    def __init__(self, session, probe, port=None, serve_local_only=None):
         """! @brief Constructor.
         
         @param self The object.
@@ -52,8 +51,8 @@ class DebugProbeServer(threading.Thread):
             @ref pyocd.probe.shared_probe_proxy.SharedDebugProbeProxy "SharedDebugProbeProxy"
             then a new proxy is created to allow the probe to be shared by multiple connections.
         @param port The TCP port number. Defaults to the 'probeserver.port' option if not provided.
-        @param serve_local_only Boolean. Whether to restrict the server to be accessible only from
-            localhost.
+        @param serve_local_only Optional Boolean. Whether to restrict the server to be accessible only from
+            localhost. If not specified (set to None), then the 'serve_local_only' session option is used.
         """
         super(DebugProbeServer, self).__init__()
         
@@ -78,6 +77,10 @@ class DebugProbeServer(threading.Thread):
         else:
             self._port = port
         
+        # Default to the serve_local_only session option.
+        if serve_local_only is None:
+            serve_local_only = session.options.get('serve_local_only')
+        
         host = 'localhost' if serve_local_only else ''
         address = (host, self._port)
         
@@ -223,6 +226,8 @@ class DebugProbeRequestHandler(StreamRequestHandler):
                 'connect':              (self._request__connect,            1   ), # 'connect', protocol:str
                 'disconnect':           (self._probe.disconnect,            0   ), # 'disconnect'
                 'swj_sequence':         (self._probe.swj_sequence,          2   ), # 'swj_sequence', length:int, bits:int
+                'swd_sequence':         (self._probe.swd_sequence,          1   ), # 'swd_sequence', sequences:List[Union[Tuple[int], Tuple[int, int]]] -> Tuple[int, List[bytes]]
+                'jtag_sequence':        (self._probe.jtag_sequence,         4   ), # 'jtag_sequence', cycles:int, tms:int, read_tdo:bool, tdi:int -> Union[None, int]
                 'set_clock':            (self._probe.set_clock,             1   ), # 'set_clock', freq:int
                 'reset':                (self._probe.reset,                 0   ), # 'reset'
                 'assert_reset':         (self._probe.assert_reset,          1   ), # 'assert_reset', asserted:bool
@@ -285,7 +290,6 @@ class DebugProbeRequestHandler(StreamRequestHandler):
             try:
                 request = None
                 request_dict = None
-                request_id = -1
                 self._current_request_id = -1
                 
                 # Read request line.
@@ -308,7 +312,6 @@ class DebugProbeRequestHandler(StreamRequestHandler):
                 if 'id' not in request_dict:
                     self._send_error_response(message="missing request ID")
                     continue
-                request_id = request_dict['id']
                 self._current_request_id = request_dict['id']
                 
                 if 'request' not in request_dict:
@@ -457,6 +460,6 @@ class DebugProbeRequestHandler(StreamRequestHandler):
     _PROPERTY_CONVERTERS = {
             'capabilities':                 lambda value: [v.name for v in value],
             'supported_wire_protocols':     lambda value: [v.name for v in value],
-            'wire_protocol':                lambda value: value.name,
+            'wire_protocol':                lambda value: value.name if (value is not None) else None,
         }
 
