@@ -1,5 +1,7 @@
 # pyOCD debugger
+# Copyright (c) 2020 NXP
 # Copyright (c) 2006-2018 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,14 +16,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from ...coresight import (dap, ap)
+import logging
+from time import sleep
+
+from ...coresight import ap
 from ...coresight.cortex_m import CortexM
 from ...core import exceptions
 from ...core.target import Target
 from ...coresight.coresight_target import CoreSightTarget
 from ...utility.timeout import Timeout
-import logging
-from time import sleep
 
 MDM_STATUS = 0x00000000
 MDM_CTRL = 0x00000004
@@ -117,8 +120,10 @@ class Kinetis(CoreSightTarget):
             canAccess = False
         else:
             try:
-                for attempt in range(ACCESS_TEST_ATTEMPTS):
-                    self.aps[0].read32(CortexM.DHCSR)
+                # Ensure to use AP#0 as a MEM_AP
+                if isinstance(self.aps[0], ap.MEM_AP):
+                    for attempt in range(ACCESS_TEST_ATTEMPTS):
+                        self.aps[0].read32(CortexM.DHCSR)
             except exceptions.TransferError:
                 LOG.debug("Access test failed with fault")
                 canAccess = False
@@ -157,9 +162,9 @@ class Kinetis(CoreSightTarget):
 
                 # Assert that halt on connect was forced above. Reset will stay asserted
                 # until halt on connect is executed.
-#                 assert self._force_halt_on_connect
+                # assert self._force_halt_on_connect
 
-                isLocked = False
+#                 isLocked = False
             else:
                 LOG.warning("%s in secure state: not automatically unlocking", self.part_number)
         else:
