@@ -1,6 +1,9 @@
+# -*- coding: utf-8 -*-
 # pyOCD debugger
-# Copyright (c) 2019 Arm Limited
+# Copyright (c) 2019-2020 Arm Limited
 # Copyright (c) 2020 Men Shiyun
+# Copyright (c) 2020 Fraunhofer-Gesellschaft zur Förderung der angewandten Forschung e.V.
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,22 +18,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import print_function
-from xml.etree.ElementTree import ElementTree
+from xml.etree.ElementTree import (ElementTree, Element)
 import zipfile
-from collections import namedtuple
 import logging
 import io
-import itertools
-import six
-import struct
+from typing import (Any, Dict, Iterator, Optional)
 
 from .flash_algo import PackFlashAlgo
-from ... import core
 from ...core import exceptions
 from ...core.target import Target
-from ...core.memory_map import (MemoryMap, MemoryType, MEMORY_TYPE_CLASS_MAP, FlashRegion)
-from ...utility.compatibility import BadZipFile
+from ...core.memory_map import (MemoryMap, MemoryRegion, MemoryType, MEMORY_TYPE_CLASS_MAP, FlashRegion, RamRegion)
 
 LOG = logging.getLogger(__name__)
 
@@ -47,6 +44,16 @@ class _DeviceInfo(object):
         self.algos = kwargs.get('algos', [])
         self.debugs = kwargs.get('debugs', [])
 
+def _get_part_number_from_element(element: Element) -> str:
+    """! @brief Extract the part number from a device or variant XML element."""
+    assert element.tag in ("device", "variant")
+    if element.tag == "device":
+        return element.attrib['Dname']
+    elif element.tag == "variant":
+        return element.attrib['Dvariant']
+    else:
+        raise ValueError("element is neither device nor variant")
+
 class CmsisPack(object):
     """! @brief Wraps a CMSIS Device Family Pack.
     
@@ -82,9 +89,8 @@ class CmsisPack(object):
         else:
             try:
                 self._pack_file = zipfile.ZipFile(file_or_path, 'r')
-            except BadZipFile as err:
-                six.raise_from(MalformedCmsisPackError("Failed to open CMSIS-Pack '{}': {}".format(
-                    file_or_path, err)), err)
+            except zipfile.BadZipFile as err:
+                raise MalformedCmsisPackError(f"Failed to open CMSIS-Pack '{file_or_path}': {err}") from err
         
         # Find the .pdsc file.
         for name in self._pack_file.namelist():
@@ -92,23 +98,66 @@ class CmsisPack(object):
                 self._pdscName = name
                 break
         else:
-            raise MalformedCmsisPackError("CMSIS-Pack '{}' is missing a .pdsc file".format(file_or_path))
+            raise MalformedCmsisPackError(f"CMSIS-Pack '{file_or_path}' is missing a .pdsc file")
+        
+        with self._pack_file.open(self._pdscName) as pdscFile:
+            self._pdsc = CmsisPackDescription(self, pdscFile)
+    
+    @property
+    def filename(self):
+        """! @brief Accessor for the filename or path of the .pack file."""
+        return self._pack_file.filename
+    
+    @property
+    def pdsc(self):
+        """! @brief Accessor for the CmsisPackDescription instance for the pack's PDSC file."""
+        return self._pdsc
+    
+    @property
+    def devices(self):
+        """! @brief A list of CmsisPackDevice objects for every part number defined in the pack."""
+        return self._pdsc.devices
+    
+    def get_file(self, filename):
+        """! @brief Return file-like object for a file within the pack.
+        
+        @param self
+        @param filename Relative path within the pack. May use forward or back slashes.
+        @return A BytesIO object is returned that contains all of the data from the file
+            in the pack. This is done to isolate the returned file from how the pack was
+            opened (due to particularities of the ZipFile implementation).
+        """
+        filename = filename.replace('\\', '/')
+        return io.BytesIO(self._pack_file.read(filename))
+
+class CmsisPackDescription(object):
+    def __init__(self, pack, pdsc_file):
+        """! @brief Constructor.
+        
+        @param self This object.
+        @param pack Reference to the CmsisPack instance.
+        @param pdsc_file A file-like object for the .pdsc contained in _pack_.
+        """
+        self._pack = pack
         
         # Convert PDSC into an ElementTree.
-        with self._pack_file.open(self._pdscName) as pdscFile:
-            self._pdsc = ElementTree(file=pdscFile)
+        self._pdsc = ElementTree(file=pdsc_file)
 
         self._state_stack = []
         self._devices = []
         
+        # Remember if we have already warned about overlapping memory regions
+        # so we can limit these to one warning per DFP
+        self._warned_overlapping_memory_regions = False
+        
         # Extract devices.
         for family in self._pdsc.iter('family'):
             self._parse_devices(family)
     
     @property
-    def pdsc(self):
-        """! @brief Accessor for the ElementTree instance for the pack's PDSC file."""
-        return self._pdsc
+    def pack(self):
+        """! @brief Reference to the containing CmsisPack object."""
+        return self._pack
     
     @property
     def devices(self):
@@ -143,7 +192,7 @@ class CmsisPack(object):
                                         debugs=self._extract_debugs()
                                         )
             
-            dev = CmsisPackDevice(self, deviceInfo)
+            dev = CmsisPackDevice(self.pack, deviceInfo)
             self._devices.append(dev)
 
         # Recursively process subelements.
@@ -173,27 +222,56 @@ class CmsisPack(object):
         return list(map.values())
 
     def _extract_memories(self):
+        def get_start_and_size(elem):
+            try:
+                start = int(elem.attrib['start'], base=0)
+                size = int(elem.attrib['size'], base=0)
+            except (KeyError, ValueError):
+                LOG.warning("memory region missing address")
+                raise
+            return (start, size)
         def filter(map, elem):
-            # 'name' takes precedence over 'id'.
-            if 'name' in elem.attrib:
+            # Inner memory regions are allowed to override outer memory
+            # regions. If this is not done properly via name/id, we must make
+            # sure not to report overlapping memory regions to gdb since it
+            # will ignore those completely, see:
+            # https://github.com/pyocd/pyOCD/issues/980
+            start, size = get_start_and_size(elem)
+            if 'name' in elem.attrib: # 'name' takes precedence over 'id'.
                 name = elem.attrib['name']
             elif 'id' in elem.attrib:
                 name = elem.attrib['id']
             else:
                 # Neither option for memory name was specified, so use the address range.
-                try:
-                    start = int(elem.attrib['start'], base=0)
-                    size = int(elem.attrib['size'], base=0)
-                except (KeyError, ValueError):
-                    LOG.warning("memory region missing address")
-                    return
-                
                 # Use the start and size for a name.
                 name = "%08x:%08x" % (start, size)
 
             pname = elem.attrib.get('Pname', None)
             info = (name, pname)
         
+            if info in map:
+                del map[info]
+            for k in list(map.keys()):
+                prev_pname = k[1]
+                # Previously, we would not check for overlaps if the pname was different. But because pyocd
+                # currently only supports one memory map for the whole device, we have to ignore the pname for
+                # now.
+                prev_elem = map[k]
+                prev_start, prev_size = get_start_and_size(prev_elem)
+                # Overlap: start or end between previous start and previous end
+                end = start + size - 1
+                prev_end = prev_start + prev_size - 1
+                if (prev_start <= start < prev_end) or (prev_start <= end < prev_end):
+                    # Only report warnings for overlapping regions from the same processor. Allow regions for different
+                    # processors to override each other, since we don't yet support maps for each processor.
+                    if (pname == prev_pname) and not self._warned_overlapping_memory_regions:
+                        filename = self.pack.filename if self.pack else "unknown"
+                        LOG.warning("Overlapping memory regions in file %s (%s); deleting outer region. "
+                                    "Further warnings will be suppressed for this file.",
+                                    filename, _get_part_number_from_element(self._state_stack[-1].element))
+                        self._warned_overlapping_memory_regions = True
+                    del map[k]
+
             map[info] = elem
         
         return self._extract_items('memories', filter)
@@ -232,18 +310,6 @@ class CmsisPack(object):
                 map['*'] = elem
         
         return self._extract_items('debugs', filter)
-    
-    def get_file(self, filename):
-        """! @brief Return file-like object for a file within the pack.
-        
-        @param self
-        @param filename Relative path within the pack. May use forward or back slashes.
-        @return A BytesIO object is returned that contains all of the data from the file
-            in the pack. This is done to isolate the returned file from how the pack was
-            opened (due to particularities of the ZipFile implementation).
-        """
-        filename = filename.replace('\\', '/')
-        return io.BytesIO(self._pack_file.read(filename))
 
 def _get_bool_attribute(elem, name, default=False):
     """! @brief Extract an XML attribute with a boolean value.
@@ -285,12 +351,7 @@ class CmsisPackDevice(object):
         """
         self._pack = pack
         self._info = device_info
-        
-        if device_info.element.tag == "device":
-            self._part = device_info.element.attrib['Dname']
-        elif device_info.element.tag == "variant":
-            self._part = device_info.element.attrib['Dvariant']
-        
+        self._part = _get_part_number_from_element(device_info.element)
         self._regions = []
         self._saw_startup = False
         self._default_ram = None
@@ -360,6 +421,13 @@ class CmsisPackDevice(object):
                 LOG.debug("ignoring error parsing memories for CMSIS-Pack devices %s: %s",
                     self.part_number, str(err))
     
+    def _get_containing_region(self, addr: int) -> Optional[MemoryRegion]:
+        """@brief Return the memory region containing the given address."""
+        for region in self._regions:
+            if region.contains_address(addr):
+                return region
+        return None
+
     def _build_flash_regions(self):
         """! @brief Converts ROM memory regions to flash regions.
         
@@ -372,7 +440,10 @@ class CmsisPackDevice(object):
         if self._default_ram is None:
             LOG.warning("CMSIS-Pack device %s has no default RAM defined, cannot program flash" % self.part_number)
             return
-        
+
+        # Can't import at top level due to import loops.
+        from ...core.session import Session
+
         regions_to_delete = [] # List of regions to delete.
         regions_to_add = [] # List of FlashRegion objects to add.
 
@@ -383,20 +454,23 @@ class CmsisPackDevice(object):
                 continue
             
             # Look for matching flash algo.
-            algo = self._find_matching_algo(region)
-            if algo is None:
+            algo_element = self._find_matching_algo(region)
+            if algo_element is None:
                 # Must be a mask ROM or non-programmable flash.
                 continue
 
+            # Load flash algo from .FLM file.
+            packAlgo = self._load_flash_algo(algo_element.attrib['name'])
+            if packAlgo is None:
+                LOG.warning("Failed to convert ROM region to flash region because flash algorithm '%s' could not be "
+                            " found (%s)", algo_element.attrib['name'], self.part_number)
+                continue
+            
             # The ROM region will be replaced with one or more flash regions.
             regions_to_delete.append(region)
 
-            # Load flash algo from .FLM file.
-            algoData = self.pack.get_file(algo.attrib['name'])
-            packAlgo = PackFlashAlgo(algoData)
-            
             # Log details of this flash algo if the debug option is enabled.
-            current_session = core.session.Session.get_current()
+            current_session = Session.get_current()
             if current_session and current_session.options.get("debug.log_flm_info"):
                 LOG.debug("Flash algo info: %s", packAlgo.flash_info)
             
@@ -406,60 +480,38 @@ class CmsisPackDevice(object):
             if page_size <= 32:
                 page_size = min(s[1] for s in packAlgo.sector_sizes)
             
+            # Select the RAM to use for the algo. 
+            try:
+                # See if an explicit RAM range was specified for the algo.
+                ram_start = int(algo_element.attrib['RAMstart'], base=0)
+
+                # The region size comes either from the RAMsize attribute, the containing region's bounds, or
+                # a large, arbitrary value.
+                if 'RAMsize' in algo_element.attrib:
+                    ram_size = int(algo_element.attrib['RAMsize'], base=0)
+                else:
+                    containing_region = self._get_containing_region(ram_start)
+                    if containing_region is not None:
+                        ram_size = containing_region.length - (ram_start - containing_region.start)
+                    else:
+                        # No size specified, and the RAMstart attribute is outside of a known region,
+                        # so just use a relatively large arbitrary size. Because the algo is packed at the
+                        # start of the provided region, this won't be a problem unless the DFP is
+                        # actually erroneous.
+                        ram_size = 128 * 1024
+                    
+                ram_for_algo = RamRegion(start=ram_start, length=ram_size)
+            except KeyError:
+                # No RAM addresses were given, so go with the RAM marked default.
+                ram_for_algo = self._default_ram
+
             # Construct the pyOCD algo using the largest sector size. We can share the same
             # algo for all sector sizes.
-            algo = packAlgo.get_pyocd_flash_algo(page_size, self._default_ram)
+            algo = packAlgo.get_pyocd_flash_algo(page_size, ram_for_algo)
 
-            # Create a separate flash region for each sector size range. The sector_sizes attribute
-            # is a list of bi-tuples of (start-address, sector-size), sorted by start address.
-            for j, sectorInfo in enumerate(packAlgo.sector_sizes):
-                # Unpack this sector range's start address and sector size.
-                offset, sector_size = sectorInfo
-                start = region.start + offset
-                
-                # Determine the end address of the this sector range. For the last range, the end
-                # is just the end of the entire region. Otherwise it's the start of the next
-                # range - 1.
-                if j + 1 >= len(packAlgo.sector_sizes):
-                    end = region.end
-                else:
-                    end = region.start + packAlgo.sector_sizes[j + 1][0] - 1
-                
-                # Skip wrong start and end addresses
-                if end < start:
-                    continue
-                
-                # Limit page size.
-                if page_size > sector_size:
-                    region_page_size = sector_size
-                    LOG.warning("Page size (%d) is larger than sector size (%d) for flash region %s; "
-                                "reducing page size to %d", page_size, sector_size, region.name,
-                                region_page_size)
-                else:
-                    region_page_size = page_size
-                
-                # If we don't have a boot memory yet, pick the first flash.
-                if not self._saw_startup:
-                    isBoot = True
-                    self._saw_startup = True
-                else:
-                    isBoot = region.is_boot_memory
-                
-                # Construct the flash region.
-                rangeRegion = FlashRegion(name=region.name,
-                                access=region.access,
-                                start=start,
-                                end=end,
-                                sector_size=sector_size,
-                                page_size=region_page_size,
-                                flm=packAlgo,
-                                algo=algo,
-                                erased_byte_value=packAlgo.flash_info.value_empty,
-                                is_default=region.is_default,
-                                is_boot_memory=isBoot,
-                                is_testable=region.is_testable,
-                                alias=region.alias)
-                regions_to_add.append(rangeRegion)
+            # Create a separate flash region for each sector size range.
+            regions_to_add += list(self._split_flash_region_by_sector_size(
+                                            region, page_size, algo, packAlgo)) # type: ignore
         
         # Now update the regions list.
         for region in regions_to_delete:
@@ -467,6 +519,64 @@ class CmsisPackDevice(object):
         for region in regions_to_add:
             self._regions.append(region)
     
+    def _split_flash_region_by_sector_size(self,
+            region: MemoryRegion,
+            page_size: int,
+            algo: Dict[str, Any],
+            pack_algo: PackFlashAlgo) -> Iterator[FlashRegion]:
+        """@brief Yield separate flash regions for each sector size range."""
+        # The sector_sizes attribute is a list of bi-tuples of (start-address, sector-size), sorted by start address.
+        for j, (offset, sector_size) in enumerate(pack_algo.sector_sizes):
+            start = region.start + offset
+            
+            # Determine the end address of the this sector range. For the last range, the end
+            # is just the end of the entire region. Otherwise it's the start of the next
+            # range - 1.
+            if j + 1 >= len(pack_algo.sector_sizes):
+                end = region.end
+            else:
+                end = region.start + pack_algo.sector_sizes[j + 1][0] - 1
+            
+            # Skip wrong start and end addresses
+            if end < start:
+                continue
+            
+            # Limit page size.
+            if page_size > sector_size:
+                region_page_size = sector_size
+                LOG.warning("Page size (%d) is larger than sector size (%d) for flash region %s; "
+                            "reducing page size to %d", page_size, sector_size, region.name,
+                            region_page_size)
+            else:
+                region_page_size = page_size
+            
+            # If we don't have a boot memory yet, pick the first flash.
+            if not self._saw_startup:
+                is_boot = True
+                self._saw_startup = True
+            else:
+                is_boot = region.is_boot_memory
+            
+            # Construct region name. If there is more than one sector size, we need to make the region's name unique.
+            region_name = region.name
+            if len(pack_algo.sector_sizes) > 1:
+                region_name += f"_{sector_size:#x}"
+            
+            # Construct the flash region.
+            yield FlashRegion(name=region_name,
+                            access=region.access,
+                            start=start,
+                            end=end,
+                            sector_size=sector_size,
+                            page_size=region_page_size,
+                            flm=pack_algo,
+                            algo=algo,
+                            erased_byte_value=pack_algo.flash_info.value_empty,
+                            is_default=region.is_default,
+                            is_boot_memory=is_boot,
+                            is_testable=region.is_testable,
+                            alias=region.alias)
+
     def _find_matching_algo(self, region):
         """! @brief Searches for a flash algo covering the regions's address range.'"""
         for algo in self._info.algos:
@@ -479,6 +589,17 @@ class CmsisPackDevice(object):
             if (algoStart <= region.start <= algoEnd) and (algoStart <= region.end <= algoEnd):
                 return algo
         return None
+    
+    def _load_flash_algo(self, filename: str) -> Optional[PackFlashAlgo]:
+        """! @brief Return the PackFlashAlgo instance for the given flash algo filename."""
+        if self.pack is not None:
+            try:
+                algo_data = self.pack.get_file(filename)
+                return PackFlashAlgo(algo_data)
+            except FileNotFoundError:
+                pass
+        # Return default value.
+        return None
 
     @property
     def pack(self):
