@@ -1,5 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2015-2020 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,7 +17,7 @@
 
 import logging
 import shlex
-import six
+from typing import (Any, Dict, Iterable, List, Union)
 
 from ..core.target import Target
 from ..core.options import OPTIONS_INFO
@@ -24,10 +25,10 @@ from ..utility.compatibility import to_str_safe
 
 LOG = logging.getLogger(__name__)
 
-def split_command_line(cmd_line):
+def split_command_line(cmd_line: Union[str, List[str]]) -> List[str]:
     """! @brief Split command line by whitespace, supporting quoted strings."""
     result = []
-    if isinstance(cmd_line, six.string_types):
+    if isinstance(cmd_line, str):
         args = [cmd_line]
     else:
         args = cmd_line
@@ -50,13 +51,13 @@ VECTOR_CATCH_CHAR_MAP = {
         'n': Target.VectorCatch.NONE,
     }
 
-def convert_vector_catch(value):
+def convert_vector_catch(vcvalue: Union[str, bytes]) -> int:
     """! @brief Convert a vector catch string to a mask.
     
     @exception ValueError Raised if an invalid vector catch character is encountered.
     """
     # Make case insensitive.
-    value = to_str_safe(value).lower()
+    value: str = to_str_safe(vcvalue).lower()
 
     # Handle special vector catch options.
     if value == 'all':
@@ -71,7 +72,7 @@ def convert_vector_catch(value):
         # Reraise an error with a more helpful message.
         raise ValueError("invalid vector catch option '{}'".format(e.args[0]))
 
-def convert_session_options(option_list):
+def convert_session_options(option_list: Iterable[str]) -> Dict[str, Any]:
     """! @brief Convert a list of session option settings to a dictionary."""
     options = {}
     if option_list is not None:
@@ -107,13 +108,23 @@ def convert_session_options(option_list):
                     continue
             # Convert string value to option type.
             elif info.type is bool:
-                value = value in ("true", "1", "yes", "on")
+                if value.lower() in ("true", "1", "yes", "on", "false", "0", "no", "off"):
+                    value = value.lower() in ("true", "1", "yes", "on")
+                else:
+                    LOG.warning("invalid value for option '%s'", name)
+                    continue
             elif info.type is int:
                 try:
                     value = int(value, base=0)
                 except ValueError:
                     LOG.warning("invalid value for option '%s'", name)
                     continue
+            elif info.type is float:
+                try:
+                    value = float(value)
+                except ValueError:
+                    LOG.warning("invalid value for option '%s'", name)
+                    continue
             
             options[name] = value
     return options
@@ -133,7 +144,7 @@ RESET_TYPE_MAP = {
         'emulated': Target.ResetType.SW_EMULATED,
     }
 
-def convert_reset_type(value):
+def convert_reset_type(value: str) -> Target.ResetType:
     """! @brief Convert a reset_type session option value to the Target.ResetType enum.
     @param value The value of the reset_type session option.
     @exception ValueError Raised if an unknown reset_type value is passed.
@@ -143,7 +154,7 @@ def convert_reset_type(value):
         raise ValueError("unexpected value for reset_type option ('%s')" % value)
     return RESET_TYPE_MAP[value]
 
-def convert_frequency(value):
+def convert_frequency(value: str) -> int:
     """! @brief Applies scale suffix to frequency value string.
     @param value String with a float and possible 'k' or 'm' suffix (case-insensitive). "Hz" may
         also follow. No space is allowed between the float and suffix. Leading and trailing
@@ -155,55 +166,22 @@ def convert_frequency(value):
         value = value[:-2]
     suffix = value[-1]
     if suffix in ('k', 'm'):
-        value = float(value[:-1])
+        fvalue = float(value[:-1])
         if suffix == 'k':
-            value *= 1000
+            fvalue *= 1000
         elif suffix == 'm':
-            value *= 1000000
-        return int(value)
+            fvalue *= 1000000
+        return int(fvalue)
     else:
         return int(float(value))
 
-class UniquePrefixMatcher(object):
-    """! @brief Manages detection of shortest unique prefix match of a set of strings."""
-    
-    def __init__(self, items=None):
-        """! @brief Constructor.
-        @param self This object.
-        @param items Optional sequence of strings.
-        """
-        self._items = set(items) if (items is not None) else set()
-    
-    def add_items(self, items):
-        """! @brief Add some items to be matched.
-        @param self This object.
-        @param items Sequence of strings.
-        """
-        self._items.update(items)
-    
-    def find_all(self, prefix):
-        """! @brief Return all items matching the given prefix.
-        @param self This object.
-        @param prefix String that is compared as a prefix against the items passed to the constructor.
-            Must not be the empty string.
-        @return List of all items where `prefix` is a valid prefix.
-        @exception ValueError Raised for an empty `prefix`.
-        """
-        if len(prefix) == 0:
-            raise ValueError("empty prefix")
-        # First look for an exact match.
-        if prefix in self._items:
-            return (prefix,)
-        return tuple(i for i in self._items if i.startswith(prefix))
-    
-    def find_one(self, prefix):
-        """! @brief Return the item matching the given prefix, or None.
-        @param self This object.
-        @param prefix String that is compared as a prefix against the items passed to the constructor.
-        @return The full value of the matching item where `prefix` is a valid prefix.
-        @exception ValueError Raised for an empty `prefix`.
-        """
-        all_matches = self.find_all(prefix)
-        if len(all_matches) == 1:
-            return all_matches[0]
-        return None
+
+def int_base_0(x: str) -> int:
+    """! @brief Converts a string to an int with support for base prefixes."""
+    return int(x, base=0)
+
+
+def flatten_args(args: Iterable[Iterable[Any]]) -> List[Any]:
+    """! @brief Converts a list of lists to a single list."""
+    return [item for sublist in args for item in sublist]
+
