@@ -71,6 +71,11 @@ def bfx(value, msb, lsb):
     mask = bitmask((msb, lsb))
     return (value & mask) >> lsb
 
+def bfxw(value, lsb, width):
+    """! @brief Extract a value from a bitfield given the LSb and width."""
+    mask = bitmask((lsb + width, lsb))
+    return (value & mask) >> lsb
+
 def bfi(value, msb, lsb, field):
     """! @brief Change a bitfield value."""
     mask = bitmask((msb, lsb))
@@ -78,6 +83,39 @@ def bfi(value, msb, lsb, field):
     value |= (field << lsb) & mask
     return value
 
+class Bitfield(object):
+    """! @brief Represents a bitfield of a register."""
+
+    def __init__(self, msb, lsb=None, name=None):
+        self._msb = msb
+        self._lsb = lsb if (lsb is not None) else msb
+        self._name = name
+        assert self._msb >= self._lsb
+    
+    @property
+    def width(self):
+        return self._msb - self._lsb + 1
+    
+    def get(self, value):
+        """! @brief Extract the bitfield value from a register value.
+        @param self The Bitfield object.
+        @param value Integer register value.
+        @return Integer value of the bitfield extracted from `value`.
+        """
+        return bfx(value, self._msb, self._lsb)
+    
+    def set(self, register_value, field_value):
+        """! @brief Modified the bitfield in a register value.
+        @param self The Bitfield object.
+        @param register_value Integer register value.
+        @param field_value New value for the bitfield. Must not be shifted into place already.
+        @return Integer register value with the bitfield updated to `field_value`.
+        """
+        return bfi(register_value, self._msb, self._lsb, field_value)
+    
+    def __repr__(self):
+        return "<{}@{:x} name={} {}:{}>".format(self.__class__.__name__, id(self), self._name, self._msb, self._lsb)
+
 def msb(n):
     """! @brief Return the bit number of the highest set bit."""
     ndx = 0
@@ -112,3 +150,18 @@ def round_up_div(value, divisor):
     """! @brief Return value divided by the divisor, rounding up to the nearest multiple of the divisor."""
     return (value + divisor - 1) // divisor
 
+def parity32_high(n):
+    """! @brief Compute parity over a 32-bit value.
+
+    This function is intended to be used for computing parity over a 32-bit value transferred in an Arm
+    ADI AP/DP register transfer. The result is returned in bit 32, ready to be OR'd into the register
+    value to form the 33-bit data + parity for the AP/DP register transfer.
+
+    @param n 32-bit integer.
+    @return Integer with 1-bit parity placed at bit 32. The lower 32 bits are 0.
+    """
+    n ^= n >> 16
+    n ^= n >> 8
+    n ^= n >> 4
+    n &= 0xf
+    return (0xD32C0000 << n) & (1 << 32)
