@@ -16,13 +16,7 @@
 
 import logging
 from collections import OrderedDict
-
-# Collection ABCs accessible directly from collections are deprecated and will be removed in
-# Python 3.9.
-try:
-    from collections.abc import Callable
-except ImportError:
-    from collections import Callable
+from collections.abc import Callable
 
 LOG = logging.getLogger(__name__)
 
@@ -85,6 +79,12 @@ class CallSequence(object):
         """! @brief Remove all tasks from the sequence."""
         self._calls = OrderedDict()
     
+    def copy(self):
+        """! @brief Duplicate the sequence."""
+        new_seq = CallSequence()
+        new_seq._calls = self._calls.copy()
+        return new_seq
+    
     def remove_task(self, name):
         """! @brief Remove a task with the given name.
         @exception KeyError Raised if no task with the specified name exists.
