@@ -1,5 +1,6 @@
 # pyOCD debugger
 # Copyright (c) 2015-2020 Arm Limited
+# Copyright (c) 2021 Chris Reed
 # SPDX-License-Identifier: Apache-2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -26,13 +27,12 @@ import tempfile
 import threading
 from pyocd.utility.compatibility import to_str_safe
 
-isPy2 = (sys.version_info[0] == 2)
-
 OBJCOPY = "arm-none-eabi-objcopy"
 
 TEST_DIR = os.path.dirname(os.path.abspath(__file__))
 PYOCD_DIR = os.path.dirname(TEST_DIR)
 TEST_DATA_DIR = os.path.join(TEST_DIR, "data")
+TEST_OUTPUT_DIR = os.path.join(TEST_DIR, "output")
 
 def get_test_binary_path(binary_name):
     return os.path.join(TEST_DATA_DIR, "binaries", binary_name)
@@ -44,6 +44,12 @@ def get_env_file_name():
     env_name = get_env_name()
     return ("_" + env_name) if env_name else ''
 
+def ensure_output_dir():
+    if not os.path.isdir(TEST_OUTPUT_DIR):
+        if os.path.exists(TEST_OUTPUT_DIR):
+            raise RuntimeError("path '%s' already exists but is not a directory" % TEST_OUTPUT_DIR)
+        os.mkdir(TEST_OUTPUT_DIR)
+
 # Returns common option values passed in when creating test sessions.
 def get_session_options():
     return {
@@ -131,6 +137,21 @@ def run_in_parallel(function, args_list):
         if result is not True:
             raise Exception("Running in thread failed")
 
+def wait_with_deadline(process, timeout):
+    try:
+        from subprocess import TimeoutExpired
+        try:
+            process.wait(timeout=timeout)
+        except TimeoutExpired as e:
+            print('Timeout while waiting for process %s to exit: %s' % (process, e))
+            process.kill()
+            return False
+    except ImportError:
+        # Python 2.7 doesn't support deadline for wait.
+        # Let's wait without deadline, as Python 2.7 support is close to end anyway.
+        process.wait()
+    return True
+
 class IOTee(object):
     def __init__(self, *args):
         self.outputs = list(args)
@@ -139,8 +160,6 @@ class IOTee(object):
         self.outputs.append(output)
 
     def write(self, message):
-        if isPy2 and isinstance(message, str):
-            message = message.decode('UTF-8')
         for out in self.outputs:
             out.write(message)
 
@@ -156,8 +175,6 @@ class RecordingLogHandler(logging.Handler):
     def emit(self, record):
         try:
             message = self.format(record)
-            if isPy2 and isinstance(message, unicode):
-                message = message.encode('UTF-8')
             self.stream.write(six.u(message + "\n"))
         except:
             self.handleError(record)
@@ -219,6 +236,7 @@ class Test(object):
     def __init__(self, name, function):
         self.name = name
         self.test_function = function
+        self.n = 0
 
     def run(self, board):
         """
